@@ -42,6 +42,7 @@ tst_KLocalSocketServer::~tst_KLocalSocketServer()
 
 class TimedConnection: public QThread
 {
+    Q_OBJECT
 public:
     ~TimedConnection()
     {
@@ -244,6 +245,9 @@ void tst_KLocalSocketServer::setMaxPendingConnections()
 
 void tst_KLocalSocketServer::abstractUnixSocket_data()
 {
+#ifndef Q_OS_LINUX
+    QSKIP("Abstract UNIX sockets are specific for Linux");
+#endif
     QTest::addColumn<QString>("path");
     QTest::addColumn<bool>("success");
 
@@ -304,3 +308,4 @@ void tst_KLocalSocketServer::abstractUnixSocket()
 
 QTEST_MAIN(tst_KLocalSocketServer)
 
+#include "klocalsocketservertest.moc"
