@@ -22,9 +22,14 @@
 #include <QTest>
 #include <QVector>
 #include <QDataStream>
+#include <QTemporaryFile>
+#include <qplatformdefs.h>
 
+#include <kfileitem.h>
 #include <udsentry.h>
 
+#include "kiotesthelper.h"
+
 struct UDSTestField {
     UDSTestField() {}
 
@@ -58,9 +63,9 @@ void UDSEntryTest::testSaveLoad()
     // Data for 1st UDSEntry.
     testCase
             << UDSTestField(KIO::UDSEntry::UDS_SIZE, 1)
-            << UDSTestField(KIO::UDSEntry::UDS_USER, "user1")
-            << UDSTestField(KIO::UDSEntry::UDS_GROUP, "group1")
-            << UDSTestField(KIO::UDSEntry::UDS_NAME, QLatin1String("filename1"))
+            << UDSTestField(KIO::UDSEntry::UDS_USER, QStringLiteral("user1"))
+            << UDSTestField(KIO::UDSEntry::UDS_GROUP, QStringLiteral("group1"))
+            << UDSTestField(KIO::UDSEntry::UDS_NAME, QStringLiteral("filename1"))
             << UDSTestField(KIO::UDSEntry::UDS_MODIFICATION_TIME, 123456)
             << UDSTestField(KIO::UDSEntry::UDS_CREATION_TIME, 12345)
             << UDSTestField(KIO::UDSEntry::UDS_DEVICE_ID, 2)
@@ -71,9 +76,9 @@ void UDSEntryTest::testSaveLoad()
     testCase.clear();
     testCase
             << UDSTestField(KIO::UDSEntry::UDS_SIZE, 2)
-            << UDSTestField(KIO::UDSEntry::UDS_USER, "user2")
-            << UDSTestField(KIO::UDSEntry::UDS_GROUP, "group1")
-            << UDSTestField(KIO::UDSEntry::UDS_NAME, QLatin1String("filename2"))
+            << UDSTestField(KIO::UDSEntry::UDS_USER, QStringLiteral("user2"))
+            << UDSTestField(KIO::UDSEntry::UDS_GROUP, QStringLiteral("group1"))
+            << UDSTestField(KIO::UDSEntry::UDS_NAME, QStringLiteral("filename2"))
             << UDSTestField(KIO::UDSEntry::UDS_MODIFICATION_TIME, 12345)
             << UDSTestField(KIO::UDSEntry::UDS_CREATION_TIME, 1234)
             << UDSTestField(KIO::UDSEntry::UDS_DEVICE_ID, 87)
@@ -84,9 +89,9 @@ void UDSEntryTest::testSaveLoad()
     testCase.clear();
     testCase
             << UDSTestField(KIO::UDSEntry::UDS_SIZE, 2)
-            << UDSTestField(KIO::UDSEntry::UDS_GROUP, "group1")
-            << UDSTestField(KIO::UDSEntry::UDS_USER, "user2")
-            << UDSTestField(KIO::UDSEntry::UDS_NAME, QLatin1String("filename2"))
+            << UDSTestField(KIO::UDSEntry::UDS_GROUP, QStringLiteral("group1"))
+            << UDSTestField(KIO::UDSEntry::UDS_USER, QStringLiteral("user2"))
+            << UDSTestField(KIO::UDSEntry::UDS_NAME, QStringLiteral("filename2"))
             << UDSTestField(KIO::UDSEntry::UDS_MODIFICATION_TIME, 12345)
             << UDSTestField(KIO::UDSEntry::UDS_DEVICE_ID, 87)
             << UDSTestField(KIO::UDSEntry::UDS_INODE, 42)
@@ -97,38 +102,38 @@ void UDSEntryTest::testSaveLoad()
     testCase.clear();
     testCase
             << UDSTestField(KIO::UDSEntry::UDS_SIZE, 2)
-            << UDSTestField(KIO::UDSEntry::UDS_USER, "user4")
-            << UDSTestField(KIO::UDSEntry::UDS_GROUP, "group4")
+            << UDSTestField(KIO::UDSEntry::UDS_USER, QStringLiteral("user4"))
+            << UDSTestField(KIO::UDSEntry::UDS_GROUP, QStringLiteral("group4"))
             << UDSTestField(KIO::UDSEntry::UDS_MODIFICATION_TIME, 12346)
             << UDSTestField(KIO::UDSEntry::UDS_DEVICE_ID, 87)
             << UDSTestField(KIO::UDSEntry::UDS_INODE, 42)
             << UDSTestField(KIO::UDSEntry::UDS_CREATION_TIME, 1235)
-            << UDSTestField(KIO::UDSEntry::UDS_NAME, QLatin1String("filename4"));
+            << UDSTestField(KIO::UDSEntry::UDS_NAME, QStringLiteral("filename4"));
     testCases << testCase;
 
     // 5th entry: remove one field.
     testCase.clear();
     testCase
             << UDSTestField(KIO::UDSEntry::UDS_SIZE, 2)
-            << UDSTestField(KIO::UDSEntry::UDS_USER, "user4")
-            << UDSTestField(KIO::UDSEntry::UDS_GROUP, "group4")
+            << UDSTestField(KIO::UDSEntry::UDS_USER, QStringLiteral("user4"))
+            << UDSTestField(KIO::UDSEntry::UDS_GROUP, QStringLiteral("group4"))
             << UDSTestField(KIO::UDSEntry::UDS_MODIFICATION_TIME, 12346)
             << UDSTestField(KIO::UDSEntry::UDS_INODE, 42)
             << UDSTestField(KIO::UDSEntry::UDS_CREATION_TIME, 1235)
-            << UDSTestField(KIO::UDSEntry::UDS_NAME, QLatin1String("filename4"));
+            << UDSTestField(KIO::UDSEntry::UDS_NAME, QStringLiteral("filename4"));
     testCases << testCase;
 
     // 6th entry: add a new field, and change some others.
     testCase.clear();
     testCase
             << UDSTestField(KIO::UDSEntry::UDS_SIZE, 89)
-            << UDSTestField(KIO::UDSEntry::UDS_ICON_NAME, "icon6")
-            << UDSTestField(KIO::UDSEntry::UDS_USER, "user6")
-            << UDSTestField(KIO::UDSEntry::UDS_GROUP, "group4")
+            << UDSTestField(KIO::UDSEntry::UDS_ICON_NAME, QStringLiteral("icon6"))
+            << UDSTestField(KIO::UDSEntry::UDS_USER, QStringLiteral("user6"))
+            << UDSTestField(KIO::UDSEntry::UDS_GROUP, QStringLiteral("group4"))
             << UDSTestField(KIO::UDSEntry::UDS_MODIFICATION_TIME, 12346)
             << UDSTestField(KIO::UDSEntry::UDS_INODE, 32)
             << UDSTestField(KIO::UDSEntry::UDS_CREATION_TIME, 1235)
-            << UDSTestField(KIO::UDSEntry::UDS_NAME, QLatin1String("filename6"));
+            << UDSTestField(KIO::UDSEntry::UDS_NAME, QStringLiteral("filename6"));
     testCases << testCase;
 
     // Store the entries in a QByteArray.
@@ -221,4 +226,51 @@ void UDSEntryTest::testSaveLoad()
     }
 }
 
+/**
+ * Test to verify that move semantics work. This is only useful when ran through callgrind.
+ */
+void UDSEntryTest::testMove()
+{
+    // Create a temporary file. Just to make a UDSEntry further down.
+    QTemporaryFile file;
+    QVERIFY(file.open());
+    const QByteArray filePath = file.fileName().toLocal8Bit();
+    const QString fileName = QUrl(file.fileName()).fileName(); // QTemporaryFile::fileName returns the full path.
+    QVERIFY(!fileName.isEmpty());
+
+    // We have a file now. Get the stat data from it to make the UDSEntry.
+    QT_STATBUF statBuf;
+    QVERIFY(QT_LSTAT(filePath.constData(), &statBuf) == 0);
+    KIO::UDSEntry entry(statBuf, fileName);
+
+    // Verify that the name in the UDSEntry is the same as we've got from the fileName var.
+    QCOMPARE(fileName, entry.stringValue(KIO::UDSEntry::UDS_NAME));
+
+    // That was the boilerplate code. Now for move semantics.
+    // First: move assignment.
+    {
+        // First a copy as we need to keep the entry for the next test.
+        KIO::UDSEntry entryCopy = entry;
+
+        // Now move-assignement (two lines to prevent compiler optimization)
+        KIO::UDSEntry movedEntry;
+        movedEntry = std::move(entryCopy);
+
+        // And veryfy that this works.
+        QCOMPARE(fileName, movedEntry.stringValue(KIO::UDSEntry::UDS_NAME));
+    }
+
+    // Move constructor
+    {
+        // First a copy again
+        KIO::UDSEntry entryCopy = entry;
+
+        // Now move-assignement
+        KIO::UDSEntry movedEntry(std::move(entryCopy));
+
+        // And veryfy that this works.
+        QCOMPARE(fileName, movedEntry.stringValue(KIO::UDSEntry::UDS_NAME));
+    }
+}
+
 QTEST_MAIN(UDSEntryTest)
