@@ -30,6 +30,7 @@
 #include "statjob.h"
 #include "deletejob.h"
 #include "filecopyjob.h"
+#include "../pathhelpers_p.h"
 
 #include <kconfiggroup.h>
 #include <klocalizedstring.h>
@@ -111,13 +112,8 @@ enum CopyJobState {
 
 static QUrl addPathToUrl(const QUrl &url, const QString &relPath)
 {
-    QString path = url.path();
-    if (!path.endsWith('/')) {
-        path += '/';
-    }
-    path += relPath;
     QUrl u(url);
-    u.setPath(path);
+    u.setPath(concatPaths(url.path(), relPath));
     return u;
 }
 
@@ -278,6 +274,22 @@ public:
             job->d_func()->m_bOverwriteAllDirs = true;
             job->d_func()->m_bOverwriteAllFiles = true;
         }
+        if (!(flags & KIO::NoPrivilegeExecution)) {
+            job->d_func()->m_privilegeExecutionEnabled = true;
+            FileOperationType copyType;
+            switch (mode) {
+            case CopyJob::Copy:
+                copyType = Copy;
+                break;
+            case CopyJob::Move:
+                copyType = Move;
+                break;
+            case CopyJob::Link:
+                copyType = Symlink;
+                break;
+            }
+            job->d_func()->m_operationType = copyType;
+        }
         return job;
     }
 };
@@ -923,6 +935,7 @@ void CopyJobPrivate::startRenameJob(const QUrl &slave_url)
 
     KIO_ARGS << m_currentSrcURL << dest << (qint8) false /*no overwrite*/;
     SimpleJob *newJob = SimpleJobPrivate::newJobNoUi(slave_url, CMD_RENAME, packedArgs);
+    newJob->setParentJob(q);
     Scheduler::setJobPriority(newJob, 1);
     q->addSubjob(newJob);
     if (m_currentSrcURL.adjusted(QUrl::RemoveFilename) != dest.adjusted(QUrl::RemoveFilename)) { // For the user, moving isn't renaming. Only renaming is.
@@ -1068,7 +1081,7 @@ void CopyJobPrivate::slotResultCreatingDirs(KJob *job)
                         const QUrl destDirectory = (*it).uDest.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash);
                         const QString newName = KIO::suggestName(destDirectory, (*it).uDest.fileName());
                         QUrl newUrl(destDirectory);
-                        newUrl.setPath(newUrl.path() + QLatin1Char('/') + newName);
+                        newUrl.setPath(concatPaths(newUrl.path(), newName));
                         renameDirectory(it, newUrl);
                     } else {
                         if (!q->uiDelegateExtension()) {
@@ -1216,8 +1229,7 @@ void CopyJobPrivate::createNextDir()
         while (it != dirs.end() && udir.isEmpty()) {
             const QString dir = (*it).uDest.path();
             if (shouldSkip(dir)) {
-                dirs.erase(it);
-                it = dirs.begin();
+                it = dirs.erase(it);
             } else {
                 udir = (*it).uDest;
             }
@@ -1227,6 +1239,7 @@ void CopyJobPrivate::createNextDir()
         // Create the directory - with default permissions so that we can put files into it
         // TODO : change permissions once all is finished; but for stuff coming from CDROM it sucks...
         KIO::SimpleJob *newjob = KIO::mkdir(udir, -1);
+        newjob->setParentJob(q);
         Scheduler::setJobPriority(newjob, 1);
         if (shouldOverwriteFile(udir.path())) { // if we are overwriting an existing file or symlink
             newjob->addMetaData(QStringLiteral("overwrite"), QStringLiteral("true"));
@@ -1277,7 +1290,7 @@ void CopyJobPrivate::slotResultCopyingFiles(KJob *job)
                     QUrl destDirectory = (*it).uDest.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash);
                     const QString newName = KIO::suggestName(destDirectory, (*it).uDest.fileName());
                     QUrl newDest(destDirectory);
-                    newDest.setPath(newDest.path() + QLatin1Char('/') + newName);
+                    newDest.setPath(concatPaths(newDest.path(), newName));
                     emit q->renamed(q, (*it).uDest, newDest); // for e.g. kpropsdlg
                     (*it).uDest = newDest;
 
@@ -1329,6 +1342,7 @@ void CopyJobPrivate::slotResultCopyingFiles(KJob *job)
             // The only problem with this trick is that the error handling for this del operation
             // is not going to be right... see 'Very special case' above.
             KIO::Job *newjob = KIO::del((*it).uSource, HideProgressInfo);
+            newjob->setParentJob(q);
             q->addSubjob(newjob);
             return; // Don't move to next file yet !
         }
@@ -1505,6 +1519,7 @@ KIO::Job *CopyJobPrivate::linkNextFile(const QUrl &uSource, const QUrl &uDest, J
         (uSource.password() == uDest.password())) {
         // This is the case of creating a real symlink
         KIO::SimpleJob *newJob = KIO::symlink(uSource.path(), uDest, flags | HideProgressInfo /*no GUI*/);
+        newJob->setParentJob(q_func());
         Scheduler::setJobPriority(newJob, 1);
         qCDebug(KIO_COPYJOB_DEBUG) << "Linking target=" << uSource.path() << "link=" << uDest;
         //emit linking( this, uSource.path(), uDest );
@@ -1541,6 +1556,10 @@ KIO::Job *CopyJobPrivate::linkNextFile(const QUrl &uSource, const QUrl &uDest, J
                     config.writeEntry("Icon", QStringLiteral("text-x-texinfo"));
                 } else if (protocol == QLatin1String("mailto")) { // sven:
                     config.writeEntry("Icon", QStringLiteral("internet-mail"));    // added mailto: support
+                } else if (protocol == QLatin1String("trash") && url.path().length() <= 1) { // trash:/ link
+                    config.writeEntry("Name", i18n("Trash"));
+                    config.writeEntry("Icon", QStringLiteral("user-trash-full"));
+                    config.writeEntry("EmptyIcon", QStringLiteral("user-trash"));
                 } else {
                     config.writeEntry("Icon", QStringLiteral("unknown"));
                 }
@@ -1579,8 +1598,7 @@ void CopyJobPrivate::copyNextFile()
         const QString destFile = (*it).uDest.path();
         bCopyFile = !shouldSkip(destFile);
         if (!bCopyFile) {
-            files.erase(it);
-            it = files.begin();
+            it = files.erase(it);
         }
     }
 
@@ -1633,6 +1651,7 @@ void CopyJobPrivate::copyNextFile()
             // Copying a symlink - only on the same protocol/host/etc. (#5601, downloading an FTP file through its link),
         {
             KIO::SimpleJob *newJob = KIO::symlink((*it).linkDest, uDest, flags | HideProgressInfo /*no GUI*/);
+            newJob->setParentJob(q);
             Scheduler::setJobPriority(newJob, 1);
             newjob = newJob;
             qCDebug(KIO_COPYJOB_DEBUG) << "Linking target=" << (*it).linkDest << "link=" << uDest;
@@ -1645,6 +1664,7 @@ void CopyJobPrivate::copyNextFile()
             // NOTE: if we are moving stuff, the deletion of the source will be done in slotResultCopyingFiles
         } else if (m_mode == CopyJob::Move) { // Moving a file
             KIO::FileCopyJob *moveJob = KIO::file_move(uSource, uDest, permissions, flags | HideProgressInfo/*no GUI*/);
+            moveJob->setParentJob(q);
             moveJob->setSourceSize((*it).size);
             moveJob->setModificationTime((*it).mtime); // #55804
             newjob = moveJob;
@@ -1686,6 +1706,7 @@ void CopyJobPrivate::deleteNextDir()
         // Take first dir to delete out of list - last ones first !
         QList<QUrl>::Iterator it = --dirsToRemove.end();
         SimpleJob *job = KIO::rmdir(*it);
+        job->setParentJob(q);
         Scheduler::setJobPriority(job, 1);
         dirsToRemove.erase(it);
         q->addSubjob(job);
@@ -1710,6 +1731,7 @@ void CopyJobPrivate::setNextDirAttribute()
         ++m_directoriesCopiedIterator;
 
         KIO::SimpleJob *job = KIO::setModificationTime(url, dt);
+        job->setParentJob(q);
         Scheduler::setJobPriority(job, 1);
         q->addSubjob(job);
 
@@ -1921,7 +1943,7 @@ void CopyJobPrivate::slotResultRenaming(KJob *job)
                 const QString newName = KIO::suggestName(destDirectory, m_currentDestURL.fileName());
 
                 m_dest = destDirectory;
-                m_dest.setPath(m_dest.path() + QLatin1Char('/') + newName);
+                m_dest.setPath(concatPaths(m_dest.path(), newName));
                 KIO::Job *job = KIO::stat(m_dest, StatJob::DestinationSide, 2, KIO::HideProgressInfo);
                 state = STATE_STATING;
                 destinationState = DEST_NOT_STATED;
