@@ -29,6 +29,7 @@
 #include "kdirwatch.h"
 #include "kprotocolmanager.h"
 #include <kdirnotify.h>
+#include "../pathhelpers_p.h"
 
 #include <klocalizedstring.h>
 #include <kio/jobuidelegatefactory.h>
@@ -75,7 +76,7 @@ public:
         , m_totalFilesDirs(0)
         , m_srcList(src)
         , m_currentStat(m_srcList.begin())
-        , m_reportTimer(0)
+        , m_reportTimer(nullptr)
     {
     }
     DeleteJobState state;
@@ -110,6 +111,10 @@ public:
         if (!(flags & HideProgressInfo)) {
             KIO::getJobTracker()->registerJob(job);
         }
+        if (!(flags & NoPrivilegeExecution)) {
+            job->d_func()->m_privilegeExecutionEnabled = true;
+            job->d_func()->m_operationType = Delete;
+        }
         return job;
     }
 };
@@ -183,7 +188,7 @@ void DeleteJobPrivate::slotEntries(KIO::Job *job, const UDSEntryList &list)
                 url = QUrl(urlStr);
             } else {
                 url = static_cast<SimpleJob *>(job)->url(); // assumed to be a dir
-                url.setPath(url.path() + '/' + displayName);
+                url.setPath(concatPaths(url.path(), displayName));
             }
 
             //qDebug() << displayName << "(" << url << ")";
@@ -292,7 +297,7 @@ void DeleteJobPrivate::deleteNextFile()
             // If local file, try do it directly
             if ((*it).isLocalFile() && QFile::remove((*it).toLocalFile())) {
                 //kdDebug(7007) << "DeleteJob deleted" << (*it).toLocalFile();
-                job = 0;
+                job = nullptr;
                 m_processedFiles++;
                 if (m_processedFiles % 300 == 1 || m_totalFilesDirs < 300) {  // update progress info every 300 files
                     m_currentURL = *it;
@@ -305,6 +310,7 @@ void DeleteJobPrivate::deleteNextFile()
                     job = KIO::http_delete(*it, KIO::HideProgressInfo);
                 } else {
                     job = KIO::file_delete(*it, KIO::HideProgressInfo);
+                    job->setParentJob(q);
                 }
                 Scheduler::setJobPriority(job, 1);
                 m_currentURL = (*it);
@@ -343,6 +349,7 @@ void DeleteJobPrivate::deleteNextDir()
                 // Call rmdir - works for kioslaves with canDeleteRecursive too,
                 // CMD_DEL will trigger the recursive deletion in the slave.
                 SimpleJob *job = KIO::rmdir(*it);
+                job->setParentJob(q);
                 job->addMetaData(QStringLiteral("recurse"), QStringLiteral("true"));
                 Scheduler::setJobPriority(job, 1);
                 dirs.erase(it);
@@ -361,7 +368,7 @@ void DeleteJobPrivate::deleteNextDir()
         //qDebug() << "KDirNotify'ing FilesRemoved" << m_srcList;
         org::kde::KDirNotify::emitFilesRemoved(m_srcList);
     }
-    if (m_reportTimer != 0) {
+    if (m_reportTimer != nullptr) {
         m_reportTimer->stop();
     }
     q->emitResult();
