@@ -33,6 +33,8 @@
 #include <QUrl>
 #include <QPointer>
 #include <QDataStream>
+#include "kiocoredebug.h"
+#include "global.h"
 
 #define KIO_ARGS QByteArray packedArgs; QDataStream stream( &packedArgs, QIODevice::WriteOnly ); stream
 
@@ -40,17 +42,18 @@ namespace KIO
 {
 class Slave;
 
-class JobPrivate
+// Exported for KIOWidgets jobs
+class KIOCORE_EXPORT JobPrivate
 {
 public:
     JobPrivate()
-        : m_parentJob(0L), m_extraFlags(0),
-          m_uiDelegateExtension(KIO::defaultJobUiDelegateExtension())
+        : m_parentJob(nullptr), m_extraFlags(0),
+          m_uiDelegateExtension(KIO::defaultJobUiDelegateExtension()),
+          m_privilegeExecutionEnabled(false), m_confirmationAsked(false)
     {
     }
 
-    virtual ~JobPrivate()
-    {}
+    virtual ~JobPrivate();
 
     /**
      * Some extra storage space for jobs that don't have their own
@@ -63,6 +66,18 @@ public:
            EF_KillCalled          = (1 << 4)
          };
 
+    enum FileOperationType {
+        ChangeAttr, // chmod(), chown(), setModificationTime()
+        Copy,
+        Delete,
+        MkDir,
+        Move,
+        Rename,
+        Symlink,
+        Transfer, // put() and get()
+        Other // if other file operation set message, caption inside the job.
+    };
+
     // Maybe we could use the QObject parent/child mechanism instead
     // (requires a new ctor, and moving the ctor code to some init()).
     Job *m_parentJob;
@@ -72,7 +87,13 @@ public:
     MetaData m_outgoingMetaData;
     JobUiDelegateExtension *m_uiDelegateExtension;
     Job *q_ptr;
+    // For privilege operation
+    bool m_privilegeExecutionEnabled;
+    bool m_confirmationAsked;
+    QString m_caption, m_message;
+    FileOperationType m_operationType;
 
+    PrivilegeOperationStatus tryAskPrivilegeOpConfirmation();
     void slotSpeed(KJob *job, unsigned long speed);
 
     static void emitMoving(KIO::Job *, const QUrl &src, const QUrl &dest);
@@ -97,7 +118,7 @@ public:
      * @param packedArgs the arguments
      */
     SimpleJobPrivate(const QUrl &url, int command, const QByteArray &packedArgs)
-        : m_slave(0), m_packedArgs(packedArgs), m_url(url), m_command(command),
+        : m_slave(nullptr), m_packedArgs(packedArgs), m_url(url), m_command(command),
           m_checkOnHold(false), m_schedSerial(0), m_redirectionHandlingEnabled(true)
     {
 #if 0
@@ -171,6 +192,11 @@ public:
      */
     void _k_slotSlaveInfoMessage(const QString &s);
 
+    /**
+     * Called when privilegeOperationRequested() is emitted by slave.
+     */
+    void slotPrivilegeOperationRequested();
+
     /**
      * @internal
      * Called by the scheduler when a slave gets to
@@ -223,6 +249,23 @@ public:
         if (!(flags & HideProgressInfo)) {
             KIO::getJobTracker()->registerJob(job);
         }
+        if (!(flags & NoPrivilegeExecution)) {
+            job->d_func()->m_privilegeExecutionEnabled = true;
+            // Only delete, rename and symlink operation accept JobFlags.
+            FileOperationType opType;
+            switch (command) {
+            case CMD_DEL:
+                opType = Delete;
+                break;
+            case CMD_RENAME:
+                opType = Rename;
+                break;
+            case CMD_SYMLINK:
+                opType = Symlink;
+                break;
+            }
+            job->d_func()->m_operationType = opType;
+        }
         return job;
     }
 };
@@ -234,14 +277,15 @@ public:
                               const QByteArray &_staticData)
         : SimpleJobPrivate(url, command, packedArgs),
           m_internalSuspended(false), m_errorPage(false),
-          staticData(_staticData), m_isMimetypeEmitted(false), m_subJob(0)
+          staticData(_staticData), m_isMimetypeEmitted(false),
+          m_closedBeforeStart(false), m_subJob(nullptr)
     { }
 
     inline TransferJobPrivate(const QUrl &url, int command, const QByteArray &packedArgs,
                               QIODevice *ioDevice)
         : SimpleJobPrivate(url, command, packedArgs),
           m_internalSuspended(false), m_errorPage(false),
-          m_isMimetypeEmitted(false), m_subJob(0),
+          m_isMimetypeEmitted(false), m_closedBeforeStart(false), m_subJob(nullptr),
           m_outgoingDataSource(QPointer<QIODevice>(ioDevice))
     { }
 
@@ -252,6 +296,7 @@ public:
     QList<QUrl> m_redirectionList;
     QString m_mimetype;
     bool m_isMimetypeEmitted;
+    bool m_closedBeforeStart;
     TransferJob *m_subJob;
     QPointer<QIODevice> m_outgoingDataSource;
 
@@ -278,6 +323,7 @@ public:
      */
     virtual void slotDataReqFromDevice();
     void slotIODeviceClosed();
+    void slotIODeviceClosedBeforeStart();
 
     void slotErrorPage();
     void slotCanResume(KIO::filesize_t offset);
@@ -296,6 +342,10 @@ public:
         if (!(flags & HideProgressInfo)) {
             KIO::getJobTracker()->registerJob(job);
         }
+        if (!(flags & NoPrivilegeExecution)) {
+            job->d_func()->m_privilegeExecutionEnabled = true;
+            job->d_func()->m_operationType = Transfer;
+        }
         return job;
     }
 
@@ -309,6 +359,10 @@ public:
         if (!(flags & HideProgressInfo)) {
             KIO::getJobTracker()->registerJob(job);
         }
+        if (!(flags & NoPrivilegeExecution)) {
+            job->d_func()->m_privilegeExecutionEnabled = true;
+            job->d_func()->m_operationType = Transfer;
+        }
         return job;
     }
 };
