@@ -98,7 +98,7 @@ class ListJob;
 class OrgKdeKDirNotifyInterface;
 struct KCoreDirListerCacheDirectoryData;
 
-class KCoreDirLister::Private
+class Q_DECL_HIDDEN KCoreDirLister::Private
 {
 public:
     Private(KCoreDirLister *parent)
@@ -112,10 +112,10 @@ public:
 
         rootFileItem = KFileItem();
 
-        lstNewItems = 0;
-        lstRefreshItems = 0;
-        lstMimeFilteredItems = 0;
-        lstRemoveItems = 0;
+        lstNewItems = nullptr;
+        lstRefreshItems = nullptr;
+        lstMimeFilteredItems = nullptr;
+        lstRemoveItems = nullptr;
 
         hasPendingChanges = false;
     }
@@ -315,7 +315,7 @@ private:
 
     void stopListJob(const QString &url, bool silent);
 
-    KIO::ListJob *jobForUrl(const QString &url, KIO::ListJob *not_job = 0);
+    KIO::ListJob *jobForUrl(const QString &url, KIO::ListJob *not_job = nullptr);
     const QUrl &joburl(KIO::ListJob *job);
 
     void killJob(KIO::ListJob *job);
@@ -554,7 +554,7 @@ class KCoreDirLister::Private::CachedItemsJob : public KJob
 public:
     CachedItemsJob(KCoreDirLister *lister, const QUrl &url, bool reload);
 
-    /*reimp*/ void start()
+    /*reimp*/ void start() Q_DECL_OVERRIDE
     {
         QMetaObject::invokeMethod(this, "done", Qt::QueuedConnection);
     }
@@ -571,7 +571,7 @@ public:
     }
 
 protected:
-    virtual bool doKill();
+    bool doKill() Q_DECL_OVERRIDE;
 
 public Q_SLOTS:
     void done();
