@@ -22,6 +22,7 @@
 
 #include "kioglobal_p.h"
 #include "kiocoredebug.h"
+#include "../pathhelpers_p.h"
 
 #include <QtCore/QDate>
 #include <QtCore/QDir>
@@ -265,10 +266,7 @@ void KFileItemPrivate::readUDSEntry(bool _urlIsDirectory)
     m_hidden = hiddenVal == 1 ? Hidden : (hiddenVal == 0 ? Shown : Auto);
 
     if (_urlIsDirectory && !UDS_URL_seen && !m_strName.isEmpty() && m_strName != QLatin1String(".")) {
-        if (!m_url.path().endsWith('/')) {
-            m_url.setPath(m_url.path() + '/');
-        }
-        m_url.setPath(m_url.path() + m_strName);
+        m_url.setPath(concatPaths(m_url.path(), m_strName));
     }
 
     m_iconName.clear();
@@ -479,14 +477,18 @@ KFileItem::KFileItem(const QUrl &url, const QString &mimeType, mode_t mode)
     }
 }
 
-KFileItem::KFileItem(const KFileItem &other)
-    : d(other.d)
-{
-}
-
-KFileItem::~KFileItem()
-{
-}
+// Default implementations for:
+// - Copy constructor
+// - Move constructor
+// - Copy assignment
+// - Move assignment
+// - Destructor
+// The compiler will now generate the content of those.
+KFileItem::KFileItem(const KFileItem&) = default;
+KFileItem::~KFileItem() = default;
+KFileItem::KFileItem(KFileItem&&) = default;
+KFileItem& KFileItem::operator=(const KFileItem&) = default;
+KFileItem& KFileItem::operator=(KFileItem&&) = default;
 
 void KFileItem::refresh()
 {
@@ -556,7 +558,9 @@ void KFileItem::setName(const QString &name)
     }
 
     d->m_strName = name;
-    d->m_strText = KIO::decodeFileName(d->m_strName);
+    if (!d->m_strName.isEmpty()) {
+        d->m_strText = KIO::decodeFileName(d->m_strName);
+    }
     if (d->m_entry.contains(KIO::UDSEntry::UDS_NAME)) {
         d->m_entry.insert(KIO::UDSEntry::UDS_NAME, d->m_strName);    // #195385
     }
@@ -810,7 +814,7 @@ QString KFileItem::mimeComment() const
     // Support for .directory file in directories
     if (isLocalUrl && isDir() && isDirectoryMounted(url)) {
         QUrl u(url);
-        u.setPath(u.path() + QLatin1String("/.directory"));
+        u.setPath(concatPaths(u.path(), QLatin1String(".directory")));
         const KDesktopFile cfg(u.toLocalFile());
         const QString comment = cfg.readComment();
         if (!comment.isEmpty()) {
@@ -1018,7 +1022,7 @@ QStringList KFileItem::overlays() const
         if (KSambaShare::instance()->isDirectoryShared(d->m_url.toLocalFile()) ||
                 KNFSShare::instance()->isDirectoryShared(d->m_url.toLocalFile())) {
             //qDebug() << d->m_url.path();
-            names.append(QStringLiteral("network-workgroup"));
+            names.append(QStringLiteral("emblem-shared"));
         }
     }
 #endif  // Q_OS_WIN
@@ -1488,12 +1492,6 @@ KIO::UDSEntry KFileItem::entry() const
     return d->m_entry;
 }
 
-KFileItem &KFileItem::operator=(const KFileItem &other)
-{
-    d = other.d;
-    return *this;
-}
-
 bool KFileItem::isNull() const
 {
     return d == nullptr;
