@@ -35,6 +35,8 @@
 class KFileItemPrivate;
 
 /**
+ * @class KFileItem kfileitem.h <KFileItem>
+ *
  * A KFileItem is a generic class to handle a file, local or remote.
  * In particular, it makes it easier to handle the result of KIO::listDir
  * (UDSEntry isn't very friendly to use).
@@ -116,17 +118,30 @@ public:
     /**
      * Copy constructor
      */
-    KFileItem(const KFileItem &other);
-    /**
-     * Assignment operator
-     */
-    KFileItem &operator=(const KFileItem &other);
+    KFileItem(const KFileItem&);
 
     /**
-     * Destructs the KFileItem.
+     * Destructor
      */
     ~KFileItem();
 
+    /**
+     * Move constructor
+     * @since 5.43
+     */
+    KFileItem(KFileItem&&);
+
+    /**
+     * Copy assignment
+     */
+    KFileItem& operator=(const KFileItem&);
+
+    /**
+     * Move assignment
+     * @since 5.43
+     */
+    KFileItem& operator=(KFileItem&&);
+
     /**
      * Throw away and re-read (for local files) all information about the file.
      * This is called when the _file_ changes.
@@ -238,7 +253,7 @@ public:
     bool isDir() const;
 
     /**
-     * Returns true if this item represents a file (and not a a directory)
+     * Returns true if this item represents a file (and not a directory)
      * @return true if the item is a file
      */
     bool isFile() const;
@@ -528,6 +543,8 @@ inline uint qHash(const KFileItem &item)
 }
 
 /**
+ * @class KFileItemList kfileitem.h <KFileItem>
+ *
  * List of KFileItems, which adds a few helper
  * methods to QList<KFileItem>.
  */
