@@ -19,6 +19,7 @@
 
 #include "ktcpsocket.h"
 #include "ktcpsocket_p.h"
+#include "kiocoredebug.h"
 
 #include <ksslcertificatemanager.h>
 #include <klocalizedstring.h>
@@ -348,11 +349,13 @@ public:
     //private slots
     void reemitSocketError(QAbstractSocket::SocketError e)
     {
+        q->setErrorString(sock.errorString());
         emit q->error(errorFromAbsSocket(e));
     }
 
     void reemitSslErrors(const QList<QSslError> &errors)
     {
+        q->setErrorString(sock.errorString());
         q->showSslErrors(); //H4X
         QList<KSslError> kErrors;
         foreach (const QSslError &e, errors) {
@@ -805,7 +808,7 @@ void KTcpSocket::startClientEncryption()
 void KTcpSocket::showSslErrors()
 {
     foreach (const QSslError &e, d->sock.sslErrors()) {
-        qDebug() << e.errorString();
+        qCDebug(KIO_CORE) << e.errorString();
     }
 }
 
