@@ -19,12 +19,13 @@
 */
 
 #include "mkpathjob.h"
-
 #include "job_p.h"
-
 #include "mkdirjob.h"
+#include "../pathhelpers_p.h"
+
 #include <QTimer>
 #include <QDebug>
+#include <QDir>
 #include <QFileInfo>
 
 using namespace KIO;
@@ -43,8 +44,9 @@ public:
         m_url.setPath(QStringLiteral("/"));
         int i = 0;
         for (; i < basePathComponents.count() && i < m_pathComponents.count(); ++i) {
-            if (m_pathComponents.at(i) == basePathComponents.at(i)) {
-                m_url.setPath(m_url.path() + '/' + m_pathComponents.at(i));
+            const QString pathComponent = m_pathComponents.at(i);
+            if (pathComponent == basePathComponents.at(i)) {
+                m_url.setPath(concatPaths(m_url.path(), pathComponent));
             } else {
                 break;
             }
@@ -57,7 +59,13 @@ public:
         if (m_url.isLocalFile()) {
             i = 0;
             for (; i < m_pathComponents.count(); ++i) {
-                QString testDir = m_url.toLocalFile() + '/' + m_pathComponents.at(i);
+                const QString localFile = m_url.toLocalFile();
+                QString testDir;
+                if (localFile == QLatin1String("/")) {
+                    testDir = localFile + m_pathComponents.at(i);
+                } else {
+                    testDir = localFile + '/' + m_pathComponents.at(i);
+                }
                 if (QFileInfo(testDir).isDir()) {
                     m_url.setPath(testDir);
                 } else {
@@ -88,6 +96,10 @@ public:
         if (!(flags & HideProgressInfo)) {
             KIO::getJobTracker()->registerJob(job);
         }
+        if (!(flags & NoPrivilegeExecution)) {
+            job->d_func()->m_privilegeExecutionEnabled = true;
+            job->d_func()->m_operationType = MkDir;
+        }
         return job;
     }
 
@@ -112,8 +124,9 @@ void MkpathJobPrivate::slotStart()
     }
 
     if (m_pathIterator != m_pathComponents.constEnd()) {
-        m_url.setPath(m_url.path() + '/' + *m_pathIterator);
+        m_url.setPath(concatPaths(m_url.path(), *m_pathIterator));
         KIO::Job* job = KIO::mkdir(m_url);
+        job->setParentJob(q);
         q->addSubjob(job);
         q->setProcessedAmount(KJob::Directories, q->processedAmount(KJob::Directories) + 1);
     } else {
