@@ -128,35 +128,34 @@ void SimpleJobPrivate::start(Slave *slave)
     m_slave = slave;
 
     // Slave::setJob can send us SSL metadata if there is a persistent connection
-    q->connect(slave, SIGNAL(metaData(KIO::MetaData)),
-               SLOT(slotMetaData(KIO::MetaData)));
+    QObject::connect(slave, &Slave::metaData, q, &SimpleJob::slotMetaData);
 
     slave->setJob(q);
 
-    q->connect(slave, SIGNAL(error(int,QString)),
-               SLOT(slotError(int,QString)));
+    QObject::connect(slave, &Slave::error, q, &SimpleJob::slotError);
 
-    q->connect(slave, SIGNAL(warning(QString)),
-               SLOT(slotWarning(QString)));
+    QObject::connect(slave, &Slave::warning, q, &SimpleJob::slotWarning);
 
-    q->connect(slave, SIGNAL(infoMessage(QString)),
-               SLOT(_k_slotSlaveInfoMessage(QString)));
+    QObject::connect(slave, &Slave::finished, q, &SimpleJob::slotFinished);
 
-    q->connect(slave, SIGNAL(connected()),
-               SLOT(slotConnected()));
+    QObject::connect(slave, &Slave::infoMessage, q,
+        [this](const QString& message){ _k_slotSlaveInfoMessage(message);} );
 
-    q->connect(slave, SIGNAL(finished()),
-               SLOT(slotFinished()));
+    QObject::connect(slave, &Slave::connected, q,
+        [this](){ slotConnected();} );
+
+    QObject::connect(slave, &Slave::privilegeOperationRequested, q,
+        [this](){ slotPrivilegeOperationRequested();} );
 
     if ((m_extraFlags & EF_TransferJobDataSent) == 0) { // this is a "get" job
-        q->connect(slave, SIGNAL(totalSize(KIO::filesize_t)),
-                   SLOT(slotTotalSize(KIO::filesize_t)));
+        QObject::connect(slave, &Slave::totalSize, q,
+            [this](KIO::filesize_t size){ slotTotalSize(size);} );
 
-        q->connect(slave, SIGNAL(processedSize(KIO::filesize_t)),
-                   SLOT(slotProcessedSize(KIO::filesize_t)));
+        QObject::connect(slave, &Slave::processedSize, q,
+            [this](KIO::filesize_t size){ slotProcessedSize(size);} );
 
-        q->connect(slave, SIGNAL(speed(ulong)),
-                   SLOT(slotSpeed(ulong)));
+        QObject::connect(slave, &Slave::speed, q,
+            [this](ulong speed){ slotSpeed(speed);} );
     }
 
     const QVariant windowIdProp = q->property("window-id"); // see KJobWidgets::setWindow
@@ -169,7 +168,7 @@ void SimpleJobPrivate::start(Slave *slave)
         m_outgoingMetaData.insert(QStringLiteral("user-timestamp"), QString::number(userTimestampProp.toULongLong()));
     }
 
-    if (q->uiDelegate() == 0) {            // not interactive
+    if (q->uiDelegate() == nullptr) {            // not interactive
         m_outgoingMetaData.insert(QStringLiteral("no-auth-prompt"), QStringLiteral("true"));
     }
 
@@ -184,6 +183,9 @@ void SimpleJobPrivate::start(Slave *slave)
     }
 
     slave->send(m_command, m_packedArgs);
+    if (q->isSuspended()) {
+       slave->suspend();
+    }
 }
 
 void SimpleJobPrivate::slaveDone()
@@ -334,6 +336,11 @@ void SimpleJob::storeSSLSessionFromJob(const QUrl &redirectionURL)
     Q_UNUSED(redirectionURL);
 }
 
+void SimpleJobPrivate::slotPrivilegeOperationRequested()
+{
+    m_slave->send(MSG_PRIVILEGE_EXEC, QByteArray::number(tryAskPrivilegeOpConfirmation()));
+}
+
 //////////
 SimpleJob *KIO::rmdir(const QUrl &url)
 {
@@ -366,7 +373,7 @@ SimpleJob *KIO::rename(const QUrl &src, const QUrl &dest, JobFlags flags)
 {
     //qDebug() << "rename " << src << " " << dest;
     KIO_ARGS << src << dest << (qint8)(flags & Overwrite);
-    return SimpleJobPrivate::newJob(src, CMD_RENAME, packedArgs);
+    return SimpleJobPrivate::newJob(src, CMD_RENAME, packedArgs, flags);
 }
 
 SimpleJob *KIO::symlink(const QString &target, const QUrl &dest, JobFlags flags)
