@@ -50,7 +50,8 @@
 #include "commands_p.h"
 #include "ioslave_defaults.h"
 #include "slaveinterface.h"
-#include "kpasswdserverclient_p.h"
+#include "kpasswdserverclient.h"
+#include "kiocoredebug.h"
 
 #ifndef NDEBUG
 #if HAVE_BACKTRACE
@@ -79,7 +80,7 @@ class SlaveBasePrivate
 {
 public:
     SlaveBase *q;
-    SlaveBasePrivate(SlaveBase *owner): q(owner), m_passwdServerClient(0)
+    SlaveBasePrivate(SlaveBase *owner): q(owner), nextTimeoutMsecs(0), m_passwdServerClient(nullptr)
     {
         if (!qEnvironmentVariableIsEmpty("KIOSLAVE_ENABLE_TESTMODE")) {
             QStandardPaths::enableTestMode(true);
@@ -96,7 +97,6 @@ public:
     Connection appConnection;
     QString poolSocket;
     bool isConnectedToApp;
-    static qlonglong s_seqNr;
 
     QString slaveid;
     bool resume: 1;
@@ -110,8 +110,9 @@ public:
     KConfigGroup *configGroup;
     QUrl onHoldUrl;
 
-    QDateTime lastTimeout;
-    QDateTime nextTimeout;
+    QElapsedTimer lastTimeout;
+    QElapsedTimer nextTimeout;
+    qint64 nextTimeoutMsecs;
     KIO::filesize_t totalSize;
     KRemoteEncoding *remotefile;
     enum { Idle, InsideMethod, FinishedCalled, ErrorCalled } m_state;
@@ -141,14 +142,14 @@ public:
     void verifyState(const char *cmdName)
     {
         if ((m_state != FinishedCalled) && (m_state != ErrorCalled)) {
-            qWarning() << cmdName << "did not call finished() or error()! Please fix the KIO slave.";
+            qCWarning(KIO_CORE) << cmdName << "did not call finished() or error()! Please fix the" << QCoreApplication::applicationName() << "KIO slave";
         }
     }
 
     void verifyErrorFinishedNotCalled(const char *cmdName)
     {
         if (m_state == FinishedCalled || m_state == ErrorCalled) {
-            qWarning() << cmdName << "called finished() or error(), but it's not supposed to! Please fix the KIO slave.";
+            qCWarning(KIO_CORE) << cmdName << "called finished() or error(), but it's not supposed to! Please fix the" << QCoreApplication::applicationName() << "KIO slave";
         }
     }
 
@@ -165,7 +166,6 @@ public:
 }
 
 static SlaveBase *globalSlave;
-qlonglong SlaveBasePrivate::s_seqNr;
 
 static volatile bool slaveWriteError = false;
 
@@ -181,7 +181,7 @@ extern "C" {
         //qDebug()<<"kioslave : exiting due to signal "<<sigNumber;
         //set the flag which will be checked in dispatchLoop() and which *should* be checked
         //in lengthy operations in the various slaves
-        if (globalSlave != 0) {
+        if (globalSlave != nullptr) {
             globalSlave->setKillFlag();
         }
         ::signal(SIGALRM, SIG_DFL);
@@ -232,7 +232,7 @@ SlaveBase::SlaveBase(const QByteArray &protocol,
     act.sa_handler = sigpipe_handler;
     sigemptyset(&act.sa_mask);
     act.sa_flags = 0;
-    sigaction(SIGPIPE, &act, 0);
+    sigaction(SIGPIPE, &act, nullptr);
 
     ::signal(SIGINT, &genericsig_handler);
     ::signal(SIGQUIT, &genericsig_handler);
@@ -257,7 +257,7 @@ SlaveBase::SlaveBase(const QByteArray &protocol,
     d->totalSize = 0;
     connectSlave(QFile::decodeName(app_socket));
 
-    d->remotefile = 0;
+    d->remotefile = nullptr;
     d->inOpenLoop = false;
     d->exit_loop = false;
 }
@@ -274,9 +274,9 @@ SlaveBase::~SlaveBase()
 void SlaveBase::dispatchLoop()
 {
     while (!d->exit_loop) {
-        if (d->nextTimeout.isValid() && (d->nextTimeout < QDateTime::currentDateTime())) {
+        if (d->nextTimeout.isValid() && (d->nextTimeout.hasExpired(d->nextTimeoutMsecs))) {
             QByteArray data = d->timeoutData;
-            d->nextTimeout = QDateTime();
+            d->nextTimeout.invalidate();
             d->timeoutData = QByteArray();
             special(data);
         }
@@ -285,7 +285,7 @@ void SlaveBase::dispatchLoop()
 
         int ms = -1;
         if (d->nextTimeout.isValid()) {
-            ms = qMax<int>(QDateTime::currentDateTime().msecsTo(d->nextTimeout), 1);
+            ms = qMax<int>(d->nextTimeout.elapsed() - d->nextTimeoutMsecs, 1);
         }
 
         int ret = -1;
@@ -325,11 +325,11 @@ void SlaveBase::dispatchLoop()
         }
 
         // execute deferred deletes
-        QCoreApplication::sendPostedEvents(NULL, QEvent::DeferredDelete);
+        QCoreApplication::sendPostedEvents(nullptr, QEvent::DeferredDelete);
     }
 
     // execute deferred deletes
-    QCoreApplication::sendPostedEvents(NULL, QEvent::DeferredDelete);
+    QCoreApplication::sendPostedEvents(nullptr, QEvent::DeferredDelete);
 }
 
 void SlaveBase::connectSlave(const QString &address)
@@ -435,10 +435,10 @@ void SlaveBase::opened()
 void SlaveBase::error(int _errid, const QString &_text)
 {
     if (d->m_state == d->ErrorCalled) {
-        qWarning() << "error() called twice! Please fix the KIO slave.";
+        qCWarning(KIO_CORE) << "error() called twice! Please fix the" << QCoreApplication::applicationName() << "KIO slave";
         return;
     } else if (d->m_state == d->FinishedCalled) {
-        qWarning() << "error() called after finished()! Please fix the KIO slave.";
+        qCWarning(KIO_CORE) << "error() called after finished()! Please fix the" << QCoreApplication::applicationName() << "KIO slave";
         return;
     }
 
@@ -463,7 +463,7 @@ void SlaveBase::finished()
 {
     if (!d->pendingListEntries.isEmpty()) {
         if (!d->m_rootEntryListed) {
-            qWarning() << "UDSEntry for '.' not found, creating a default one. Please fix the" << QCoreApplication::applicationName() << "KIO slave";
+            qCWarning(KIO_CORE) << "UDSEntry for '.' not found, creating a default one. Please fix the" << QCoreApplication::applicationName() << "KIO slave";
             KIO::UDSEntry entry;
             entry.insert(KIO::UDSEntry::UDS_NAME, QStringLiteral("."));
             entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
@@ -477,10 +477,10 @@ void SlaveBase::finished()
     }
 
     if (d->m_state == d->FinishedCalled) {
-        qWarning() << "finished() called twice! Please fix the KIO slave.";
+        qCWarning(KIO_CORE) << "finished() called twice! Please fix the" << QCoreApplication::applicationName() << "KIO slave";
         return;
     } else if (d->m_state == d->ErrorCalled) {
-        qWarning() << "finished() called after error()! Please fix the KIO slave.";
+        qCWarning(KIO_CORE) << "finished() called after error()! Please fix the" << QCoreApplication::applicationName() << "KIO slave";
         return;
     }
 
@@ -530,13 +530,11 @@ void SlaveBase::processedSize(KIO::filesize_t _bytes)
 {
     bool emitSignal = false;
 
-    QDateTime now = QDateTime::currentDateTime();
-
     if (_bytes == d->totalSize) {
         emitSignal = true;
     } else {
         if (d->lastTimeout.isValid()) {
-            emitSignal = d->lastTimeout.msecsTo(now);    // emit size 10 times a second
+            emitSignal = d->lastTimeout.hasExpired(100); // emit size 10 times a second
         } else {
             emitSignal = true;
         }
@@ -545,7 +543,7 @@ void SlaveBase::processedSize(KIO::filesize_t _bytes)
     if (emitSignal) {
         KIO_DATA << KIO_FILESIZE_T(_bytes);
         send(INF_PROCESSED_SIZE, data);
-        d->lastTimeout = now;
+        d->lastTimeout.start();
     }
 
     //    d->processed_size = _bytes;
@@ -929,7 +927,7 @@ void SlaveBase::slave_status()
 void SlaveBase::reparseConfiguration()
 {
     delete d->remotefile;
-    d->remotefile = 0;
+    d->remotefile = nullptr;
 }
 
 bool SlaveBase::openPasswordDialog(AuthInfo &info, const QString &errorMsg)
@@ -958,18 +956,11 @@ int SlaveBase::openPasswordDialogV2(AuthInfo &info, const QString &errorMsg)
     dlgInfo.setExtraField(QStringLiteral("skip-caching-on-query"), true);
 
     KPasswdServerClient *passwdServerClient = d->passwdServerClient();
-    qlonglong seqNr = passwdServerClient->queryAuthInfo(dlgInfo, errorMessage, windowId,
-                                                        SlaveBasePrivate::s_seqNr, userTimestamp);
-    if (seqNr > 0) {
-        SlaveBasePrivate::s_seqNr = seqNr;
-        if (dlgInfo.isModified()) {
-            info = dlgInfo;
-            return KJob::NoError;
-        }
-    } else if (seqNr < 0) {
-        return ERR_PASSWD_SERVER;
+    const int errCode = passwdServerClient->queryAuthInfo(&dlgInfo, errorMessage, windowId, userTimestamp);
+    if (errCode == KJob::NoError) {
+        info = dlgInfo;
     }
-    return ERR_USER_CANCELED;
+    return errCode;
 }
 
 int SlaveBase::messageBox(MessageBoxType type, const QString &text, const QString &caption,
@@ -1054,11 +1045,13 @@ int SlaveBase::readData(QByteArray &buffer)
 void SlaveBase::setTimeoutSpecialCommand(int timeout, const QByteArray &data)
 {
     if (timeout > 0) {
-        d->nextTimeout = QDateTime::currentDateTime().addSecs(timeout);
+        d->nextTimeoutMsecs = timeout*1000; // from seconds to miliseconds
+        d->nextTimeout.start();
     } else if (timeout == 0) {
-        d->nextTimeout = QDateTime::currentDateTime().addSecs(1);    // Immediate timeout
+        d->nextTimeoutMsecs = 1000;  // Immediate timeout
+        d->nextTimeout.start();
     } else {
-        d->nextTimeout = QDateTime();    // Canceled
+        d->nextTimeout.invalidate();  // Canceled
     }
 
     d->timeoutData = data;
@@ -1073,8 +1066,6 @@ void SlaveBase::dispatch(int command, const QByteArray &data)
 
     switch (command) {
     case CMD_HOST: {
-        // Reset s_seqNr, see kpasswdserver/DESIGN
-        SlaveBasePrivate::s_seqNr = 0;
         QString passwd;
         QString host, user;
         quint16 port;
@@ -1106,7 +1097,7 @@ void SlaveBase::dispatch(int command, const QByteArray &data)
         disconnectSlave();
         d->isConnectedToApp = true;
         connectSlave(app_socket);
-        virtual_hook(AppConnectionMade, 0);
+        virtual_hook(AppConnectionMade, nullptr);
     } break;
     case CMD_SLAVE_HOLD: {
         QUrl url;
@@ -1129,7 +1120,7 @@ void SlaveBase::dispatch(int command, const QByteArray &data)
         stream >> d->configData;
         d->rebuildConfig();
         delete d->remotefile;
-        d->remotefile = 0;
+        d->remotefile = nullptr;
     } break;
     case CMD_GET: {
         stream >> url;
@@ -1285,7 +1276,7 @@ void SlaveBase::dispatch(int command, const QByteArray &data)
         d->m_state = d->Idle;
     } break;
     case CMD_NONE: {
-        qWarning() << "Got unexpected CMD_NONE!";
+        qCWarning(KIO_CORE) << "Got unexpected CMD_NONE!";
     } break;
     case CMD_MULTI_GET: {
         d->m_state = d->InsideMethod;
@@ -1313,7 +1304,7 @@ void SlaveBase::dispatch(int command, const QByteArray &data)
 bool SlaveBase::checkCachedAuthentication(AuthInfo &info)
 {
     KPasswdServerClient *passwdServerClient = d->passwdServerClient();
-    return (passwdServerClient->checkAuthInfo(info, metaData(QStringLiteral("window-id")).toLong(),
+    return (passwdServerClient->checkAuthInfo(&info, metaData(QStringLiteral("window-id")).toLong(),
                                         metaData(QStringLiteral("user-timestamp")).toULong()));
 }
 
@@ -1467,3 +1458,11 @@ int SlaveBase::waitForHostInfo(QHostInfo &info)
 
     return result;
 }
+
+PrivilegeOperationStatus SlaveBase::requestPrivilegeOperation()
+{
+    QByteArray buffer;
+    send(MSG_PRIVILEGE_EXEC);
+    waitForAnswer(MSG_PRIVILEGE_EXEC, 0, buffer);
+    return KIO::PrivilegeOperationStatus(buffer.toInt());
+}
