@@ -25,6 +25,7 @@
  */
 
 #include "tcpslavebase.h"
+#include "kiocoredebug.h"
 
 #include <kconfiggroup.h>
 #include <ksslcertificatemanager.h>
@@ -79,7 +80,7 @@ Q_DECLARE_OPERATORS_FOR_FLAGS(TCPSlaveBase::SslResult)
  */
 
 /** @internal */
-class TCPSlaveBase::TcpSlaveBasePrivate
+class Q_DECL_HIDDEN TCPSlaveBase::TcpSlaveBasePrivate
 {
 public:
     TcpSlaveBasePrivate(TCPSlaveBase *qq) : q(qq) {}
@@ -492,7 +493,7 @@ bool TCPSlaveBase::startSsl()
     if (d->usingSSL) {
         return false;
     }
-    return d->startTLSInternal(KTcpSocket::TlsV1) & ResultOk;
+    return d->startTLSInternal(KTcpSocket::SecureProtocols) & ResultOk;
 }
 
 TCPSlaveBase::SslResult TCPSlaveBase::TcpSlaveBasePrivate::startTLSInternal(KTcpSocket::SslVersion version,
@@ -843,7 +844,7 @@ TCPSlaveBase::SslResult TCPSlaveBase::verifyServerCertificate()
         case SlaveBase::Cancel:
             return ResultFailed;
         default:
-            qWarning() << "Unexpected MessageBox response received:" << msgResult;
+            qCWarning(KIO_CORE) << "Unexpected MessageBox response received:" << msgResult;
             return ResultFailed;
         }
     } while (msgResult == SlaveBase::Yes);
@@ -856,7 +857,7 @@ TCPSlaveBase::SslResult TCPSlaveBase::verifyServerCertificate()
     cm->setRule(rule);
 
     return ResultOk | ResultOverridden;
-#if 0 //### need to to do something like the old code about the main and subframe stuff
+#if 0 //### need to do something like the old code about the main and subframe stuff
     //qDebug() << "SSL HTTP frame the parent? " << metaData("main_frame_request");
     if (!hasMetaData("main_frame_request") || metaData("main_frame_request") == "TRUE") {
         // Since we're the parent, we need to teach the child.
@@ -984,7 +985,7 @@ bool TCPSlaveBase::waitForResponse(int t)
 void TCPSlaveBase::setBlocking(bool b)
 {
     if (!b) {
-        qWarning() << "Caller requested non-blocking mode, but that doesn't work";
+        qCWarning(KIO_CORE) << "Caller requested non-blocking mode, but that doesn't work";
         return;
     }
     d->isBlocking = b;
