@@ -28,6 +28,7 @@
 #include "kfilemetapreview_p.h"
 #include "kpreviewwidgetbase.h"
 #include "knewfilemenu.h"
+#include "../pathhelpers_p.h"
 
 #include <config-kiofilewidgets.h>
 #include <defaults-kfile.h> // ConfigGroup, DefaultShowHidden, DefaultDirsFirst, DefaultSortReversed
@@ -165,7 +166,7 @@ void KDirOperator::keyPressEvent(QKeyEvent *e)
     }
 }
 
-class KDirOperator::Private
+class Q_DECL_HIDDEN KDirOperator::Private
 {
 public:
     Private(KDirOperator *parent);
@@ -186,6 +187,7 @@ public:
     void updateSorting(QDir::SortFlags sort);
 
     static bool isReadable(const QUrl &url);
+    bool isSchemeSupported(const QString &scheme) const;
 
     KFile::FileView allViews();
 
@@ -288,6 +290,7 @@ public:
     QList<QUrl> itemsToBeSetAsCurrent;
     bool shouldFetchForItems;
     InlinePreviewState inlinePreviewState;
+    QStringList supportedSchemes;
 };
 
 KDirOperator::Private::Private(KDirOperator *_parent) :
@@ -372,7 +375,11 @@ KDirOperator::KDirOperator(const QUrl &_url, QWidget *parent) :
             d->currUrl.setScheme(QStringLiteral("file"));
         }
 
-        d->currUrl.setPath(d->currUrl.path() + "/"); // make sure we have a trailing slash!
+        QString path = d->currUrl.path();
+        if (!path.endsWith('/')) {
+            path.append('/'); // make sure we have a trailing slash!
+        }
+        d->currUrl.setPath(path);
     }
 
     // We set the direction of this widget to LTR, since even on RTL desktops
@@ -730,7 +737,7 @@ bool KDirOperator::mkdir(const QString &directory, bool enterDirectory)
     QStringList::ConstIterator it = dirs.begin();
 
     for (; it != dirs.end(); ++it) {
-        folderurl.setPath(folderurl.path() + '/' + *it);
+        folderurl.setPath(concatPaths(folderurl.path(), *it));
         if (folderurl.isLocalFile()) {
             exists = QFile::exists(folderurl.toLocalFile());
         } else {
@@ -789,7 +796,7 @@ KIO::DeleteJob *KDirOperator::del(const KFileItemList &items,
         KIO::JobFlags flags = showProgress ? KIO::DefaultFlags : KIO::HideProgressInfo;
         KIO::DeleteJob *job = KIO::del(urls, flags);
         KJobWidgets::setWindow(job, this);
-        job->ui()->setAutoErrorHandlingEnabled(true);
+        job->uiDelegate()->setAutoErrorHandlingEnabled(true);
         return job;
     }
 
@@ -828,7 +835,7 @@ KIO::CopyJob *KDirOperator::trash(const KFileItemList &items,
         KIO::JobFlags flags = showProgress ? KIO::DefaultFlags : KIO::HideProgressInfo;
         KIO::CopyJob *job = KIO::trash(urls, flags);
         KJobWidgets::setWindow(job, this);
-        job->ui()->setAutoErrorHandlingEnabled(true);
+        job->uiDelegate()->setAutoErrorHandlingEnabled(true);
         return job;
     }
 
@@ -993,6 +1000,9 @@ void KDirOperator::setUrl(const QUrl &_newurl, bool clearforward)
         return;
     }
 
+    if (!d->isSchemeSupported(newurl.scheme()))
+        return;
+
     if (!Private::isReadable(newurl)) {
         // maybe newurl is a file? check its parent directory
         newurl = newurl.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash);
@@ -1055,9 +1065,16 @@ void KDirOperator::rereadDir()
     d->openUrl(d->currUrl, KDirLister::Reload);
 }
 
+bool KDirOperator::Private::isSchemeSupported(const QString &scheme) const
+{
+    return supportedSchemes.isEmpty() || supportedSchemes.contains(scheme);
+}
+
 bool KDirOperator::Private::openUrl(const QUrl &url, KDirLister::OpenUrlFlags flags)
 {
-    const bool result = KProtocolManager::supportsListing(url) && dirLister->openUrl(url, flags);
+    const bool result = KProtocolManager::supportsListing(url)
+                        && isSchemeSupported(url.scheme())
+                        && dirLister->openUrl(url, flags);
     if (!result) { // in that case, neither completed() nor canceled() will be emitted by KDL
         _k_slotCanceled();
     }
@@ -1940,16 +1957,19 @@ void KDirOperator::setupActions()
 
     KToggleAction *showHiddenAction = new KToggleAction(i18n("Show Hidden Files"), this);
     d->actionCollection->addAction(QStringLiteral("show hidden"), showHiddenAction);
+    showHiddenAction->setShortcuts({Qt::ALT + Qt::Key_Period, Qt::CTRL + Qt::Key_H, Qt::Key_F8});
     connect(showHiddenAction, SIGNAL(toggled(bool)), SLOT(_k_slotToggleHidden(bool)));
 
     KToggleAction *previewAction = new KToggleAction(i18n("Show Aside Preview"), this);
     d->actionCollection->addAction(QStringLiteral("preview"), previewAction);
+    previewAction->setShortcut(Qt::Key_F11);
     connect(previewAction, SIGNAL(toggled(bool)),
             SLOT(_k_togglePreview(bool)));
 
     KToggleAction *inlinePreview = new KToggleAction(QIcon::fromTheme(QStringLiteral("view-preview")),
             i18n("Show Preview"), this);
     d->actionCollection->addAction(QStringLiteral("inline preview"), inlinePreview);
+    inlinePreview->setShortcut(Qt::Key_F12);
     connect(inlinePreview, SIGNAL(toggled(bool)), SLOT(_k_toggleInlinePreviews(bool)));
 
     QAction *fileManager = new QAction(i18n("Open File Manager"), this);
@@ -2628,5 +2648,16 @@ void KDirOperator::Private::_k_slotDirectoryCreated(const QUrl &url)
     parent->setUrl(url, true);
 }
 
+void KDirOperator::setSupportedSchemes(const QStringList &schemes)
+{
+    d->supportedSchemes = schemes;
+    rereadDir();
+}
+
+QStringList KDirOperator::supportedSchemes() const
+{
+    return d->supportedSchemes;
+}
+
 #include "moc_kdiroperator.cpp"
 #include "kdiroperator.moc"
