@@ -50,6 +50,8 @@ class DeleteJob;
 }
 
 /**
+ * @class KDirOperator kdiroperator.h <KDirOperator>
+ *
  * This widget works as a network transparent filebrowser. You specify a URL
  * to display and this url will be loaded via KDirLister. The user can
  * browse through directories, highlight and select files, delete or rename
@@ -122,7 +124,7 @@ public:
      * This constructor doesn't start loading the url, setView will do it.
      */
     explicit KDirOperator(const QUrl &urlName  = QUrl(),
-                          QWidget *parent = 0);
+                          QWidget *parent = nullptr);
     /**
      * Destroys the KDirOperator.
      */
@@ -506,7 +508,7 @@ public:
      * @param ask specifies whether a confirmation dialog should be shown
      * @param showProgress passed to the DeleteJob to show a progress dialog
      */
-    virtual KIO::DeleteJob *del(const KFileItemList &items, QWidget *parent = 0,
+    virtual KIO::DeleteJob *del(const KFileItemList &items, QWidget *parent = nullptr,
                                 bool ask = true, bool showProgress = true);
 
     /**
@@ -625,6 +627,16 @@ public:
      */
     bool isSaving() const;
 
+    /**
+     * Returns the URL schemes that the file widget should allow navigating to.
+     *
+     * If the returned list is empty, all schemes are supported.
+     *
+     * @sa QFileDialog::supportedSchemes
+     * @since 5.43
+     */
+    QStringList supportedSchemes() const;
+
 protected:
     /**
      * A view factory for creating predefined fileviews. Called internally by setView,
@@ -772,6 +784,17 @@ public Q_SLOTS:
      */
     void setIconsZoom(int value);
 
+    /**
+     * Set the URL schemes that the file widget should allow navigating to.
+     *
+     * If the returned list is empty, all schemes are supported. Examples for
+     * schemes are @c "file" or @c "ftp".
+     *
+     * @sa QFileDialog::setSupportedSchemes
+     * @since 5.43
+     */
+    void setSupportedSchemes(const QStringList &schemes);
+
 protected Q_SLOTS:
     /**
      * Restores the normal cursor after showing the busy-cursor. Also hides
