@@ -18,7 +18,6 @@
 */
 
 #include "kfileplacesitem_p.h"
-#include "kfileplacesmodel.h"
 
 #include <QtCore/QDateTime>
 #include <QIcon>
@@ -28,7 +27,9 @@
 #include <klocalizedstring.h>
 #include <KConfig>
 #include <KConfigGroup>
+#include <kprotocolinfo.h>
 #include <solid/block.h>
+#include <solid/networkshare.h>
 #include <solid/opticaldisc.h>
 #include <solid/opticaldrive.h>
 #include <solid/storageaccess.h>
@@ -44,8 +45,9 @@ KFilePlacesItem::KFilePlacesItem(KBookmarkManager *manager,
                                  const QString &address,
                                  const QString &udi)
     : m_manager(manager), m_folderIsEmpty(true), m_isCdrom(false),
-      m_isAccessible(false), m_device(udi)
+      m_isAccessible(false)
 {
+    updateDeviceInfo(udi);
     setBookmark(m_manager->findByAddress(address));
 
     if (udi.isEmpty() && m_bookmark.metaDataItem(QStringLiteral("ID")).isEmpty()) {
@@ -56,18 +58,6 @@ KFilePlacesItem::KFilePlacesItem(KBookmarkManager *manager,
             const KConfigGroup group = cfg.group("Status");
             m_folderIsEmpty = group.readEntry("Empty", true);
         }
-    } else if (!udi.isEmpty() && m_device.isValid()) {
-        m_access = m_device.as<Solid::StorageAccess>();
-        m_volume = m_device.as<Solid::StorageVolume>();
-        m_disc = m_device.as<Solid::OpticalDisc>();
-        m_mtp = m_device.as<Solid::PortableMediaPlayer>();
-        if (m_access) {
-            connect(m_access, SIGNAL(accessibilityChanged(bool,QString)),
-                    this, SLOT(onAccessibilityChanged(bool)));
-            onAccessibilityChanged(m_access->isAccessible());
-        }
-        m_iconPath = m_device.icon();
-        m_emblems = m_device.emblems();
     }
 }
 
@@ -98,6 +88,8 @@ void KFilePlacesItem::setBookmark(const KBookmark &bookmark)
 {
     m_bookmark = bookmark;
 
+    updateDeviceInfo(m_bookmark.metaDataItem(QStringLiteral("UDI")));
+
     if (bookmark.metaDataItem(QStringLiteral("isSystemItem")) == QLatin1String("true")) {
         // This context must stay as it is - the translated system bookmark names
         // are created with 'KFile System Bookmarks' as their context, so this
@@ -109,20 +101,26 @@ void KFilePlacesItem::setBookmark(const KBookmark &bookmark)
         m_text = bookmark.text();
     }
 
-    const GroupType type = groupType();
+    const KFilePlacesModel::GroupType type = groupType();
     switch (type) {
-    case PlacesType:
+    case KFilePlacesModel::PlacesType:
         m_groupName = i18nc("@item", "Places");
         break;
-    case RecentlySavedType:
+    case KFilePlacesModel::RemoteType:
+        m_groupName = i18nc("@item", "Remote");
+        break;
+    case KFilePlacesModel::RecentlySavedType:
         m_groupName = i18nc("@item", "Recently Saved");
         break;
-    case SearchForType:
+    case KFilePlacesModel::SearchForType:
         m_groupName = i18nc("@item", "Search For");
         break;
-    case DevicesType:
+    case KFilePlacesModel::DevicesType:
         m_groupName = i18nc("@item", "Devices");
         break;
+    case KFilePlacesModel::RemovableDevicesType:
+        m_groupName = i18nc("@item", "Removable Devices");
+        break;
     default:
         Q_UNREACHABLE();
         break;
@@ -131,20 +129,6 @@ void KFilePlacesItem::setBookmark(const KBookmark &bookmark)
 
 Solid::Device KFilePlacesItem::device() const
 {
-    if (m_device.udi().isEmpty()) {
-        m_device = Solid::Device(bookmark().metaDataItem(QStringLiteral("UDI")));
-        if (m_device.isValid()) {
-            m_access = m_device.as<Solid::StorageAccess>();
-            m_volume = m_device.as<Solid::StorageVolume>();
-            m_disc = m_device.as<Solid::OpticalDisc>();
-            m_mtp = m_device.as<Solid::PortableMediaPlayer>();
-        } else {
-            m_access = nullptr;
-            m_volume = nullptr;
-            m_disc = nullptr;
-            m_mtp = nullptr;
-        }
-    }
     return m_device;
 }
 
@@ -160,28 +144,52 @@ QVariant KFilePlacesItem::data(int role) const
     }
 }
 
-KFilePlacesItem::GroupType KFilePlacesItem::groupType() const
+KFilePlacesModel::GroupType KFilePlacesItem::groupType() const
 {
     if (!isDevice()) {
         const QString protocol = bookmark().url().scheme();
         if (protocol == QLatin1String("timeline")) {
-            return RecentlySavedType;
+            return KFilePlacesModel::RecentlySavedType;
         }
 
         if (protocol.contains(QLatin1String("search"))) {
-            return SearchForType;
+            return KFilePlacesModel::SearchForType;
         }
 
         if (protocol == QLatin1String("bluetooth") ||
             protocol == QLatin1String("obexftp") ||
             protocol == QLatin1String("kdeconnect")) {
-            return DevicesType;
+            return KFilePlacesModel::DevicesType;
         }
 
-        return PlacesType;
+        if (protocol == QLatin1String("remote") ||
+            KProtocolInfo::protocolClass(protocol) != QLatin1String(":local")) {
+            return KFilePlacesModel::RemoteType;
+        } else {
+            return KFilePlacesModel::PlacesType;
+        }
     }
 
-    return DevicesType;
+    if (m_drive && (m_drive->isHotpluggable() || m_drive->isRemovable())) {
+        return KFilePlacesModel::RemovableDevicesType;
+    } else if (m_networkShare) {
+        return KFilePlacesModel::RemoteType;
+    } else {
+        return KFilePlacesModel::DevicesType;
+    }
+}
+
+bool KFilePlacesItem::isHidden() const
+{
+    return m_bookmark.metaDataItem(QStringLiteral("IsHidden")) == QLatin1String("true");
+}
+
+void KFilePlacesItem::setHidden(bool hide)
+{
+    if (m_bookmark.isNull() || isHidden() == hide) {
+        return;
+    }
+    m_bookmark.setMetaDataItem(QStringLiteral("IsHidden"), hide ? QStringLiteral("true") : QStringLiteral("false"));
 }
 
 QVariant KFilePlacesItem::bookmarkData(int role) const
@@ -198,7 +206,7 @@ QVariant KFilePlacesItem::bookmarkData(int role) const
     case Qt::DecorationRole:
         return QIcon::fromTheme(iconNameForBookmark(b));
     case Qt::BackgroundRole:
-        if (b.metaDataItem(QStringLiteral("IsHidden")) == QLatin1String("true")) {
+        if (isHidden()) {
             return QColor(Qt::lightGray);
         } else {
             return QVariant();
@@ -208,7 +216,9 @@ QVariant KFilePlacesItem::bookmarkData(int role) const
     case KFilePlacesModel::SetupNeededRole:
         return false;
     case KFilePlacesModel::HiddenRole:
-        return b.metaDataItem(QStringLiteral("IsHidden")) == QLatin1String("true");
+        return isHidden();
+    case KFilePlacesModel::IconNameRole:
+        return iconNameForBookmark(b);
     default:
         return QVariant();
     }
@@ -251,14 +261,8 @@ QVariant KFilePlacesItem::deviceData(int role) const
             }
 
         case KFilePlacesModel::FixedDeviceRole: {
-            Solid::StorageDrive *drive = nullptr;
-            Solid::Device parentDevice = m_device;
-            while (parentDevice.isValid() && !drive) {
-                drive = parentDevice.as<Solid::StorageDrive>();
-                parentDevice = parentDevice.parent();
-            }
-            if (drive != nullptr) {
-                return !drive->isHotpluggable() && !drive->isRemovable();
+            if (m_drive != nullptr) {
+                return !m_drive->isHotpluggable() && !m_drive->isRemovable();
             }
             return true;
         }
@@ -266,6 +270,9 @@ QVariant KFilePlacesItem::deviceData(int role) const
         case KFilePlacesModel::CapacityBarRecommendedRole:
             return m_isAccessible && !m_isCdrom;
 
+        case KFilePlacesModel::IconNameRole:
+            return m_iconPath;
+
         default:
             return QVariant();
         }
@@ -344,6 +351,52 @@ QString KFilePlacesItem::generateNewId()
 //         + '/' + QString::number(qrand());
 }
 
+bool KFilePlacesItem::updateDeviceInfo(const QString &udi)
+{
+    if (m_device.udi() == udi) {
+        return false;
+    }
+
+    if (m_access) {
+        m_access->disconnect(this);
+    }
+
+    m_device = Solid::Device(udi);
+    if (m_device.isValid()) {
+        m_access = m_device.as<Solid::StorageAccess>();
+        m_volume = m_device.as<Solid::StorageVolume>();
+        m_disc = m_device.as<Solid::OpticalDisc>();
+        m_mtp = m_device.as<Solid::PortableMediaPlayer>();
+        m_networkShare = m_device.as<Solid::NetworkShare>();
+        m_iconPath = m_device.icon();
+        m_emblems = m_device.emblems();
+
+        m_drive = nullptr;
+        Solid::Device parentDevice = m_device;
+        while (parentDevice.isValid() && !m_drive) {
+            m_drive = parentDevice.as<Solid::StorageDrive>();
+            parentDevice = parentDevice.parent();
+        }
+
+        if (m_access) {
+            connect(m_access.data(), &Solid::StorageAccess::accessibilityChanged,
+                    this, &KFilePlacesItem::onAccessibilityChanged);
+            onAccessibilityChanged(m_access->isAccessible());
+        }
+    } else {
+        m_access = nullptr;
+        m_volume = nullptr;
+        m_disc = nullptr;
+        m_mtp = nullptr;
+        m_drive = nullptr;
+        m_networkShare = nullptr;
+        m_iconPath.clear();
+        m_emblems.clear();
+    }
+
+    return true;
+}
+
 void KFilePlacesItem::onAccessibilityChanged(bool isAccessible)
 {
     m_isAccessible = isAccessible;
