@@ -32,6 +32,7 @@
 #include <QtCore/QFile>
 #include <QColor>
 #include <QAction>
+#include <QUrlQuery>
 #include <qmimedatabase.h>
 
 #include <kfileitem.h>
@@ -46,6 +47,9 @@
 #include <kio/job.h>
 #include <kprotocolinfo.h>
 
+#include <kconfig.h>
+#include <kconfiggroup.h>
+
 #include <solid/devicenotifier.h>
 #include <solid/storageaccess.h>
 #include <solid/storagedrive.h>
@@ -56,10 +60,123 @@
 #include <solid/predicate.h>
 #include <qstandardpaths.h>
 
-class KFilePlacesModel::Private
+namespace {
+    QString stateNameForGroupType(KFilePlacesModel::GroupType type)
+    {
+        switch (type) {
+        case KFilePlacesModel::PlacesType:
+                return QStringLiteral("GroupState-Places-IsHidden");
+        case KFilePlacesModel::RemoteType:
+                return QStringLiteral("GroupState-Remote-IsHidden");
+        case KFilePlacesModel::RecentlySavedType:
+                return QStringLiteral("GroupState-RecentlySaved-IsHidden");
+        case KFilePlacesModel::SearchForType:
+                return QStringLiteral("GroupState-SearchFor-IsHidden");
+        case KFilePlacesModel::DevicesType:
+                return QStringLiteral("GroupState-Devices-IsHidden");
+        case KFilePlacesModel::RemovableDevicesType:
+                return QStringLiteral("GroupState-RemovableDevices-IsHidden");
+        default:
+            Q_UNREACHABLE();
+        }
+    }
+
+    static bool isFileIndexingEnabled()
+    {
+        KConfig config(QStringLiteral("baloofilerc"));
+        KConfigGroup basicSettings = config.group("Basic Settings");
+        return basicSettings.readEntry("Indexing-Enabled", true);
+    }
+
+    static QString timelineDateString(int year, int month, int day = 0)
+    {
+        const QString dateFormat("%1-%2");
+
+        QString date = dateFormat.arg(year).arg(month, 2, 10, QChar('0'));
+        if (day > 0) {
+            date += QString("-%1").arg(day, 2, 10, QChar('0'));
+        }
+        return date;
+    }
+
+    static QUrl createTimelineUrl(const QUrl &url)
+    {
+        // based on dolphin urls
+        const QString timelinePrefix = QStringLiteral("timeline:") + QLatin1Char('/');
+        QUrl timelineUrl;
+
+        const QString path = url.toDisplayString(QUrl::PreferLocalFile);
+        if (path.endsWith(QLatin1String("/yesterday"))) {
+            const QDate date = QDate::currentDate().addDays(-1);
+            const int year = date.year();
+            const int month = date.month();
+            const int day = date.day();
+
+            timelineUrl = QUrl(timelinePrefix + timelineDateString(year, month) +
+                  '/' + timelineDateString(year, month, day));
+        } else if (path.endsWith(QLatin1String("/thismonth"))) {
+            const QDate date = QDate::currentDate();
+            timelineUrl = QUrl(timelinePrefix + timelineDateString(date.year(), date.month()));
+        } else if (path.endsWith(QLatin1String("/lastmonth"))) {
+            const QDate date = QDate::currentDate().addMonths(-1);
+            timelineUrl = QUrl(timelinePrefix + timelineDateString(date.year(), date.month()));
+        } else {
+            Q_ASSERT(path.endsWith(QLatin1String("/today")));
+            timelineUrl = url;
+        }
+
+        return timelineUrl;
+    }
+
+    static QUrl searchUrlForType(const QString &type)
+    {
+        const QString jsonQuery(QStringLiteral("{\"dayFilter\": 0,\
+                                                 \"monthFilter\": 0, \
+                                                 \"yearFilter\": 0, \
+                                                 \"type\": [ \"%1\"]}"));
+        QUrl url;
+        url.setScheme(QStringLiteral("baloosearch"));
+
+        QUrlQuery urlQuery;
+        urlQuery.addQueryItem(QStringLiteral("json"), jsonQuery.arg(type).simplified());
+        url.setQuery(urlQuery);
+
+        return url;
+    }
+
+    static QUrl createSearchUrl(const QUrl &url)
+    {
+        QUrl searchUrl;
+
+        const QString path = url.toDisplayString(QUrl::PreferLocalFile);
+
+        if (path.endsWith(QLatin1String("/documents"))) {
+            searchUrl = searchUrlForType(QStringLiteral("Document"));
+        } else if (path.endsWith(QLatin1String("/images"))) {
+            searchUrl = searchUrlForType(QStringLiteral("Image"));
+        } else if (path.endsWith(QLatin1String("/audio"))) {
+            searchUrl = searchUrlForType(QStringLiteral("Audio"));
+        } else if (path.endsWith(QLatin1String("/videos"))) {
+            searchUrl = searchUrlForType(QStringLiteral("Video"));
+        } else {
+            qWarning() << "Invalid search url:" << url;
+            searchUrl = url;
+        }
+
+        return searchUrl;
+    }
+}
+
+class Q_DECL_HIDDEN KFilePlacesModel::Private
 {
 public:
-    Private(KFilePlacesModel *self) : q(self), bookmarkManager(0) {}
+    Private(KFilePlacesModel *self)
+        : q(self),
+          bookmarkManager(nullptr),
+          fileIndexingEnabled(isFileIndexingEnabled())
+    {
+    }
+
     ~Private()
     {
         qDeleteAll(items);
@@ -70,12 +187,18 @@ public:
     QList<KFilePlacesItem *> items;
     QVector<QString> availableDevices;
     QMap<QObject *, QPersistentModelIndex> setupInProgress;
+    QStringList supportedSchemes;
 
     Solid::Predicate predicate;
     KBookmarkManager *bookmarkManager;
 
+    const bool fileIndexingEnabled;
+
+    QString alternativeApplicationName;
+
     void reloadAndSignal();
     QList<KFilePlacesItem *> loadBookmarkList();
+    int findNearestPosition(int source, int target);
 
     void _k_initDeviceList();
     void _k_deviceAdded(const QString &udi);
@@ -84,16 +207,25 @@ public:
     void _k_reloadBookmarks();
     void _k_storageSetupDone(Solid::ErrorType error, QVariant errorData);
     void _k_storageTeardownDone(Solid::ErrorType error, QVariant errorData);
+
+private:
+    bool isBalooUrl(const QUrl &url) const;
 };
 
-KFilePlacesModel::KFilePlacesModel(QObject *parent)
+KFilePlacesModel::KFilePlacesModel(const QString &alternativeApplicationName, QObject *parent)
     : QAbstractItemModel(parent), d(new Private(this))
 {
     const QString file = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + "/user-places.xbel";
     d->bookmarkManager = KBookmarkManager::managerForExternalFile(file);
+    d->alternativeApplicationName = alternativeApplicationName;
 
     // Let's put some places in there if it's empty.
     KBookmarkGroup root = d->bookmarkManager->root();
+
+    const auto setDefaultMetadataItemForGroup = [&root](KFilePlacesModel::GroupType type) {
+        root.setMetaDataItem(stateNameForGroupType(type), QStringLiteral("false"));
+    };
+
     if (root.first().isNull() || !QFile::exists(file)) {
 
         // NOTE: The context for these I18N_NOOP2 calls has to be "KFile System Bookmarks".
@@ -105,10 +237,10 @@ KFilePlacesModel::KFilePlacesModel(QObject *parent)
         // (coles, 13th May 2009)
 
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                              QStringLiteral("Home"), I18N_NOOP2("KFile System Bookmarks", QStringLiteral("Home")),
+                                              QStringLiteral("Home"), I18N_NOOP2("KFile System Bookmarks", "Home"),
                                               QUrl::fromLocalFile(QDir::homePath()), QStringLiteral("user-home"));
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                              QStringLiteral("Network"), I18N_NOOP2("KFile System Bookmarks", QStringLiteral("Network")),
+                                              QStringLiteral("Network"), I18N_NOOP2("KFile System Bookmarks", "Network"),
                                               QUrl(QStringLiteral("remote:/")), QStringLiteral("network-workgroup"));
 #if defined(_WIN32_WCE)
         // adding drives
@@ -120,19 +252,60 @@ KFilePlacesModel::KFilePlacesModel(QObject *parent)
         }
 #elif !defined(Q_OS_WIN)
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                              QStringLiteral("Root"), I18N_NOOP2("KFile System Bookmarks", QStringLiteral("Root")),
+                                              QStringLiteral("Root"), I18N_NOOP2("KFile System Bookmarks", "Root"),
                                               QUrl::fromLocalFile(QStringLiteral("/")), QStringLiteral("folder-red"));
 #endif
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                              QStringLiteral("Trash"), I18N_NOOP2("KFile System Bookmarks", QStringLiteral("Trash")),
+                                              QStringLiteral("Trash"), I18N_NOOP2("KFile System Bookmarks", "Trash"),
                                               QUrl(QStringLiteral("trash:/")), QStringLiteral("user-trash"));
 
+        setDefaultMetadataItemForGroup(PlacesType);
+        setDefaultMetadataItemForGroup(RemoteType);
+        setDefaultMetadataItemForGroup(DevicesType);
+        setDefaultMetadataItemForGroup(RemovableDevicesType);
+
         // Force bookmarks to be saved. If on open/save dialog and the bookmarks are not saved, QFile::exists
         // will always return false, which opening/closing all the time the open/save dialog would case the
         // bookmarks to be added once each time, having lots of times each bookmark. (ereslibre)
         d->bookmarkManager->saveAs(file);
     }
 
+    // if baloo is enabled, add new urls even if the bookmark file is not empty
+    if (d->fileIndexingEnabled &&
+        root.metaDataItem(QStringLiteral("withBaloo")) != QLatin1String("true")) {
+
+        root.setMetaDataItem(QStringLiteral("withBaloo"), QStringLiteral("true"));
+        KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
+                                              QStringLiteral("Today"), I18N_NOOP2("KFile System Bookmarks", "Today"),
+                                              QUrl(QStringLiteral("timeline:/today")),  QStringLiteral("go-jump-today"));
+        KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
+                                              QStringLiteral("Yesterday"), I18N_NOOP2("KFile System Bookmarks", "Yesterday"),
+                                              QUrl(QStringLiteral("timeline:/yesterday")),  QStringLiteral("view-calendar-day"));
+        KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
+                                              QStringLiteral("This Month"), I18N_NOOP2("KFile System Bookmarks", "This Month"),
+                                              QUrl(QStringLiteral("timeline:/thismonth")),  QStringLiteral("view-calendar-month"));
+        KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
+                                              QStringLiteral("Last Month"), I18N_NOOP2("KFile System Bookmarks", "Last Month"),
+                                              QUrl(QStringLiteral("timeline:/lastmonth")),  QStringLiteral("view-calendar-month"));
+        KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
+                                              QStringLiteral("Documents"), I18N_NOOP2("KFile System Bookmarks", "Documents"),
+                                             QUrl(QStringLiteral("search:/documents")),  QStringLiteral("folder-text"));
+        KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
+                                              QStringLiteral("Images"), I18N_NOOP2("KFile System Bookmarks", "Images"),
+                                              QUrl(QStringLiteral("search:/images")),  QStringLiteral("folder-images"));
+        KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
+                                              QStringLiteral("Audio Files"), I18N_NOOP2("KFile System Bookmarks", "Audio Files"),
+                                              QUrl(QStringLiteral("search:/audio")),  QStringLiteral("folder-sound"));
+        KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
+                                              QStringLiteral("Videos"), I18N_NOOP2("KFile System Bookmarks", "Videos"),
+                                              QUrl(QStringLiteral("search:/videos")),  QStringLiteral("folder-videos"));
+
+        setDefaultMetadataItemForGroup(SearchForType);
+        setDefaultMetadataItemForGroup(RecentlySavedType);
+
+        d->bookmarkManager->save();
+    }
+
     QString predicate(QString::fromLatin1("[[[[ StorageVolume.ignored == false AND [ StorageVolume.usage == 'FileSystem' OR StorageVolume.usage == 'Encrypted' ]]"
                       " OR "
                       "[ IS StorageAccess AND StorageDrive.driveType == 'Floppy' ]]"
@@ -159,6 +332,11 @@ KFilePlacesModel::KFilePlacesModel(QObject *parent)
     QTimer::singleShot(0, this, SLOT(_k_initDeviceList()));
 }
 
+KFilePlacesModel::KFilePlacesModel(QObject *parent)
+    : KFilePlacesModel({}, parent)
+{
+}
+
 KFilePlacesModel::~KFilePlacesModel()
 {
     delete d;
@@ -186,7 +364,24 @@ QString KFilePlacesModel::text(const QModelIndex &index) const
 
 bool KFilePlacesModel::isHidden(const QModelIndex &index) const
 {
-    return data(index, HiddenRole).toBool();
+    //Note: we do not want to show an index if its parent is hidden
+    return data(index, HiddenRole).toBool() || isGroupHidden(index);
+}
+
+bool KFilePlacesModel::isGroupHidden(const GroupType type) const
+{
+    const QString hidden = d->bookmarkManager->root().metaDataItem(stateNameForGroupType(type));
+    return hidden == QStringLiteral("true") ? true : false;
+}
+
+bool KFilePlacesModel::isGroupHidden(const QModelIndex &index) const
+{
+    if (!index.isValid()) {
+        return false;
+    }
+
+    KFilePlacesItem *item = static_cast<KFilePlacesItem *>(index.internalPointer());
+    return isGroupHidden(item->groupType());
 }
 
 bool KFilePlacesModel::isDevice(const QModelIndex &index) const
@@ -222,12 +417,35 @@ KBookmark KFilePlacesModel::bookmarkForIndex(const QModelIndex &index) const
     }
 
     KFilePlacesItem *item = static_cast<KFilePlacesItem *>(index.internalPointer());
+    return item->bookmark();
+}
 
-    if (!item->isDevice()) {
-        return item->bookmark();
-    } else {
-        return KBookmark();
+KFilePlacesModel::GroupType KFilePlacesModel::groupType(const QModelIndex &index) const
+{
+    if (!index.isValid()) {
+        return UnknownType;
     }
+
+    KFilePlacesItem *item = static_cast<KFilePlacesItem *>(index.internalPointer());
+    return item->groupType();
+}
+
+QModelIndexList KFilePlacesModel::groupIndexes(const KFilePlacesModel::GroupType type) const
+{
+    if (type == UnknownType) {
+        return QModelIndexList();
+    }
+
+    QModelIndexList indexes;
+    const int rows = rowCount();
+    for (int row = 0; row < rows ; ++row) {
+        const QModelIndex current = index(row, 0);
+        if (groupType(current) == type) {
+            indexes << current;
+        }
+    }
+
+    return indexes;
 }
 
 QVariant KFilePlacesModel::data(const QModelIndex &index, int role) const
@@ -237,7 +455,11 @@ QVariant KFilePlacesModel::data(const QModelIndex &index, int role) const
     }
 
     KFilePlacesItem *item = static_cast<KFilePlacesItem *>(index.internalPointer());
-    return item->data(role);
+    if (role == KFilePlacesModel::GroupHiddenRole) {
+        return isGroupHidden(item->groupType());
+    } else {
+        return item->data(role);
+    }
 }
 
 QModelIndex KFilePlacesModel::index(int row, int column, const QModelIndex &parent) const
@@ -422,6 +644,13 @@ void KFilePlacesModel::Private::_k_reloadBookmarks()
     currentItems.clear();
 }
 
+bool KFilePlacesModel::Private::isBalooUrl(const QUrl &url) const
+{
+    const QString scheme = url.scheme();
+    return ((scheme == QLatin1String("timeline")) ||
+            (scheme == QLatin1String("search")));
+}
+
 QList<KFilePlacesItem *> KFilePlacesModel::Private::loadBookmarkList()
 {
     QList<KFilePlacesItem *> items;
@@ -433,15 +662,21 @@ QList<KFilePlacesItem *> KFilePlacesModel::Private::loadBookmarkList()
     while (!bookmark.isNull()) {
         QString udi = bookmark.metaDataItem(QStringLiteral("UDI"));
         QString appName = bookmark.metaDataItem(QStringLiteral("OnlyInApp"));
+        QUrl url = bookmark.url();
         auto it = std::find(devices.begin(), devices.end(), udi);
         bool deviceAvailable = (it != devices.end());
         if (it != devices.end()) {
             devices.erase(it);
         }
 
-        bool allowedHere = appName.isEmpty() || (appName == QCoreApplication::instance()->applicationName());
+        bool allowedHere = appName.isEmpty() ||
+                ((appName == QCoreApplication::instance()->applicationName()) ||
+                 (appName == alternativeApplicationName));
+        bool isSupportedUrl = isBalooUrl(url) ? fileIndexingEnabled : true;
+        bool isSupportedScheme = supportedSchemes.isEmpty() || supportedSchemes.contains(url.scheme());
+
+        if (isSupportedScheme && ((isSupportedUrl && udi.isEmpty() && allowedHere) || deviceAvailable)) {
 
-        if ((udi.isEmpty() && allowedHere) || deviceAvailable) {
             KFilePlacesItem *item;
             if (deviceAvailable) {
                 item = new KFilePlacesItem(bookmarkManager, bookmark.address(), udi);
@@ -470,9 +705,50 @@ QList<KFilePlacesItem *> KFilePlacesModel::Private::loadBookmarkList()
         }
     }
 
+    // return a sorted list based on groups
+    qStableSort(items.begin(), items.end(),
+                [](KFilePlacesItem *itemA, KFilePlacesItem *itemB) {
+       return (itemA->groupType() < itemB->groupType());
+    });
+
     return items;
 }
 
+int KFilePlacesModel::Private::findNearestPosition(int source, int target)
+{
+    const KFilePlacesItem *item = items.at(source);
+    const KFilePlacesModel::GroupType groupType = item->groupType();
+    int newTarget = qMin(target, items.count() - 1);
+
+    // moving inside the same group is ok
+    if ((items.at(newTarget)->groupType() == groupType)) {
+        return target;
+    }
+
+    if (target > source) { // moving down, move it to the end of the group
+        int groupFooter = source;
+        while (items.at(groupFooter)->groupType() == groupType) {
+            groupFooter++;
+            // end of the list move it there
+            if (groupFooter == items.count()) {
+                break;
+            }
+        }
+        target = groupFooter;
+    } else { // moving up, move it to beginning of the group
+        int groupHead = source;
+        while (items.at(groupHead)->groupType() == groupType) {
+            groupHead--;
+            // beginning of the list move it there
+            if (groupHead == 0) {
+                break;
+            }
+        }
+        target = groupHead;
+    }
+    return target;
+}
+
 void KFilePlacesModel::Private::reloadAndSignal()
 {
     bookmarkManager->emitChanged(bookmarkManager->root()); // ... we'll get relisted anyway
@@ -556,24 +832,6 @@ bool KFilePlacesModel::dropMimeData(const QMimeData *data, Qt::DropAction action
         // let's do it in the views to get the good old drop menu
     }
 
-    QMimeDatabase db;
-    KBookmark afterBookmark;
-
-    if (row == -1) {
-        // The dropped item is moved or added to the last position
-
-        KFilePlacesItem *lastItem = d->items.last();
-        afterBookmark = lastItem->bookmark();
-
-    } else {
-        // The dropped item is moved or added before position 'row', ie after position 'row-1'
-
-        if (row > 0) {
-            KFilePlacesItem *afterItem = d->items[row - 1];
-            afterBookmark = afterItem->bookmark();
-        }
-    }
-
     if (data->hasFormat(_k_internalMimetype(this))) {
         // The operation is an internal move
         QByteArray itemData = data->data(_k_internalMimetype(this));
@@ -582,28 +840,31 @@ bool KFilePlacesModel::dropMimeData(const QMimeData *data, Qt::DropAction action
 
         stream >> itemRow;
 
-        KFilePlacesItem *item = d->items[itemRow];
-        KBookmark bookmark = item->bookmark();
-
-        int destRow = row == -1 ? d->items.count() : row;
-        // The item is not moved when the drop indicator is on either item edge
-        if (itemRow == destRow || itemRow + 1 == destRow) {
+        if (!movePlace(itemRow, row)) {
             return false;
         }
 
-        beginMoveRows(QModelIndex(), itemRow, itemRow, QModelIndex(), destRow);
-        d->bookmarkManager->root().moveBookmark(bookmark, afterBookmark);
-        // Move item ourselves so that _k_reloadBookmarks() does not consider
-        // the move as a remove + insert.
-        //
-        // 2nd argument of QList::move() expects the final destination index,
-        // but 'row' is the value of the destination index before the moved
-        // item has been removed from its original position. That is why we
-        // adjust if necessary.
-        d->items.move(itemRow, itemRow < destRow ? (destRow - 1) : destRow);
-        endMoveRows();
     } else if (data->hasFormat(QStringLiteral("text/uri-list"))) {
         // The operation is an add
+
+        QMimeDatabase db;
+        KBookmark afterBookmark;
+
+        if (row == -1) {
+            // The dropped item is moved or added to the last position
+
+            KFilePlacesItem *lastItem = d->items.last();
+            afterBookmark = lastItem->bookmark();
+
+        } else {
+            // The dropped item is moved or added before position 'row', ie after position 'row-1'
+
+            if (row > 0) {
+                KFilePlacesItem *afterItem = d->items[row - 1];
+                afterBookmark = afterItem->bookmark();
+            }
+        }
+
         const QList<QUrl> urls = KUrlMimeData::urlsFromMimeData(data);
 
         KBookmarkGroup group = d->bookmarkManager->root();
@@ -646,11 +907,28 @@ bool KFilePlacesModel::dropMimeData(const QMimeData *data, Qt::DropAction action
         return false;
     }
 
-    d->reloadAndSignal();
+    refresh();
 
     return true;
 }
 
+void KFilePlacesModel::refresh() const
+{
+    d->reloadAndSignal();
+}
+
+QUrl KFilePlacesModel::convertedUrl(const QUrl &url)
+{
+    QUrl newUrl = url;
+    if (url.scheme() == QLatin1String("timeline")) {
+        newUrl = createTimelineUrl(url);
+    } else if (url.scheme() == QLatin1String("search")) {
+        newUrl = createSearchUrl(url);
+    }
+
+    return newUrl;
+}
+
 void KFilePlacesModel::addPlace(const QString &text, const QUrl &url,
                                 const QString &iconName, const QString &appName)
 {
@@ -673,7 +951,7 @@ void KFilePlacesModel::addPlace(const QString &text, const QUrl &url,
         d->bookmarkManager->root().moveBookmark(bookmark, item->bookmark());
     }
 
-    d->reloadAndSignal();
+    refresh();
 }
 
 void KFilePlacesModel::editPlace(const QModelIndex &index, const QString &text, const QUrl &url,
@@ -695,13 +973,32 @@ void KFilePlacesModel::editPlace(const QModelIndex &index, const QString &text,
         return;
     }
 
-    bookmark.setFullText(text);
-    bookmark.setUrl(url);
-    bookmark.setIcon(iconName);
-    bookmark.setMetaDataItem(QStringLiteral("OnlyInApp"), appName);
+    bool changed = false;
+    if (text != bookmark.fullText()) {
+        bookmark.setFullText(text);
+        changed = true;
+    }
 
-    d->reloadAndSignal();
-    emit dataChanged(index, index);
+    if (url != bookmark.url()) {
+        bookmark.setUrl(url);
+        changed = true;
+    }
+
+    if (iconName != bookmark.icon()) {
+        bookmark.setIcon(iconName);
+        changed = true;
+    }
+
+    const QString onlyInApp = bookmark.metaDataItem(QStringLiteral("OnlyInApp"));
+    if (appName != onlyInApp) {
+        bookmark.setMetaDataItem(QStringLiteral("OnlyInApp"), appName);
+        changed = true;
+    }
+
+    if (changed) {
+        refresh();
+        emit dataChanged(index, index);
+    }
 }
 
 void KFilePlacesModel::removePlace(const QModelIndex &index) const
@@ -723,7 +1020,7 @@ void KFilePlacesModel::removePlace(const QModelIndex &index) const
     }
 
     d->bookmarkManager->root().deleteBookmark(bookmark);
-    d->reloadAndSignal();
+    refresh();
 }
 
 void KFilePlacesModel::setPlaceHidden(const QModelIndex &index, bool hidden)
@@ -734,16 +1031,85 @@ void KFilePlacesModel::setPlaceHidden(const QModelIndex &index, bool hidden)
 
     KFilePlacesItem *item = static_cast<KFilePlacesItem *>(index.internalPointer());
 
-    KBookmark bookmark = item->bookmark();
-
-    if (bookmark.isNull()) {
+    if (item->bookmark().isNull() || item->isHidden() == hidden) {
         return;
     }
 
-    bookmark.setMetaDataItem(QStringLiteral("IsHidden"), (hidden ? QStringLiteral("true") : QStringLiteral("false")));
+    const bool groupHidden = isGroupHidden(item->groupType());
+    const bool hidingChildOnShownParent = hidden && !groupHidden;
+    const bool showingChildOnShownParent = !hidden && !groupHidden;
 
+    if (hidingChildOnShownParent || showingChildOnShownParent) {
+        item->setHidden(hidden);
+
+        d->reloadAndSignal();
+        emit dataChanged(index, index);
+    }
+}
+
+void KFilePlacesModel::setGroupHidden(const GroupType type, bool hidden)
+{
+    if (isGroupHidden(type) == hidden)
+        return;
+
+    d->bookmarkManager->root().setMetaDataItem(stateNameForGroupType(type), (hidden ? QStringLiteral("true") : QStringLiteral("false")));    
     d->reloadAndSignal();
-    emit dataChanged(index, index);
+    emit groupHiddenChanged(type, hidden);
+}
+
+bool KFilePlacesModel::movePlace(int itemRow, int row)
+{
+    KBookmark afterBookmark;
+
+    if ((itemRow < 0) || (itemRow >= d->items.count())) {
+        return false;
+    }
+
+    if (row >= d->items.count()) {
+        row = -1;
+    }
+
+    if (row == -1) {
+        // The dropped item is moved or added to the last position
+
+        KFilePlacesItem *lastItem = d->items.last();
+        afterBookmark = lastItem->bookmark();
+
+    } else {
+        // The dropped item is moved or added before position 'row', ie after position 'row-1'
+
+        if (row > 0) {
+            KFilePlacesItem *afterItem = d->items[row - 1];
+            afterBookmark = afterItem->bookmark();
+        }
+    }
+
+    KFilePlacesItem *item = d->items[itemRow];
+    KBookmark bookmark = item->bookmark();
+
+    int destRow = row == -1 ? d->items.count() : row;
+
+    // avoid move item away from its group
+    destRow = d->findNearestPosition(itemRow, destRow);
+
+    // The item is not moved when the drop indicator is on either item edge
+    if (itemRow == destRow || itemRow + 1 == destRow) {
+        return false;
+    }
+
+    beginMoveRows(QModelIndex(), itemRow, itemRow, QModelIndex(), destRow);
+    d->bookmarkManager->root().moveBookmark(bookmark, afterBookmark);
+    // Move item ourselves so that _k_reloadBookmarks() does not consider
+    // the move as a remove + insert.
+    //
+    // 2nd argument of QList::move() expects the final destination index,
+    // but 'row' is the value of the destination index before the moved
+    // item has been removed from its original position. That is why we
+    // adjust if necessary.
+    d->items.move(itemRow, itemRow < destRow ? (destRow - 1) : destRow);
+    endMoveRows();
+
+    return true;
 }
 
 int KFilePlacesModel::hiddenCount() const
@@ -768,14 +1134,14 @@ QAction *KFilePlacesModel::teardownActionForIndex(const QModelIndex &index) cons
 
         Solid::StorageDrive *drive = device.as<Solid::StorageDrive>();
 
-        if (drive == 0) {
+        if (drive == nullptr) {
             drive = device.parent().as<Solid::StorageDrive>();
         }
 
         bool hotpluggable = false;
         bool removable = false;
 
-        if (drive != 0) {
+        if (drive != nullptr) {
             hotpluggable = drive->isHotpluggable();
             removable = drive->isRemovable();
         }
@@ -795,13 +1161,13 @@ QAction *KFilePlacesModel::teardownActionForIndex(const QModelIndex &index) cons
         }
 
         if (!iconName.isEmpty()) {
-            return new QAction(QIcon::fromTheme(iconName), text, 0);
+            return new QAction(QIcon::fromTheme(iconName), text, nullptr);
         } else {
-            return new QAction(text, 0);
+            return new QAction(text, nullptr);
         }
     }
 
-    return 0;
+    return nullptr;
 }
 
 QAction *KFilePlacesModel::ejectActionForIndex(const QModelIndex &index) const
@@ -813,10 +1179,10 @@ QAction *KFilePlacesModel::ejectActionForIndex(const QModelIndex &index) const
         QString label = data(index, Qt::DisplayRole).toString().replace('&', QLatin1String("&&"));
         QString text = i18n("&Eject '%1'", label);
 
-        return new QAction(QIcon::fromTheme(QStringLiteral("media-eject")), text, 0);
+        return new QAction(QIcon::fromTheme(QStringLiteral("media-eject")), text, nullptr);
     }
 
-    return 0;
+    return nullptr;
 }
 
 void KFilePlacesModel::requestTeardown(const QModelIndex &index)
@@ -824,7 +1190,7 @@ void KFilePlacesModel::requestTeardown(const QModelIndex &index)
     Solid::Device device = deviceForIndex(index);
     Solid::StorageAccess *access = device.as<Solid::StorageAccess>();
 
-    if (access != 0) {
+    if (access != nullptr) {
         connect(access, SIGNAL(teardownDone(Solid::ErrorType,QVariant,QString)),
                 this, SLOT(_k_storageTeardownDone(Solid::ErrorType,QVariant)));
 
@@ -838,7 +1204,7 @@ void KFilePlacesModel::requestEject(const QModelIndex &index)
 
     Solid::OpticalDrive *drive = device.parent().as<Solid::OpticalDrive>();
 
-    if (drive != 0) {
+    if (drive != nullptr) {
         connect(drive, SIGNAL(ejectDone(Solid::ErrorType,QVariant,QString)),
                 this, SLOT(_k_storageTeardownDone(Solid::ErrorType,QVariant)));
 
@@ -900,4 +1266,15 @@ void KFilePlacesModel::Private::_k_storageTeardownDone(Solid::ErrorType error, Q
     }
 }
 
+void KFilePlacesModel::setSupportedSchemes(const QStringList &schemes)
+{
+    d->supportedSchemes = schemes;
+    d->_k_reloadBookmarks();
+}
+
+QStringList KFilePlacesModel::supportedSchemes() const
+{
+    return d->supportedSchemes;
+}
+
 #include "moc_kfileplacesmodel.cpp"
