@@ -47,10 +47,33 @@ public:
         HiddenRole = 0x0741CAAC,
         SetupNeededRole = 0x059A935D,
         FixedDeviceRole = 0x332896C1,
-        CapacityBarRecommendedRole = 0x1548C5C4
+        CapacityBarRecommendedRole = 0x1548C5C4,
+        GroupRole = 0x0a5b64ee,
+        /// @since 5.41
+        IconNameRole = 0x00a45c00,
+        GroupHiddenRole = 0x21a4b936
+    };
+
+    /// @since 5.42
+    enum GroupType {
+        PlacesType,
+        RemoteType,
+        RecentlySavedType,
+        SearchForType,
+        DevicesType,
+        RemovableDevicesType,
+        UnknownType
     };
 
     KFilePlacesModel(QObject *parent = nullptr);
+    /**
+    * @brief Construct a new KFilePlacesModel with an alternativeApplicationName
+    * @param alternativeApplicationName This value will be used to filter bookmarks in addition to the actual application name
+    * @param parent Parent object
+    * @since 5.43
+    * @todo kf6: merge contstructors
+    */
+    KFilePlacesModel(const QString &alternativeApplicationName, QObject *parent = nullptr);
     ~KFilePlacesModel();
 
     QUrl url(const QModelIndex &index) const;
@@ -58,9 +81,16 @@ public:
     QIcon icon(const QModelIndex &index) const;
     QString text(const QModelIndex &index) const;
     bool isHidden(const QModelIndex &index) const;
+    /// @since 5.42
+    bool isGroupHidden(const GroupType type) const;
+    /// @since 5.42
+    bool isGroupHidden(const QModelIndex &index) const;
     bool isDevice(const QModelIndex &index) const;
     Solid::Device deviceForIndex(const QModelIndex &index) const;
     KBookmark bookmarkForIndex(const QModelIndex &index) const;
+    /// @since 5.42
+    GroupType groupType(const QModelIndex &index) const;
+    QModelIndexList groupIndexes(const GroupType type) const;
 
     QAction *teardownActionForIndex(const QModelIndex &index) const;
     QAction *ejectActionForIndex(const QModelIndex &index) const;
@@ -73,6 +103,14 @@ public:
     void editPlace(const QModelIndex &index, const QString &text, const QUrl &url, const QString &iconName = QString(), const QString &appName = QString());
     void removePlace(const QModelIndex &index) const;
     void setPlaceHidden(const QModelIndex &index, bool hidden);
+    /// @since 5.42
+    void setGroupHidden(const GroupType type, bool hidden);
+
+    /**
+     * @brief Move place at @p itemRow to a position before @p row
+     * @since 5.41
+     */
+    bool movePlace(int itemRow, int row);
 
     int hiddenCount() const;
 
@@ -130,9 +168,49 @@ public:
     bool dropMimeData(const QMimeData *data, Qt::DropAction action,
                       int row, int column, const QModelIndex &parent) Q_DECL_OVERRIDE;
 
+    /**
+     * @brief Reload bookmark information
+     * @since 5.41
+     */
+    void refresh() const;
+
+    /**
+     * @brief  Converts the URL, which contains "virtual" URLs for system-items like
+     *         "timeline:/lastmonth" into a Query-URL "timeline:/2017-10"
+     *         that will be handled by the corresponding IO-slave.
+     *         Virtual URLs for bookmarks are used to be independent from
+     *         internal format changes.
+     * @param an url
+     * @return the converted URL, which can be handled by an ioslave
+     * @since 5.41
+     */
+    static QUrl convertedUrl(const QUrl &url);
+
+    /**
+     * Set the URL schemes that the file widget should allow navigating to.
+     *
+     * If the returned list is empty, all schemes are supported. Examples for
+     * schemes are @c "file" or @c "ftp".
+     *
+     * @sa QFileDialog::setSupportedSchemes
+     * @since 5.43
+     */
+    void setSupportedSchemes(const QStringList &schemes);
+
+    /**
+     * Returns the URL schemes that the file widget should allow navigating to.
+     *
+     * If the returned list is empty, all schemes are supported.
+     *
+     * @sa QFileDialog::supportedSchemes
+     * @since 5.43
+     */
+    QStringList supportedSchemes() const;
+
 Q_SIGNALS:
     void errorMessage(const QString &message);
     void setupDone(const QModelIndex &index, bool success);
+    void groupHiddenChanged(KFilePlacesModel::GroupType group, bool hidden);
 
 private:
     Q_PRIVATE_SLOT(d, void _k_initDeviceList())
