@@ -70,10 +70,7 @@
  *
  * QListView does not invoke QItemDelegate::sizeHint() when the
  * uniformItemSize property has been set to true, so this property is
- * set before exchanging a block of icons. It is important to reset
- * it again before the event loop is entered, otherwise QListView
- * would not get the correct size hints after dispatching the layoutChanged()
- * signal.
+ * set before exchanging a block of icons.
  */
 class KFilePreviewGenerator::LayoutBlocker
 {
@@ -82,7 +79,7 @@ public:
         m_uniformSizes(false),
         m_view(qobject_cast<QListView *>(view))
     {
-        if (m_view != 0) {
+        if (m_view != nullptr) {
             m_uniformSizes = m_view->uniformItemSizes();
             m_view->setUniformItemSizes(true);
         }
@@ -90,8 +87,13 @@ public:
 
     ~LayoutBlocker()
     {
-        if (m_view != 0) {
+        if (m_view != nullptr) {
             m_view->setUniformItemSizes(m_uniformSizes);
+            /* The QListView did the layout with uniform item
+             * sizes, so trigger a relayout with the expected sizes. */
+            if (!m_uniformSizes) {
+                m_view->setGridSize(m_view->gridSize());
+            }
         }
     }
 
@@ -160,7 +162,7 @@ private:
     QPixmap m_tiles[NumTiles];
 };
 
-class KFilePreviewGenerator::Private
+class Q_DECL_HIDDEN KFilePreviewGenerator::Private
 {
 public:
     Private(KFilePreviewGenerator *parent,
@@ -419,21 +421,21 @@ KFilePreviewGenerator::Private::Private(KFilePreviewGenerator *parent,
     m_internalDataChange(0),
     m_pendingVisibleIconUpdates(0),
     m_viewAdapter(viewAdapter),
-    m_itemView(0),
-    m_iconUpdateTimer(0),
-    m_scrollAreaTimer(0),
+    m_itemView(nullptr),
+    m_iconUpdateTimer(nullptr),
+    m_scrollAreaTimer(nullptr),
     m_previewJobs(),
-    m_proxyModel(0),
+    m_proxyModel(nullptr),
     m_cutItemsCache(),
     m_previews(),
     m_sequenceIndices(),
     m_changedItems(),
-    m_changedItemsTimer(0),
+    m_changedItemsTimer(nullptr),
     m_pendingItems(),
     m_dispatchedItems(),
     m_resolvedMimeTypes(),
     m_enabledPlugins(),
-    m_tileSet(0),
+    m_tileSet(nullptr),
     q(parent)
 {
     if (!m_viewAdapter->iconSize().isValid()) {
@@ -441,7 +443,7 @@ KFilePreviewGenerator::Private::Private(KFilePreviewGenerator *parent,
     }
 
     m_proxyModel = qobject_cast<QAbstractProxyModel *>(model);
-    m_dirModel = (m_proxyModel == 0) ?
+    m_dirModel = (m_proxyModel == nullptr) ?
                  qobject_cast<KDirModel *>(model) :
                  qobject_cast<KDirModel *>(m_proxyModel->sourceModel());
     if (!m_dirModel) {
@@ -606,8 +608,8 @@ void KFilePreviewGenerator::Private::updateIcons(const QModelIndex &topLeft,
 void KFilePreviewGenerator::Private::addToPreviewQueue(const KFileItem &item, const QPixmap &pixmap)
 {
     KIO::PreviewJob *senderJob = qobject_cast<KIO::PreviewJob *>(q->sender());
-    Q_ASSERT(senderJob != 0);
-    if (senderJob != 0) {
+    Q_ASSERT(senderJob != nullptr);
+    if (senderJob != nullptr) {
         QMap<QUrl, int>::iterator it = m_sequenceIndices.find(item.url());
         if (senderJob->sequenceIndex() && (it == m_sequenceIndices.end() || *it != senderJob->sequenceIndex())) {
             return; // the sequence index does not match the one we want
@@ -794,7 +796,7 @@ void KFilePreviewGenerator::Private::pauseIconUpdates()
 {
     m_iconUpdatesPaused = true;
     foreach (KJob *job, m_previewJobs) {
-        Q_ASSERT(job != 0);
+        Q_ASSERT(job != nullptr);
         job->suspend();
     }
     m_scrollAreaTimer->start();
@@ -958,7 +960,7 @@ bool KFilePreviewGenerator::Private::applyImageFrame(QPixmap &icon)
                      maxSize.height() - TileSet::TopMargin - TileSet::BottomMargin);
     limitToSize(icon, size);
 
-    if (m_tileSet == 0) {
+    if (m_tileSet == nullptr) {
         m_tileSet = new TileSet();
     }
 
@@ -1082,7 +1084,7 @@ void KFilePreviewGenerator::Private::startPreviewJob(const KFileItemList &items,
 void KFilePreviewGenerator::Private::killPreviewJobs()
 {
     foreach (KJob *job, m_previewJobs) {
-        Q_ASSERT(job != 0);
+        Q_ASSERT(job != nullptr);
         job->kill();
     }
     m_previewJobs.clear();
@@ -1102,7 +1104,7 @@ void KFilePreviewGenerator::Private::orderItems(KFileItemList &items)
 
     // Order the items in a way that the preview for the visible items
     // is generated first, as this improves the feeled performance a lot.
-    const bool hasProxy = (m_proxyModel != 0);
+    const bool hasProxy = (m_proxyModel != nullptr);
     const int itemCount = items.count();
     const QRect visibleArea = m_viewAdapter->visibleArea();
 
@@ -1236,30 +1238,7 @@ void KFilePreviewGenerator::setPreviewShown(bool show)
 
     d->m_previewShown = show;
     if (!show) {
-        // Clear the icon for all items so that the MIME type
-        // gets reloaded
-        KFileItemList itemList;
-        d->addItemsToList(QModelIndex(), itemList);
-
-        const bool blocked = dirModel->signalsBlocked();
-        dirModel->blockSignals(true);
-
-        QList<QModelIndex> indexesWithKnownMimeType;
-        foreach (const KFileItem &item, itemList) {
-            const QModelIndex index = dirModel->indexForItem(item);
-            if (item.isMimeTypeKnown()) {
-                indexesWithKnownMimeType.append(index);
-            }
-            dirModel->setData(index, QIcon(), Qt::DecorationRole);
-        }
-
-        dirModel->blockSignals(blocked);
-
-        // Items without a known mimetype will be handled (delayed) by updateIcons.
-        // So we need to update items with a known mimetype ourselves.
-        foreach (const QModelIndex &index, indexesWithKnownMimeType) {
-            dirModel->itemChanged(index);
-        }
+        dirModel->clearAllPreviews();
     }
     updateIcons();
 }
