@@ -63,7 +63,7 @@ struct LocationData {
     QByteArray state;
 };
 
-class KUrlNavigator::Private
+class Q_DECL_HIDDEN KUrlNavigator::Private
 {
 public:
     Private(KUrlNavigator *q, KFilePlacesModel *placesModel);
@@ -222,6 +222,8 @@ KUrlNavigator::Private::Private(KUrlNavigator *q, KFilePlacesModel *placesModel)
         m_placesSelector = new KUrlNavigatorPlacesSelector(q, placesModel);
         connect(m_placesSelector, SIGNAL(placeActivated(QUrl)),
                 q, SLOT(setLocationUrl(QUrl)));
+        connect(m_placesSelector, &KUrlNavigatorPlacesSelector::tabRequested,
+                q, &KUrlNavigator::tabRequested);
 
         connect(placesModel, SIGNAL(rowsInserted(QModelIndex,int,int)),
                 q, SLOT(updateContent()));
@@ -344,7 +346,13 @@ void KUrlNavigator::Private::slotProtocolChanged(const QString &protocol)
 
     QUrl url;
     url.setScheme(protocol);
-    url.setPath((protocol == QLatin1String("file")) ? QStringLiteral("/") : QStringLiteral("//"));
+    if (protocol == QLatin1String("file")) {
+        url.setPath(QStringLiteral("/"));
+    } else {
+        // With no authority set we'll get e.g. "ftp:" instead of "ftp://".
+        // We want the latter, so let's set an empty authority.
+        url.setAuthority(QStringLiteral(""));
+    }
 
     m_pathBox->setEditUrl(url);
 }
