@@ -23,6 +23,7 @@
 #include "kurlnavigator.h"
 #include "kurlnavigatormenu_p.h"
 #include "kdirsortfilterproxymodel.h"
+#include "../pathhelpers_p.h"
 
 #include <kio/job.h>
 #include <klocalizedstring.h>
@@ -438,7 +439,7 @@ void KUrlNavigatorButton::urlsDropped(QAction *action, QDropEvent *event)
 {
     const int result = action->data().toInt();
     QUrl url(m_url);
-    url.setPath(url.path() + '/' + m_subDirs.at(result).first);
+    url.setPath(concatPaths(url.path(), m_subDirs.at(result).first));
     urlsDropped(url, event);
 }
 
@@ -446,7 +447,7 @@ void KUrlNavigatorButton::slotMenuActionClicked(QAction *action, Qt::MouseButton
 {
     const int result = action->data().toInt();
     QUrl url(m_url);
-    url.setPath(url.path() + '/' + m_subDirs.at(result).first);
+    url.setPath(concatPaths(url.path(), m_subDirs.at(result).first));
     emit clicked(url, button);
 }
 
@@ -498,7 +499,7 @@ void KUrlNavigatorButton::openSubDirsMenu(KJob *job)
     }
 
     NaturalLessThan nlt;
-    qSort(m_subDirs.begin(), m_subDirs.end(), nlt);
+    std::sort(m_subDirs.begin(), m_subDirs.end(), nlt);
     setDisplayHintEnabled(PopupActiveHint, true);
     update(); // ensure the button is drawn highlighted
 
@@ -543,7 +544,7 @@ void KUrlNavigatorButton::replaceButton(KJob *job)
     }
 
     NaturalLessThan nlt;
-    qSort(m_subDirs.begin(), m_subDirs.end(), nlt);
+    std::sort(m_subDirs.begin(), m_subDirs.end(), nlt);
 
     // Get index of the directory that is shown currently in the button
     const QString currentDir = m_url.fileName();
@@ -566,7 +567,7 @@ void KUrlNavigatorButton::replaceButton(KJob *job)
     }
 
     QUrl url(KIO::upUrl(m_url));
-    url.setPath(url.path() + '/' + m_subDirs[targetIndex].first);
+    url.setPath(concatPaths(url.path(), m_subDirs[targetIndex].first));
     emit clicked(url, Qt::LeftButton);
 
     m_subDirs.clear();
