@@ -154,7 +154,7 @@ static QFile::Permissions modeToQFilePermissions(int mode)
 }
 
 FileProtocol::FileProtocol(const QByteArray &pool, const QByteArray &app)
-    : SlaveBase(QByteArrayLiteral("file"), pool, app), mFile(0)
+    : SlaveBase(QByteArrayLiteral("file"), pool, app), mFile(nullptr)
 {
 }
 
@@ -182,7 +182,7 @@ int FileProtocol::setACL(const char *path, mode_t perm, bool directoryDefault)
     const QString defaultACLString = metaData(QStringLiteral("DEFAULT_ACL_STRING"));
     // Empty strings mean leave as is
     if (!ACLString.isEmpty()) {
-        acl_t acl = 0;
+        acl_t acl = nullptr;
         if (ACLString == QLatin1String("ACL_DELETE")) {
             // user told us to delete the extended ACL, so let's write only
             // the minimal (UNIX permission bits) part
@@ -225,30 +225,39 @@ void FileProtocol::chmod(const QUrl &url, int permissions)
     const QString path(url.toLocalFile());
     const QByteArray _path(QFile::encodeName(path));
     /* FIXME: Should be atomic */
+#ifdef Q_OS_UNIX
+    // QFile::Permissions does not support special attributes like sticky
+    if (::chmod(_path.constData(), permissions) == -1 ||
+#else
     if (!QFile::setPermissions(path, modeToQFilePermissions(permissions)) ||
+#endif
             (setACL(_path.data(), permissions, false) == -1) ||
             /* if not a directory, cannot set default ACLs */
             (setACL(_path.data(), permissions, true) == -1 && errno != ENOTDIR)) {
-
-        switch (errno) {
-        case EPERM:
-        case EACCES:
-            error(KIO::ERR_ACCESS_DENIED, path);
-            break;
-#if defined(ENOTSUP)
-        case ENOTSUP: // from setACL since chmod can't return ENOTSUP
-            error(KIO::ERR_UNSUPPORTED_ACTION, i18n("Setting ACL for %1", path));
-            break;
-#endif
-        case ENOSPC:
-            error(KIO::ERR_DISK_FULL, path);
-            break;
-        default:
-            error(KIO::ERR_CANNOT_CHMOD, path);
+        if (auto err = execWithElevatedPrivilege(CHMOD, {_path, permissions}, errno)) {
+            if (!err.wasCanceled()) {
+                switch (err) {
+                case EPERM:
+                case EACCES:
+                    error(KIO::ERR_ACCESS_DENIED, path);
+                    break;
+        #if defined(ENOTSUP)
+                case ENOTSUP: // from setACL since chmod can't return ENOTSUP
+                    error(KIO::ERR_UNSUPPORTED_ACTION, i18n("Setting ACL for %1", path));
+                    break;
+        #endif
+                case ENOSPC:
+                    error(KIO::ERR_DISK_FULL, path);
+                    break;
+                default:
+                    error(KIO::ERR_CANNOT_CHMOD, path);
+                }
+                return;
+            }
         }
-    } else {
-        finished();
     }
+
+    finished();
 }
 
 void FileProtocol::setModificationTime(const QUrl &url, const QDateTime &mtime)
@@ -260,8 +269,12 @@ void FileProtocol::setModificationTime(const QUrl &url, const QDateTime &mtime)
         utbuf.actime = statbuf.st_atime; // access time, unchanged
         utbuf.modtime = mtime.toTime_t(); // modification time
         if (::utime(QFile::encodeName(path).constData(), &utbuf) != 0) {
-            // TODO: errno could be EACCES, EPERM, EROFS
-            error(KIO::ERR_CANNOT_SETTIME, path);
+            if (auto err = execWithElevatedPrivilege(UTIME, {path, qint64(utbuf.actime), qint64(utbuf.modtime)}, errno)) {
+                if (!err.wasCanceled()) {
+                    // TODO: errno could be EACCES, EPERM, EROFS
+                    error(KIO::ERR_CANNOT_SETTIME, path);
+                }
+            }
         } else {
             finished();
         }
@@ -278,16 +291,26 @@ void FileProtocol::mkdir(const QUrl &url, int permissions)
 
     // Remove existing file or symlink, if requested (#151851)
     if (metaData(QStringLiteral("overwrite")) == QLatin1String("true")) {
-        QFile::remove(path);
+        if (!QFile::remove(path)) {
+            execWithElevatedPrivilege(DEL, {path}, errno);
+        }
     }
 
     QT_STATBUF buff;
     if (QT_LSTAT(QFile::encodeName(path).constData(), &buff) == -1) {
-        if (!QDir().mkdir(path)) {
-            //TODO: add access denied & disk full (or another reasons) handling (into Qt, possibly)
-            error(KIO::ERR_CANNOT_MKDIR, path);
-            return;
-        } else {
+        bool dirCreated = QDir().mkdir(path);
+        if (!dirCreated) {
+            if (auto err = execWithElevatedPrivilege(MKDIR, {path}, errno)) {
+                if (!err.wasCanceled()) {
+                    //TODO: add access denied & disk full (or another reasons) handling (into Qt, possibly)
+                    error(KIO::ERR_CANNOT_MKDIR, path);
+                }
+                return;
+            }
+            dirCreated = true;
+        }
+
+        if (dirCreated) {
             if (permissions != -1) {
                 chmod(url, permissions);
             } else {
@@ -338,8 +361,12 @@ void FileProtocol::get(const QUrl &url)
 
     QFile f(path);
     if (!f.open(QIODevice::ReadOnly)) {
-        error(KIO::ERR_CANNOT_OPEN_FOR_READING, path);
-        return;
+        if (auto err = tryOpen(f, QFile::encodeName(path), O_RDONLY, S_IRUSR, errno)) {
+            if (!err.wasCanceled()) {
+                error(KIO::ERR_CANNOT_OPEN_FOR_READING, path);
+            }
+            return;
+        }
     }
 
 #if HAVE_FADVISE
@@ -526,13 +553,18 @@ void FileProtocol::close()
     Q_ASSERT(mFile);
 
     delete mFile;
-    mFile = 0;
+    mFile = nullptr;
 
     finished();
 }
 
 void FileProtocol::put(const QUrl &url, int _mode, KIO::JobFlags _flags)
 {
+    if (privilegeOperationUnitTestMode()) {
+        finished();
+        return;
+    }
+
     const QString dest_orig = url.toLocalFile();
 
     // qDebug() << dest_orig << "mode=" << _mode;
@@ -614,15 +646,38 @@ void FileProtocol::put(const QUrl &url, int _mode, KIO::JobFlags _flags)
                 }
 
                 if (!f.isOpen()) {
-                    // qDebug() << "####################### COULD NOT WRITE" << dest << "_mode=" << _mode;
-                    // qDebug() << "QFile error==" << f.error() << "(" << f.errorString() << ")";
+                    int oflags = 0;
+                    int filemode = _mode;
 
-                    if (f.error() == QFileDevice::PermissionsError) {
-                        error(KIO::ERR_WRITE_ACCESS_DENIED, dest);
+                    if ((_flags & KIO::Resume)) {
+                        oflags = O_RDWR | O_APPEND;
                     } else {
-                        error(KIO::ERR_CANNOT_OPEN_FOR_WRITING, dest);
+                        oflags = O_WRONLY | O_TRUNC | O_CREAT;
+                        if (_mode != -1) {
+                            filemode = _mode | S_IWUSR | S_IRUSR;
+                        }
+                    }
+
+                    if (auto err = tryOpen(f, QFile::encodeName(dest), oflags, filemode, errno)) {
+                        if (!err.wasCanceled()) {
+                            // qDebug() << "####################### COULD NOT WRITE" << dest << "_mode=" << _mode;
+                            // qDebug() << "QFile error==" << f.error() << "(" << f.errorString() << ")";
+
+                            if (f.error() == QFileDevice::PermissionsError) {
+                                error(KIO::ERR_WRITE_ACCESS_DENIED, dest);
+                            } else {
+                                error(KIO::ERR_CANNOT_OPEN_FOR_WRITING, dest);
+                            }
+                        }
+                        return;
+                    } else {
+#ifndef Q_OS_WIN
+                        if ((_flags & KIO::Resume)) {
+                            execWithElevatedPrivilege(CHOWN, {dest, getuid(), getgid()}, errno);
+                            QFile::setPermissions(dest, modeToQFilePermissions(filemode));
+                        }
+#endif
                     }
-                    return;
                 }
             }
 
@@ -676,13 +731,19 @@ void FileProtocol::put(const QUrl &url, int _mode, KIO::JobFlags _flags)
         //QFile::rename() never overwrites the destination file unlike ::remove,
         //so we must remove it manually first
         if (_flags & KIO::Overwrite) {
-            QFile::remove(dest_orig);
+            if (!QFile::remove(dest_orig)) {
+                execWithElevatedPrivilege(DEL, {dest_orig}, errno);
+            }
         }
 
         if (!QFile::rename(dest, dest_orig)) {
-            qCWarning(KIO_FILE) << " Couldn't rename " << dest << " to " << dest_orig;
-            error(KIO::ERR_CANNOT_RENAME_PARTIAL, dest_orig);
-            return;
+            if (auto err = execWithElevatedPrivilege(RENAME, {dest, dest_orig}, errno)) {
+                if (!err.wasCanceled()) {
+                    qCWarning(KIO_FILE) << " Couldn't rename " << dest << " to " << dest_orig;
+                    error(KIO::ERR_CANNOT_RENAME_PARTIAL, dest_orig);
+                }
+                return;
+            }
         }
         org::kde::KDirNotify::emitFileRenamed(QUrl::fromLocalFile(dest), QUrl::fromLocalFile(dest_orig));
     }
@@ -693,7 +754,9 @@ void FileProtocol::put(const QUrl &url, int _mode, KIO::JobFlags _flags)
             // couldn't chmod. Eat the error if the filesystem apparently doesn't support it.
             KMountPoint::Ptr mp = KMountPoint::currentMountPoints().findByPath(dest_orig);
             if (mp && mp->testFileSystemFlag(KMountPoint::SupportsChmod)) {
-                warning(i18n("Could not change permissions for\n%1",  dest_orig));
+                if (tryChangeFileAttr(CHMOD, {dest_orig, _mode}, errno)) {
+                    warning(i18n("Could not change permissions for\n%1",  dest_orig));
+                }
             }
         }
     }
@@ -718,7 +781,9 @@ void FileProtocol::put(const QUrl &url, int _mode, KIO::JobFlags _flags)
                 struct utimbuf utbuf;
                 utbuf.actime = dest_statbuf.st_atime;
                 utbuf.modtime = dt.toTime_t();
-                utime(QFile::encodeName(dest_orig).constData(), &utbuf);
+                if (utime(QFile::encodeName(dest_orig).constData(), &utbuf) != 0) {
+                    tryChangeFileAttr(UTIME, {dest_orig, qint64(utbuf.actime), qint64(utbuf.modtime)}, errno);
+                }
 #endif
             }
         }
@@ -854,11 +919,29 @@ notype:
 #pragma message("TODO: st_uid and st_gid are always zero, use GetSecurityInfo to find the owner")
 #endif
         entry.insert(KIO::UDSEntry::UDS_ACCESS_TIME, buff.st_atime);
+#ifdef st_birthtime
+        /* For example FreeBSD's and NetBSD's stat contains a field for
+         * the inode birth time: st_birthtime
+         * This however only works on UFS and ZFS, and not, on say, NFS.
+         * Instead of setting a bogus fallback like st_mtime, only use
+         * it if it is greater than 0. */
+        if (buff.st_birthtime > 0) {
+            entry.insert(KIO::UDSEntry::UDS_CREATION_TIME, buff.st_birthtime);
+        }
+#endif
+#ifdef __st_birthtime
+        /* As above, but OpenBSD calls it slightly differently. */
+        if (buff.__st_birthtime > 0) {
+            entry.insert(KIO::UDSEntry::UDS_CREATION_TIME, buff.__st_birthtime);
+        }
+#endif
     }
 
     // Note: buff.st_ctime isn't the creation time !
     // We made that mistake for KDE 2.0, but it's in fact the
     // "file status" change time, which we don't care about.
+    // For FreeBSD and NetBSD, use st_birthtime. For OpenBSD,
+    // use __st_birthtime.
 
     return true;
 }
@@ -1248,7 +1331,7 @@ static QString readLogFile(const QByteArray &_filename)
 
 bool FileProtocol::isExtendedACL(acl_t acl)
 {
-    return (acl_equiv_mode(acl, 0) != 0);
+    return (acl_equiv_mode(acl, nullptr) != 0);
 }
 
 static void appendACLAtoms(const QByteArray &path, UDSEntry &entry, mode_t type)
@@ -1258,8 +1341,8 @@ static void appendACLAtoms(const QByteArray &path, UDSEntry &entry, mode_t type)
         return;
     }
 
-    acl_t acl = 0;
-    acl_t defaultAcl = 0;
+    acl_t acl = nullptr;
+    acl_t defaultAcl = nullptr;
     bool isDir = (type & QT_STAT_MASK) == QT_STAT_DIR;
     // do we have an acl for the file, and/or a default acl for the dir, if it is one?
     acl = acl_get_file(path.data(), ACL_TYPE_ACCESS);
@@ -1269,7 +1352,7 @@ static void appendACLAtoms(const QByteArray &path, UDSEntry &entry, mode_t type)
         if (acl) {
             if (!FileProtocol::isExtendedACL(acl)) {
                 acl_free(acl);
-                acl = 0;
+                acl = nullptr;
             }
         }
         defaultAcl = acl_get_file(path.data(), ACL_TYPE_DEFAULT);
@@ -1312,8 +1395,12 @@ bool FileProtocol::deleteRecursive(const QString &path)
         } else {
             //qDebug() << "QFile::remove" << itemPath;
             if (!QFile::remove(itemPath)) {
-                error(KIO::ERR_CANNOT_DELETE, itemPath);
-                return false;
+                if (auto err = execWithElevatedPrivilege(DEL, {itemPath}, errno)) {
+                    if (!err.wasCanceled()) {
+                        error(KIO::ERR_CANNOT_DELETE, itemPath);
+                    }
+                    return false;
+                }
             }
         }
     }
@@ -1321,8 +1408,12 @@ bool FileProtocol::deleteRecursive(const QString &path)
     Q_FOREACH (const QString &itemPath, dirsToDelete) {
         //qDebug() << "QDir::rmdir" << itemPath;
         if (!dir.rmdir(itemPath)) {
-            error(KIO::ERR_CANNOT_DELETE, itemPath);
-            return false;
+            if (auto err = execWithElevatedPrivilege(RMDIR, {itemPath}, errno)) {
+                if (!err.wasCanceled()) {
+                    error(KIO::ERR_CANNOT_DELETE, itemPath);
+                }
+                return false;
+            }
         }
     }
     return true;
