@@ -39,6 +39,10 @@
 #include <errno.h>
 #include <utime.h>
 
+#include <KAuth>
+
+#include "fdreceiver.h"
+
 //sendfile has different semantics in different platforms
 #if defined HAVE_SENDFILE && defined Q_OS_LINUX
 #define USE_SENDFILE 1
@@ -63,9 +67,71 @@ same_inode(const QT_STATBUF &src, const QT_STATBUF &dest)
     return false;
 }
 
+static const QString socketPath()
+{
+    return QStringLiteral("org_kde_kio_file_helper_%1").arg(getpid());
+}
+
+bool FileProtocol::privilegeOperationUnitTestMode()
+{
+    return (metaData(QStringLiteral("UnitTesting")) == QLatin1String("true"))
+            && (requestPrivilegeOperation() == KIO::OperationAllowed);
+}
+
+PrivilegeOperationReturnValue FileProtocol::tryOpen(QFile &f, const QByteArray &path, int flags, int mode, int errcode)
+{
+    const QString sockPath = socketPath();
+    FdReceiver fdRecv(sockPath);
+    if (!fdRecv.isListening()) {
+        return PrivilegeOperationReturnValue::failure(errcode);
+    }
+
+    QIODevice::OpenMode openMode;
+    if (flags & O_RDONLY) {
+        openMode |= QIODevice::ReadOnly;
+    }
+    if (flags & O_WRONLY || flags & O_CREAT) {
+        openMode |= QIODevice::WriteOnly;
+    }
+    if (flags & O_RDWR) {
+        openMode |= QIODevice::ReadWrite;
+    }
+    if (flags & O_TRUNC) {
+        openMode |= QIODevice::Truncate;
+    }
+    if (flags & O_APPEND) {
+        openMode |= QIODevice::Append;
+    }
+
+    if (auto err = execWithElevatedPrivilege(OPEN, {path, flags, mode, sockPath}, errcode)) {
+        return err;
+    } else {
+        int fd = fdRecv.fileDescriptor();
+        if (fd < 3 || !f.open(fd, openMode, QFileDevice::AutoCloseHandle)) {
+            return PrivilegeOperationReturnValue::failure(errcode);
+        }
+    }
+    return PrivilegeOperationReturnValue::success();
+}
+
+PrivilegeOperationReturnValue FileProtocol::tryChangeFileAttr(ActionType action, const QVariantList &args, int errcode)
+{
+    KAuth::Action execAction(QStringLiteral("org.kde.kio.file.exec"));
+    execAction.setHelperId(QStringLiteral("org.kde.kio.file"));
+    if (execAction.status() == KAuth::Action::AuthorizedStatus) {
+        return execWithElevatedPrivilege(action, args, errcode);
+    }
+    return PrivilegeOperationReturnValue::failure(errcode);
+}
+
 void FileProtocol::copy(const QUrl &srcUrl, const QUrl &destUrl,
                         int _mode, JobFlags _flags)
 {
+    if (privilegeOperationUnitTestMode()) {
+        finished();
+        return;
+    }
+
     // qDebug() << "copy(): " << srcUrl << " -> " << destUrl << ", mode=" << _mode;
 
     const QString src = srcUrl.toLocalFile();
@@ -118,14 +184,25 @@ void FileProtocol::copy(const QUrl &srcUrl, const QUrl &destUrl,
         // the symlink actually points to current source!
         if ((_flags & KIO::Overwrite) && ((buff_dest.st_mode & QT_STAT_MASK) == QT_STAT_LNK)) {
             //qDebug() << "copy(): LINK DESTINATION";
-            QFile::remove(dest);
+            if (!QFile::remove(dest)) {
+                if (auto err = execWithElevatedPrivilege(DEL, {_dest}, errno)) {
+                    if (!err.wasCanceled()) {
+                        error(KIO::ERR_CANNOT_DELETE_ORIGINAL, dest);
+                    }
+                    return;
+                }
+            }
         }
     }
 
     QFile src_file(src);
     if (!src_file.open(QIODevice::ReadOnly)) {
-        error(KIO::ERR_CANNOT_OPEN_FOR_READING, src);
-        return;
+        if (auto err = tryOpen(src_file, _src, O_RDONLY, S_IRUSR, errno)) {
+            if (!err.wasCanceled()) {
+                error(KIO::ERR_CANNOT_OPEN_FOR_READING, src);
+            }
+            return;
+        }
     }
 
 #if HAVE_FADVISE
@@ -134,16 +211,24 @@ void FileProtocol::copy(const QUrl &srcUrl, const QUrl &destUrl,
 
     QFile dest_file(dest);
     if (!dest_file.open(QIODevice::Truncate | QIODevice::WriteOnly)) {
-        // qDebug() << "###### COULD NOT WRITE " << dest;
-        if (errno == EACCES) {
-            error(KIO::ERR_WRITE_ACCESS_DENIED, dest);
-        } else {
-            error(KIO::ERR_CANNOT_OPEN_FOR_WRITING, dest);
+        if (auto err = tryOpen(dest_file, _dest, O_WRONLY | O_TRUNC | O_CREAT, S_IRUSR | S_IWUSR, errno)) {
+            if (!err.wasCanceled()) {
+                // qDebug() << "###### COULD NOT WRITE " << dest;
+                if (err == EACCES) {
+                    error(KIO::ERR_WRITE_ACCESS_DENIED, dest);
+                } else {
+                    error(KIO::ERR_CANNOT_OPEN_FOR_WRITING, dest);
+                }
+            }
+            src_file.close();
+            return;
         }
-        src_file.close();
-        return;
     }
 
+    // nobody shall be allowed to peek into the file during creation
+    // Note that error handling is omitted for this call, we don't want to error on e.g. VFAT
+    dest_file.setPermissions(QFileDevice::ReadOwner | QFileDevice::WriteOwner);
+
 #if HAVE_FADVISE
     posix_fadvise(dest_file.handle(), 0, 0, POSIX_FADV_SEQUENTIAL);
 #endif
@@ -153,7 +238,7 @@ void FileProtocol::copy(const QUrl &srcUrl, const QUrl &destUrl,
     if (acl && !isExtendedACL(acl)) {
         // qDebug() << _dest.data() << " doesn't have extended ACL";
         acl_free(acl);
-        acl = NULL;
+        acl = nullptr;
     }
 #endif
     totalSize(buff_src.st_size);
@@ -188,7 +273,6 @@ void FileProtocol::copy(const QUrl &srcUrl, const QUrl &destUrl,
                 // qDebug() << "sendfile() error:" << strerror(errno);
                 if (errno == ENOSPC) { // disk full
                     error(KIO::ERR_DISK_FULL, dest);
-                    remove(_dest.data());
                 } else {
                     error(KIO::ERR_SLAVE_DEFINED,
                           i18n("Cannot copy file from %1 to %2. (Errno: %3)",
@@ -204,6 +288,9 @@ void FileProtocol::copy(const QUrl &srcUrl, const QUrl &destUrl,
                 acl_free(acl);
             }
 #endif
+            if (!QFile::remove(dest)) {  // don't keep partly copied file
+                execWithElevatedPrivilege(DEL, {_dest}, errno);
+            }
             return;
         }
         if (n == 0) {
@@ -215,9 +302,8 @@ void FileProtocol::copy(const QUrl &srcUrl, const QUrl &destUrl,
             if (dest_file.write(buffer, n) != n) {
                 if (dest_file.error() == QFileDevice::ResourceError) {  // disk full
                     error(KIO::ERR_DISK_FULL, dest);
-                    remove(_dest.data());
                 } else {
-                    qWarning() << "Couldn't write[2]. Error:" << dest_file.errorString();
+                    qCWarning(KIO_FILE) << "Couldn't write[2]. Error:" << dest_file.errorString();
                     error(KIO::ERR_CANNOT_WRITE, dest);
                 }
 #if HAVE_POSIX_ACL
@@ -225,6 +311,9 @@ void FileProtocol::copy(const QUrl &srcUrl, const QUrl &destUrl,
                     acl_free(acl);
                 }
 #endif
+                if (!QFile::remove(dest)) {  // don't keep partly copied file
+                    execWithElevatedPrivilege(DEL, {_dest}, errno);
+                }
                 return;
             }
             processed_size += n;
@@ -238,27 +327,36 @@ void FileProtocol::copy(const QUrl &srcUrl, const QUrl &destUrl,
     dest_file.close();
 
     if (dest_file.error() != QFile::NoError) {
-        qWarning() << "Error when closing file descriptor[2]:" << dest_file.errorString();
+        qCWarning(KIO_FILE) << "Error when closing file descriptor[2]:" << dest_file.errorString();
         error(KIO::ERR_CANNOT_WRITE, dest);
 #if HAVE_POSIX_ACL
         if (acl) {
             acl_free(acl);
         }
 #endif
+        if (!QFile::remove(dest)) {  // don't keep partly copied file
+            execWithElevatedPrivilege(DEL, {_dest}, errno);
+        }
         return;
     }
 
     // set final permissions
-    if (_mode != -1) {
-        if ((::chmod(_dest.data(), _mode) != 0)
+    // if no special mode given, preserve the mode from the sourcefile
+    if (_mode == -1) {
+        _mode = buff_src.st_mode;
+    }
+
+    if ((::chmod(_dest.data(), _mode) != 0)
 #if HAVE_POSIX_ACL
-                || (acl && acl_set_file(_dest.data(), ACL_TYPE_ACCESS, acl) != 0)
+        || (acl && acl_set_file(_dest.data(), ACL_TYPE_ACCESS, acl) != 0)
 #endif
-           ) {
-            KMountPoint::Ptr mp = KMountPoint::currentMountPoints().findByPath(dest);
-            // Eat the error if the filesystem apparently doesn't support chmod.
-            if (mp && mp->testFileSystemFlag(KMountPoint::SupportsChmod)) {
-                warning(i18n("Could not change permissions for\n%1", dest));
+       ) {
+        const int errCode = errno;
+        KMountPoint::Ptr mp = KMountPoint::currentMountPoints().findByPath(dest);
+        // Eat the error if the filesystem apparently doesn't support chmod.
+        if (mp && mp->testFileSystemFlag(KMountPoint::SupportsChmod)) {
+            if (tryChangeFileAttr(CHMOD, {_dest, _mode}, errCode)) {
+                warning(i18n("Could not change permissions for '%1'", dest));
             }
         }
     }
@@ -268,12 +366,25 @@ void FileProtocol::copy(const QUrl &srcUrl, const QUrl &destUrl,
     }
 #endif
 
+    // preserve ownership
+    if (::chown(_dest.data(), -1 /*keep user*/, buff_src.st_gid) == 0) {
+        // as we are the owner of the new file, we can always change the group, but
+        // we might not be allowed to change the owner
+        (void)::chown(_dest.data(), buff_src.st_uid, -1 /*keep group*/);
+    } else {
+        if (tryChangeFileAttr(CHOWN, {_dest, buff_src.st_uid, buff_src.st_gid}, errno)) {
+            qCWarning(KIO_FILE) << QStringLiteral("Couldn't preserve group for '%1'").arg(dest);
+        }
+    }
+
     // copy access and modification time
     struct utimbuf ut;
     ut.actime = buff_src.st_atime;
     ut.modtime = buff_src.st_mtime;
     if (::utime(_dest.data(), &ut) != 0) {
-        qWarning() << QStringLiteral("Couldn't preserve access and modification time for\n%1").arg(dest);
+        if (tryChangeFileAttr(UTIME, {_dest, qint64(ut.actime), qint64(ut.modtime)}, errno)) {
+            qCWarning(KIO_FILE) << QStringLiteral("Couldn't preserve access and modification time for '%1'").arg(dest);
+        }
     }
 
     processedSize(buff_src.st_size);
@@ -312,7 +423,7 @@ void FileProtocol::listDir(const QUrl &url)
     const QString path(url.toLocalFile());
     const QByteArray _path(QFile::encodeName(path));
     DIR *dp = opendir(_path.data());
-    if (dp == 0) {
+    if (dp == nullptr) {
         switch (errno) {
         case ENOENT:
             error(KIO::ERR_DOES_NOT_EXIST, path);
@@ -356,7 +467,7 @@ void FileProtocol::listDir(const QUrl &url)
     QT_STATBUF st;
 #endif
     QT_DIRENT *ep;
-    while ((ep = QT_READDIR(dp)) != 0) {
+    while ((ep = QT_READDIR(dp)) != nullptr) {
         entry.clear();
 
         const QString filename = QFile::decodeName(ep->d_name);
@@ -448,16 +559,20 @@ void FileProtocol::rename(const QUrl &srcUrl, const QUrl &destUrl,
     }
 
     if (::rename(_src.data(), _dest.data())) {
-        if ((errno == EACCES) || (errno == EPERM)) {
-            error(KIO::ERR_ACCESS_DENIED, dest);
-        } else if (errno == EXDEV) {
-            error(KIO::ERR_UNSUPPORTED_ACTION, QStringLiteral("rename"));
-        } else if (errno == EROFS) { // The file is on a read-only filesystem
-            error(KIO::ERR_CANNOT_DELETE, src);
-        } else {
-            error(KIO::ERR_CANNOT_RENAME, src);
+        if (auto err = execWithElevatedPrivilege(RENAME, {_src, _dest}, errno)) {
+            if (!err.wasCanceled()) {
+                if ((err == EACCES) || (err == EPERM)) {
+                    error(KIO::ERR_ACCESS_DENIED, dest);
+                } else if (err == EXDEV) {
+                    error(KIO::ERR_UNSUPPORTED_ACTION, QStringLiteral("rename"));
+                } else if (err == EROFS) { // The file is on a read-only filesystem
+                    error(KIO::ERR_CANNOT_DELETE, src);
+                } else {
+                    error(KIO::ERR_CANNOT_RENAME, src);
+                }
+            }
+            return;
         }
-        return;
     }
 
     finished();
@@ -473,8 +588,12 @@ void FileProtocol::symlink(const QString &target, const QUrl &destUrl, KIO::JobF
             if ((flags & KIO::Overwrite)) {
                 // Try to delete the destination
                 if (unlink(QFile::encodeName(dest).constData()) != 0) {
-                    error(KIO::ERR_CANNOT_DELETE, dest);
-                    return;
+                    if (auto err = execWithElevatedPrivilege(DEL, {dest}, errno)) {
+                        if (!err.wasCanceled()) {
+                            error(KIO::ERR_CANNOT_DELETE, dest);
+                        }
+                        return;
+                    }
                 }
                 // Try again - this won't loop forever since unlink succeeded
                 symlink(target, destUrl, flags);
@@ -489,9 +608,13 @@ void FileProtocol::symlink(const QString &target, const QUrl &destUrl, KIO::JobF
                 return;
             }
         } else {
-            // Some error occurred while we tried to symlink
-            error(KIO::ERR_CANNOT_SYMLINK, dest);
-            return;
+            if (auto err = execWithElevatedPrivilege(SYMLINK, {dest, target}, errno)) {
+                if (!err.wasCanceled()) {
+                    // Some error occurred while we tried to symlink
+                    error(KIO::ERR_CANNOT_SYMLINK, dest);
+                }
+                return;
+            }
         }
     }
     finished();
@@ -509,14 +632,18 @@ void FileProtocol::del(const QUrl &url, bool isfile)
         // qDebug() << "Deleting file "<< url;
 
         if (unlink(_path.data()) == -1) {
-            if ((errno == EACCES) || (errno == EPERM)) {
-                error(KIO::ERR_ACCESS_DENIED, path);
-            } else if (errno == EISDIR) {
-                error(KIO::ERR_IS_DIRECTORY, path);
-            } else {
-                error(KIO::ERR_CANNOT_DELETE, path);
+            if (auto err = execWithElevatedPrivilege(DEL, {_path}, errno)) {
+                if (!err.wasCanceled()) {
+                    if ((err == EACCES) || (err ==  EPERM)) {
+                        error(KIO::ERR_ACCESS_DENIED, path);
+                    } else if (err == EISDIR) {
+                        error(KIO::ERR_IS_DIRECTORY, path);
+                    } else {
+                        error(KIO::ERR_CANNOT_DELETE, path);
+                    }
+                }
+                return;
             }
-            return;
         }
     } else {
 
@@ -531,11 +658,15 @@ void FileProtocol::del(const QUrl &url, bool isfile)
             }
         }
         if (QT_RMDIR(_path.data()) == -1) {
-            if ((errno == EACCES) || (errno == EPERM)) {
-                error(KIO::ERR_ACCESS_DENIED, path);
-            } else {
-                // qDebug() << "could not rmdir " << perror;
-                error(KIO::ERR_CANNOT_RMDIR, path);
+            if (auto err = execWithElevatedPrivilege(RMDIR, {_path}, errno)) {
+                if (!err.wasCanceled()) {
+                    if ((err == EACCES) || (err == EPERM)) {
+                        error(KIO::ERR_ACCESS_DENIED, path);
+                    } else {
+                        // qDebug() << "could not rmdir " << perror;
+                        error(KIO::ERR_CANNOT_RMDIR, path);
+                    }
+                }
                 return;
             }
         }
@@ -578,16 +709,20 @@ void FileProtocol::chown(const QUrl &url, const QString &owner, const QString &g
     }
 
     if (::chown(_path.constData(), uid, gid) == -1) {
-        switch (errno) {
-        case EPERM:
-        case EACCES:
-            error(KIO::ERR_ACCESS_DENIED, path);
-            break;
-        case ENOSPC:
-            error(KIO::ERR_DISK_FULL, path);
-            break;
-        default:
-            error(KIO::ERR_CANNOT_CHOWN, path);
+        if (auto err = execWithElevatedPrivilege(CHOWN, {_path, uid, gid}, errno)) {
+            if (!err.wasCanceled()) {
+                switch (err) {
+                case EPERM:
+                case EACCES:
+                    error(KIO::ERR_ACCESS_DENIED, path);
+                    break;
+                case ENOSPC:
+                    error(KIO::ERR_DISK_FULL, path);
+                    break;
+                default:
+                    error(KIO::ERR_CANNOT_CHOWN, path);
+                }
+            }
         }
     } else {
         finished();
@@ -596,7 +731,7 @@ void FileProtocol::chown(const QUrl &url, const QString &owner, const QString &g
 
 void FileProtocol::stat(const QUrl &url)
 {
-    if (!url.isLocalFile()) {
+    if (!isLocalFileSameHost(url)) {
         QUrl redir(url);
         redir.setScheme(config()->readEntry("DefaultRemoteProtocol", "smb"));
         redirection(redir);
@@ -634,3 +769,49 @@ void FileProtocol::stat(const QUrl &url)
 
     finished();
 }
+
+PrivilegeOperationReturnValue FileProtocol::execWithElevatedPrivilege(ActionType action, const QVariantList &args, int errcode)
+{
+    if (privilegeOperationUnitTestMode()) {
+        return PrivilegeOperationReturnValue::success();
+    }
+
+    // temporarily disable privilege execution
+    if (true) {
+        return PrivilegeOperationReturnValue::failure(errcode);
+    }
+
+    if (!(errcode == EACCES || errcode == EPERM)) {
+        return PrivilegeOperationReturnValue::failure(errcode);
+    }
+
+    KIO::PrivilegeOperationStatus opStatus = requestPrivilegeOperation();
+    if (opStatus != KIO::OperationAllowed) {
+        if (opStatus == KIO::OperationCanceled) {
+            error(KIO::ERR_USER_CANCELED, QString());
+            return PrivilegeOperationReturnValue::canceled();
+        }
+        return PrivilegeOperationReturnValue::failure(errcode);
+    }
+
+    QByteArray helperArgs;
+    QDataStream out(&helperArgs, QIODevice::WriteOnly);
+    out << action;
+    foreach (const QVariant &arg, args) {
+        out << arg;
+    }
+
+    KAuth::Action execAction(QStringLiteral("org.kde.kio.file.exec"));
+    execAction.setHelperId(QStringLiteral("org.kde.kio.file"));
+
+    QVariantMap argv;
+    argv.insert(QStringLiteral("arguments"), helperArgs);
+    execAction.setArguments(argv);
+
+    auto reply = execAction.execute();
+    if (reply->exec()) {
+        return PrivilegeOperationReturnValue::success();
+    }
+
+    return PrivilegeOperationReturnValue::failure(KIO::ERR_ACCESS_DENIED);
+}
