@@ -52,7 +52,7 @@ QDateTime g_currentDate;
 int g_maxCacheAge;
 qint64 g_maxCacheSize;
 
-static const char appFullName[] = "org.kde.kio_http_cache_cleaner";
+static const char appFullName[] = "org.kio5.kio_http_cache_cleaner";
 static const char appName[] = "kio_http_cache_cleaner";
 
 // !START OF SYNC!
@@ -187,9 +187,14 @@ static QString filenameFromUrl(const QByteArray &url)
     return QString::fromLatin1(hash.result().toHex());
 }
 
+static QString cacheDir()
+{
+    return QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation) + "/kio_http";
+}
+
 static QString filePath(const QString &baseName)
 {
-    QString cacheDirName = QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation) + '/' + "http";
+    QString cacheDirName = cacheDir();
     if (!cacheDirName.endsWith('/')) {
         cacheDirName.append('/');
     }
@@ -396,18 +401,19 @@ public:
     {
         // read in the scoreboard...
         QFile sboard(filePath(QStringLiteral("scoreboard")));
-        sboard.open(QIODevice::ReadOnly);
-        while (true) {
-            QByteArray baIndex = sboard.read(ScoreboardEntry::indexSize);
-            QByteArray baRest = sboard.read(ScoreboardEntry::size - ScoreboardEntry::indexSize);
-            if (baIndex.size() + baRest.size() != ScoreboardEntry::size) {
-                break;
-            }
+        if (sboard.open(QIODevice::ReadOnly)) {
+            while (true) {
+                QByteArray baIndex = sboard.read(ScoreboardEntry::indexSize);
+                QByteArray baRest = sboard.read(ScoreboardEntry::size - ScoreboardEntry::indexSize);
+                if (baIndex.size() + baRest.size() != ScoreboardEntry::size) {
+                    break;
+                }
 
-            const QString entryBasename = QString::fromLatin1(baIndex.toHex());
-            MiniCacheFileInfo mcfi;
-            if (readAndValidateMcfi(baRest, entryBasename, &mcfi)) {
-                m_scoreboard.insert(CacheIndex(baIndex), mcfi);
+                const QString entryBasename = QString::fromLatin1(baIndex.toHex());
+                MiniCacheFileInfo mcfi;
+                if (readAndValidateMcfi(baRest, entryBasename, &mcfi)) {
+                    m_scoreboard.insert(CacheIndex(baIndex), mcfi);
+                }
             }
         }
     }
@@ -611,7 +617,7 @@ public:
 
     // Delete some of the files that need to be deleted. Return true when done, false otherwise.
     // This makes interleaved cleaning / serving ioslaves possible.
-    bool processSlice(Scoreboard *scoreboard = 0)
+    bool processSlice(Scoreboard *scoreboard = nullptr)
     {
         QTime t;
         t.start();
@@ -664,7 +670,7 @@ public:
 
             if (m_fileNameList.isEmpty()) {
                 // final step of phase one
-                qSort(m_fiList.begin(), m_fiList.end(), CacheFileInfoPtrLessThan);
+                std::sort(m_fiList.begin(), m_fiList.end(), CacheFileInfoPtrLessThan);
             }
             return false;
         }
@@ -753,7 +759,8 @@ int main(int argc, char **argv)
     g_maxCacheAge = KProtocolManager::maxCacheAge();
     g_maxCacheSize = mode == DeleteCache ? -1 : KProtocolManager::maxCacheSize() * 1024;
 
-    QString cacheDirName = QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation) + '/' + "http";
+    QString cacheDirName = cacheDir();
+    QDir().mkpath(cacheDirName);
     QDir cacheDir(cacheDirName);
     if (!cacheDir.exists()) {
         fprintf(stderr, "%s: '%s' does not exist.\n", appName, qPrintable(cacheDirName));
@@ -776,12 +783,14 @@ int main(int argc, char **argv)
     QString socketFileName = QStandardPaths::writableLocation(QStandardPaths::RuntimeLocation) + QLatin1Char('/') + "kio_http_cache_cleaner";
     // we need to create the file by opening the socket, otherwise it won't work
     QFile::remove(socketFileName);
-    lServer.listen(socketFileName);
+    if (!lServer.listen(socketFileName)) {
+        qWarning() << "Error listening on" << socketFileName;
+    }
     QList<QLocalSocket *> sockets;
     qint64 newBytesCounter = LLONG_MAX;  // force cleaner run on startup
 
     Scoreboard scoreboard;
-    CacheCleaner *cleaner = 0;
+    CacheCleaner *cleaner = nullptr;
     while (true) {
         g_currentDate = QDateTime::currentDateTime();
         if (cleaner) {
@@ -829,7 +838,7 @@ int main(int argc, char **argv)
             if (cleaner->processSlice(&scoreboard)) {
                 // that was the last slice, done
                 delete cleaner;
-                cleaner = 0;
+                cleaner = nullptr;
             }
         } else if (newBytesCounter > (g_maxCacheSize / 8)) {
             cacheDir.refresh();
