@@ -46,8 +46,11 @@
 #include <QtCore/QDir>
 #include <QtCore/QRegExp>
 #include <QtCore/QTextStream>
+#include <QtCore/QLocale>
 #include <QUrl>
 
+Q_LOGGING_CATEGORY(KIO_COOKIEJAR, "kf5.kio.cookiejar")
+
 // BR87227
 // Waba: Should the number of cookies be limited?
 // I am not convinced of the need of such limit
@@ -72,9 +75,11 @@ static QString removeWeekday(const QString &value)
     if (index > -1) {
         int pos = 0;
         const QString weekday = value.left(index);
+        const QLocale cLocale = QLocale::c();
         for (int i = 1; i < 8; ++i) {
-            if (weekday.startsWith(QDate::shortDayName(i), Qt::CaseInsensitive) ||
-                    weekday.startsWith(QDate::longDayName(i), Qt::CaseInsensitive)) {
+            // No need to check for long names since the short names are
+            // prefixes of the long names
+            if (weekday.startsWith(cLocale.dayName(i, QLocale::ShortFormat), Qt::CaseInsensitive)) {
                 pos = index + 1;
                 break;
             }
@@ -106,11 +111,13 @@ static QDateTime parseDate(const QString &_value)
             "dd-MMM-yyyy HH:mm:ss 'GMT'", /* cookies.test: Y2K38 problem: 06-Dec-2039 00:30:42 GMT */
             "MMM dd HH:mm:ss yyyy 'GMT'", /* cookies.test: Non-standard expiration dates: Sep 12 07:00:00 2020 GMT */
             "MMM dd yyyy HH:mm:ss 'GMT'", /* cookies.test: Non-standard expiration dates: Sep 12 2020 07:00:00 GMT */
-            0
+            nullptr
         };
 
+        // Only English month names are allowed, thus use the C locale.
+        const QLocale cLocale = QLocale::c();
         for (int i = 0; date_formats[i]; ++i) {
-            dt = QDateTime::fromString(value, QL1S(date_formats[i]));
+            dt = cLocale.toDateTime(value, QL1S(date_formats[i]));
             if (dt.isValid()) {
                 break;
             }
@@ -375,10 +382,10 @@ QString KCookieJar::findCookies(const QString &_url, bool useDOMFormat, WId wind
 
     KHttpCookieList allCookies;
     for (QStringList::ConstIterator it = domains.constBegin(), itEnd = domains.constEnd();; ++it) {
-        KHttpCookieList *cookieList = 0;
+        KHttpCookieList *cookieList = nullptr;
         if (it == itEnd) {
             cookieList = pendingCookies; // Add pending cookies
-            pendingCookies = 0;
+            pendingCookies = nullptr;
             if (!cookieList) {
                 break;
             }
@@ -1253,9 +1260,9 @@ void KCookieJar::eatSessionCookies(const QString &fqdn, WId windowId,
 #ifndef NDEBUG
             if (ids.contains(windowId)) {
                 if (ids.count() > 1) {
-                    qDebug() << "removing window id" << windowId << "from session cookie";
+                    qCDebug(KIO_COOKIEJAR) << "removing window id" << windowId << "from session cookie";
                 } else {
-                    qDebug() << "deleting session cookie";
+                    qCDebug(KIO_COOKIEJAR) << "deleting session cookie";
                 }
             }
 #endif
@@ -1387,7 +1394,7 @@ static const char *parseField(char *&buffer, bool keepQuotes = false)
     return result;
 }
 
-static QString extractHostAndPorts(const QString &str, QList<int> *ports = 0)
+static QString extractHostAndPorts(const QString &str, QList<int> *ports = nullptr)
 {
     if (str.isEmpty()) {
         return str;
@@ -1475,7 +1482,7 @@ bool KCookieJar::loadCookies(const QString &_filename)
             bool secure = false;
             bool httpOnly = false;
             bool explicitPath = false;
-            const char *value = 0;
+            const char *value = nullptr;
             if ((version == 2) || (protVer >= 200)) {
                 if (protVer >= 200) {
                     protVer -= 200;
