@@ -40,7 +40,7 @@
 K_PLUGIN_FACTORY_DECLARATION(KioConfigFactory)
 
 CacheConfigModule::CacheConfigModule(QWidget *parent, const QVariantList &)
-                  :KCModule(/*KioConfigFactory::componentData(),*/ parent)
+                  :KCModule(parent)
 {
   ui.setupUi(this);
 }
@@ -106,7 +106,7 @@ void CacheConfigModule::defaults()
 QString CacheConfigModule::quickHelp() const
 {
   return i18n( "<h1>Cache</h1><p>This module lets you configure your cache settings.</p>"
-                "<p>The cache is an internal memory in Konqueror where recently "
+                "<p>This specific cache is an area on the disk where recently "
                 "read web pages are stored. If you want to retrieve a web "
                 "page again that you have recently read, it will not be "
                 "downloaded from the Internet, but rather retrieved from the "
@@ -120,7 +120,7 @@ void CacheConfigModule::configChanged()
 
 void CacheConfigModule::on_clearCacheButton_clicked()
 {
-    QString exe = QFile::decodeName(CMAKE_INSTALL_FULL_LIBEXECDIR_KF5 "/kio_http_cache_cleaner");
+    const QString exe = QFile::decodeName(CMAKE_INSTALL_FULL_LIBEXECDIR_KF5 "/kio_http_cache_cleaner");
 
     if (QFile::exists(exe)) {
         QProcess::startDetached(exe, QStringList(QStringLiteral("--clear-all")));
