@@ -25,6 +25,7 @@
 #include "pastejob_p.h"
 #include "jobuidelegate.h"
 #include "jobuidelegateextension.h"
+#include "kio_widgets_debug.h"
 
 #include <KConfigGroup>
 #include <KCoreDirLister>
@@ -56,6 +57,27 @@ using namespace KIO;
 
 Q_DECLARE_METATYPE(Qt::DropAction)
 
+namespace KIO {
+class DropMenu;
+}
+
+class KIO::DropMenu : public QMenu
+{
+public:
+    DropMenu(QWidget *parent = nullptr);
+    ~DropMenu();
+
+    void addCancelAction();
+    void addExtraActions(const QList<QAction *> &appActions, const QList<QAction *> &pluginActions);
+
+private:
+    QList<QAction *> m_appActions;
+    QList<QAction *> m_pluginActions;
+    QAction *m_lastSeparator;
+    QAction *m_extraActionsSeparator;
+    QAction *m_cancelAction;
+};
+
 class KIO::DropJobPrivate : public KIO::JobPrivate
 {
 public:
@@ -65,7 +87,7 @@ public:
           m_mimeData(dropEvent->mimeData()),
           m_urls(KUrlMimeData::urlsFromMimeData(m_mimeData, KUrlMimeData::PreferLocalUrls, &m_metaData)),
           m_dropAction(dropEvent->dropAction()),
-          m_globalPos(QCursor::pos()), // record mouse pos at time of drop
+          m_relativePos(dropEvent->pos()),
           m_keyboardModifiers(dropEvent->keyboardModifiers()),
           m_destUrl(destUrl),
           m_destItem(KCoreDirLister::cachedItemForUrl(destUrl)),
@@ -80,6 +102,24 @@ public:
         if (m_destItem.isNull() && m_destUrl.isLocalFile()) {
             m_destItem = KFileItem(m_destUrl);
         }
+
+        if (!(m_flags & KIO::NoPrivilegeExecution)) {
+            m_privilegeExecutionEnabled = true;
+            switch (m_dropAction) {
+            case Qt::CopyAction:
+                m_operationType = Copy;
+                break;
+            case Qt::MoveAction:
+                m_operationType = Move;
+                break;
+            case Qt::LinkAction:
+                m_operationType = Symlink;
+                break;
+            default:
+                m_operationType = Other;
+                break;
+            }
+        }
     }
 
     bool destIsDirectory() const
@@ -95,15 +135,15 @@ public:
     void handleDropToDesktopFile();
     void handleDropToExecutable();
     int determineDropAction();
-    void fillPopupMenu(QMenu *popup);
-    void addPluginActions(QMenu *popup, const KFileItemListProperties &itemProps);
+    void fillPopupMenu(KIO::DropMenu *popup);
+    void addPluginActions(KIO::DropMenu *popup, const KFileItemListProperties &itemProps);
     void doCopyToDirectory();
 
     const QMimeData *m_mimeData;
     const QList<QUrl> m_urls;
     QMap<QString, QString> m_metaData;
     Qt::DropAction m_dropAction;
-    QPoint m_globalPos;
+    QPoint m_relativePos;
     Qt::KeyboardModifiers m_keyboardModifiers;
     QUrl m_destUrl;
     KFileItem m_destItem; // null for remote URLs not found in the dirlister cache
@@ -111,6 +151,7 @@ public:
     QList<QAction *> m_appActions;
     QList<QAction *> m_pluginActions;
     bool m_triggered;  // Tracks whether an action has been triggered in the popup menu.
+    QSet<KIO::DropMenu *> m_menus;
 
     Q_DECLARE_PUBLIC(DropJob)
 
@@ -130,6 +171,50 @@ public:
 
 };
 
+DropMenu::DropMenu(QWidget *parent)
+    : QMenu(parent),
+      m_extraActionsSeparator(nullptr)
+{
+    m_cancelAction = new QAction(i18n("C&ancel") + '\t' + QKeySequence(Qt::Key_Escape).toString(), this);
+    m_cancelAction->setIcon(QIcon::fromTheme(QStringLiteral("process-stop")));
+
+    m_lastSeparator = new QAction(this);
+    m_lastSeparator->setSeparator(true);
+}
+
+DropMenu::~DropMenu()
+{
+}
+
+void DropMenu::addExtraActions(const QList<QAction *> &appActions, const QList<QAction *> &pluginActions)
+{
+    removeAction(m_lastSeparator);
+    removeAction(m_cancelAction);
+
+    removeAction(m_extraActionsSeparator);
+    for (QAction *action : m_appActions) {
+        removeAction(action);
+    }
+    for (QAction *action : m_pluginActions) {
+        removeAction(action);
+    }
+
+    m_appActions = appActions;
+    m_pluginActions = pluginActions;
+    if (!m_appActions.isEmpty() || !m_pluginActions.isEmpty()) {
+        if (!m_extraActionsSeparator) {
+            m_extraActionsSeparator = new QAction(this);
+            m_extraActionsSeparator->setSeparator(true);
+        }
+        addAction(m_extraActionsSeparator);
+        addActions(appActions);
+        addActions(pluginActions);
+    }
+
+    addAction(m_lastSeparator);
+    addAction(m_cancelAction);
+}
+
 DropJob::DropJob(DropJobPrivate &dd)
     : Job(dd)
 {
@@ -176,8 +261,9 @@ int DropJobPrivate::determineDropAction()
     if (!KProtocolManager::supportsWriting(m_destUrl)) {
         return KIO::ERR_CANNOT_WRITE;
     }
-    if (!m_destItem.isNull() && !m_destItem.isWritable()) {
-        return KIO::ERR_WRITE_ACCESS_DENIED;
+
+    if (!m_destItem.isNull() && !m_destItem.isWritable() && (m_flags & KIO::NoPrivilegeExecution)) {
+            return KIO::ERR_WRITE_ACCESS_DENIED;
     }
 
     bool allItemsAreFromTrash = true;
@@ -204,6 +290,11 @@ int DropJobPrivate::determineDropAction()
         }
         return KJob::NoError; // ok
     }
+    const bool implicitCopy = m_destUrl.scheme() == QLatin1String("stash");
+    if (implicitCopy) {
+        m_dropAction = Qt::CopyAction;
+        return KJob::NoError; // ok
+    }
     if (containsTrashRoot) {
         // Dropping a link to the trash: don't move the full contents, just make a link (#319660)
         m_dropAction = Qt::LinkAction;
@@ -223,7 +314,7 @@ int DropJobPrivate::determineDropAction()
     return KIO::ERR_UNKNOWN;
 }
 
-void DropJobPrivate::fillPopupMenu(QMenu *popup)
+void DropJobPrivate::fillPopupMenu(KIO::DropMenu *popup)
 {
     Q_Q(DropJob);
 
@@ -236,7 +327,7 @@ void DropJobPrivate::fillPopupMenu(QMenu *popup)
     }
     const KFileItemListProperties itemProps(fileItems);
 
-    emit q->popupMenuAboutToShow(fileItems);
+    emit q->popupMenuAboutToShow(itemProps);
 
     const bool sReading = itemProps.supportsReading();
     const bool sDeleting = itemProps.supportsDeleting();
@@ -260,8 +351,6 @@ void DropJobPrivate::fillPopupMenu(QMenu *popup)
     QAction* popupLinkAction = new QAction(i18n("&Link Here") + '\t' + seq, popup);
     popupLinkAction->setIcon(QIcon::fromTheme(QStringLiteral("edit-link")));
     popupLinkAction->setData(QVariant::fromValue(Qt::LinkAction));
-    QAction* popupCancelAction = new QAction(i18n("C&ancel") + '\t' + QKeySequence(Qt::Key_Escape).toString(), popup);
-    popupCancelAction->setIcon(QIcon::fromTheme(QStringLiteral("process-stop")));
 
     if (sMoving || (sReading && sDeleting)) {
         bool equalDestination = true;
@@ -286,10 +375,9 @@ void DropJobPrivate::fillPopupMenu(QMenu *popup)
     addPluginActions(popup, itemProps);
 
     popup->addSeparator();
-    popup->addAction(popupCancelAction);
 }
 
-void DropJobPrivate::addPluginActions(QMenu *popup, const KFileItemListProperties &itemProps)
+void DropJobPrivate::addPluginActions(KIO::DropMenu *popup, const KFileItemListProperties &itemProps)
 {
     const QVector<KPluginMetaData> plugin_offers = KPluginLoader::findPlugins(QStringLiteral("kf5/kio_dnd"));
     foreach (const KPluginMetaData &service, plugin_offers) {
@@ -297,23 +385,27 @@ void DropJobPrivate::addPluginActions(QMenu *popup, const KFileItemListPropertie
         if (factory) {
             KIO::DndPopupMenuPlugin *plugin = factory->create<KIO::DndPopupMenuPlugin>();
             if (plugin) {
-                m_pluginActions += plugin->setup(itemProps, m_destUrl);
+                auto actions = plugin->setup(itemProps, m_destUrl);
+                foreach (auto action, actions) {
+                    action->setParent(popup);
+                }
+                m_pluginActions += actions;
             }
         }
     }
 
-    if (!m_appActions.isEmpty() || !m_pluginActions.isEmpty()) {
-        popup->addSeparator();
-        popup->addActions(m_appActions);
-        popup->addActions(m_pluginActions);
-    }
-
+    popup->addExtraActions(m_appActions, m_pluginActions);
 }
 
 void DropJob::setApplicationActions(const QList<QAction *> &actions)
 {
     Q_D(DropJob);
+
     d->m_appActions = actions;
+
+    for (KIO::DropMenu *menu : d->m_menus) {
+        menu->addExtraActions(d->m_appActions, d->m_pluginActions);
+    }
 }
 
 void DropJobPrivate::slotTriggered(QAction *action)
@@ -353,7 +445,8 @@ void DropJobPrivate::handleCopyToDirectory()
 
     if (int error = determineDropAction()) {
         if (error == KIO::ERR_UNKNOWN) {
-            QMenu *menu = new QMenu(KJobWidgets::window(q));
+            auto window = KJobWidgets::window(q);
+            KIO::DropMenu *menu = new KIO::DropMenu(window);
             QObject::connect(menu, &QMenu::aboutToHide, menu, &QObject::deleteLater);
 
             // If the user clicks outside the menu, it will be destroyed without emitting the triggered signal.
@@ -364,7 +457,11 @@ void DropJobPrivate::handleCopyToDirectory()
                 m_triggered = true;
                 slotTriggered(action);
             });
-            menu->popup(m_globalPos);
+            menu->popup(window ? window->mapToGlobal(m_relativePos) : QCursor::pos());
+            m_menus.insert(menu);
+            QObject::connect(menu, &QObject::destroyed, q, [this, menu]() {
+                m_menus.remove(menu);
+            });
         } else {
             q->setError(error);
             q->emitResult();
@@ -377,7 +474,7 @@ void DropJobPrivate::handleCopyToDirectory()
 void DropJobPrivate::doCopyToDirectory()
 {
     Q_Q(DropJob);
-    KIO::CopyJob * job = 0;
+    KIO::CopyJob * job = nullptr;
     switch (m_dropAction) {
     case Qt::MoveAction:
         job = KIO::move(m_urls, m_destUrl, m_flags);
@@ -394,12 +491,13 @@ void DropJobPrivate::doCopyToDirectory()
         KIO::FileUndoManager::self()->recordCopyJob(job);
         break;
     default:
-        qWarning() << "Unknown drop action" << int(m_dropAction);
+        qCWarning(KIO_WIDGETS) << "Unknown drop action" << int(m_dropAction);
         q->setError(KIO::ERR_UNSUPPORTED_ACTION);
         q->emitResult();
         return;
     }
     Q_ASSERT(job);
+    job->setParentJob(q);
     job->setMetaData(m_metaData);
     QObject::connect(job, &KIO::CopyJob::copyingDone, q, [q](KIO::Job*, const QUrl &, const QUrl &to) {
             emit q->itemCreated(to);
@@ -408,6 +506,8 @@ void DropJobPrivate::doCopyToDirectory()
             emit q->itemCreated(to);
     });
     q->addSubjob(job);
+
+    emit q->copyJobStarted(job);
 }
 
 void DropJobPrivate::handleDropToDesktopFile()
@@ -431,7 +531,7 @@ void DropJobPrivate::handleDropToDesktopFile()
         handleCopyToDirectory();
     } else {
         if (desktopFile.hasDeviceType()) {
-            qWarning() << "Not re-implemented; please email kde-frameworks-devel@kde.org if you need this.";
+            qCWarning(KIO_WIDGETS) << "Not re-implemented; please email kde-frameworks-devel@kde.org if you need this.";
             // take code from libkonq's old konq_operations.cpp
             // for now, fallback
         }
@@ -443,12 +543,14 @@ void DropJobPrivate::handleDropToDesktopFile()
 
 void DropJobPrivate::handleDropToExecutable()
 {
+    Q_Q(DropJob);
     // Launch executable for each of the files
     QStringList args;
     Q_FOREACH(const QUrl &url, m_urls) {
         args << url.toLocalFile(); // assume local files
     }
     QProcess::startDetached(m_destUrl.toLocalFile(), args);
+    q->emitResult();
 }
 
 void DropJob::slotResult(KJob *job)
