@@ -28,11 +28,13 @@
 #include <kio/job.h>
 #include <kio/mkdirjob.h>
 #include <kio/mkpathjob.h>
+#include <kio/batchrenamejob.h>
 #include <kjobwidgets.h>
 #include <klocalizedstring.h>
 #include <kmessagebox.h>
 #include <kjobtrackerinterface.h>
 #include <kio/jobuidelegate.h>
+#include <job_p.h>
 
 #include <QDateTime>
 #include <QDBusConnection>
@@ -105,12 +107,18 @@ static QDataStream &operator>>(QDataStream &stream, UndoCommand &cmd)
 
 class KIO::UndoJob : public KIO::Job
 {
+    Q_OBJECT
 public:
     UndoJob(bool showProgressInfo) : KIO::Job()
     {
         if (showProgressInfo) {
             KIO::getJobTracker()->registerJob(this);
         }
+
+        d_ptr->m_privilegeExecutionEnabled = true;
+        d_ptr->m_operationType = d_ptr->Other;
+        d_ptr->m_caption = i18n("Undo Changes");
+        d_ptr->m_message = i18n("Undoing this operation requires root privileges. Do you want to continue?");
     }
     virtual ~UndoJob() {}
 
@@ -152,7 +160,6 @@ CommandRecorder::CommandRecorder(FileUndoManager::CommandType op, const QList<QU
     m_cmd.m_dst = dst;
     connect(job, SIGNAL(result(KJob*)),
             this, SLOT(slotResult(KJob*)));
-
     if (qobject_cast<KIO::CopyJob *>(job)) {
         connect(job, SIGNAL(copyingDone(KIO::Job*,QUrl,QUrl,QDateTime,bool,bool)),
                 this, SLOT(slotCopyingDone(KIO::Job*,QUrl,QUrl,QDateTime,bool,bool)));
@@ -161,6 +168,9 @@ CommandRecorder::CommandRecorder(FileUndoManager::CommandType op, const QList<QU
     } else if (KIO::MkpathJob *mkpathJob = qobject_cast<KIO::MkpathJob *>(job)) {
         connect(mkpathJob, &KIO::MkpathJob::directoryCreated,
                 this, &CommandRecorder::slotDirectoryCreated);
+    } else if (KIO::BatchRenameJob *batchRenameJob = qobject_cast<KIO::BatchRenameJob *>(job)) {
+        connect(batchRenameJob, &KIO::BatchRenameJob::fileRenamed,
+                this, &CommandRecorder::slotBatchRenamingDone);
     }
 }
 
@@ -215,6 +225,18 @@ void CommandRecorder::slotDirectoryCreated(const QUrl &dir)
     m_cmd.m_opStack.prepend(op);
 }
 
+void CommandRecorder::slotBatchRenamingDone(const QUrl &from, const QUrl &to)
+{
+    BasicOperation op;
+    op.m_valid = true;
+    op.m_type = BasicOperation::Directory;
+    op.m_renamed = true;
+    op.m_src = from;
+    op.m_dst = to;
+    op.m_mtime = QDateTime();
+    m_cmd.m_opStack.prepend(op);
+}
+
 ////
 
 class KIO::FileUndoManagerSingleton
@@ -235,7 +257,7 @@ FileUndoManager *FileUndoManager::self()
 // order of the undo items.
 FileUndoManagerPrivate::FileUndoManagerPrivate(FileUndoManager *qq)
     : m_uiInterface(new FileUndoManager::UiInterface()),
-      m_undoJob(0), m_nextCommandIndex(1000), q(qq)
+      m_undoJob(nullptr), m_nextCommandIndex(1000), q(qq)
 {
     (void) new KIOFileUndoManagerAdaptor(this);
     const QString dbusPath = QStringLiteral("/FileUndoManager");
@@ -253,7 +275,7 @@ FileUndoManager::FileUndoManager()
 {
     d = new FileUndoManagerPrivate(this);
     d->m_lock = false;
-    d->m_currentJob = 0;
+    d->m_currentJob = nullptr;
 }
 
 FileUndoManager::~FileUndoManager()
@@ -321,6 +343,8 @@ QString FileUndoManager::undoText() const
         return i18n("Und&o: Create Folder(s)");
     case FileUndoManager::Put:
         return i18n("Und&o: Create File");
+    case FileUndoManager::BatchRename:
+        return i18n("Und&o: Batch Rename");
     }
     /* NOTREACHED */
     return QString();
@@ -428,13 +452,13 @@ void FileUndoManagerPrivate::stopUndo(bool step)
     m_dirCleanupStack.clear();
     m_fileCleanupStack.clear();
     m_undoState = REMOVINGDIRS;
-    m_undoJob = 0;
+    m_undoJob = nullptr;
 
     if (m_currentJob) {
         m_currentJob->kill();
     }
 
-    m_currentJob = 0;
+    m_currentJob = nullptr;
 
     if (step) {
         undoStep();
@@ -443,7 +467,7 @@ void FileUndoManagerPrivate::stopUndo(bool step)
 
 void FileUndoManagerPrivate::slotResult(KJob *job)
 {
-    m_currentJob = 0;
+    m_currentJob = nullptr;
     if (job->error()) {
         m_uiInterface->jobError(static_cast<KIO::Job *>(job));
         delete m_undoJob;
@@ -475,7 +499,7 @@ void FileUndoManagerPrivate::addDirToUpdate(const QUrl &url)
 
 void FileUndoManagerPrivate::undoStep()
 {
-    m_currentJob = 0;
+    m_currentJob = nullptr;
 
     if (m_undoState == MAKINGDIRS) {
         stepMakingDirectories();
@@ -508,6 +532,7 @@ void FileUndoManagerPrivate::stepMakingDirectories()
         QUrl dir = m_dirStack.pop();
         //qDebug() << "creatingDir" << dir;
         m_currentJob = KIO::mkdir(dir);
+        m_currentJob->setParentJob(m_undoJob);
         m_undoJob->emitCreatingDir(dir);
     } else {
         m_undoState = MOVINGFILES;
@@ -555,6 +580,10 @@ void FileUndoManagerPrivate::stepMovingFiles()
             m_undoJob->emitMoving(op.m_dst, op.m_src);
         }
 
+        if (m_currentJob) {
+            m_currentJob->setParentJob(m_undoJob);
+        }
+
         m_current.m_opStack.removeLast();
         // The above KIO jobs are lowlevel, they don't trigger KDirNotify notification
         // So we need to do it ourselves (but schedule it to the end of the undo, to compress them)
@@ -575,6 +604,7 @@ void FileUndoManagerPrivate::stepRemovingLinks()
         QUrl file = m_fileCleanupStack.pop();
         //qDebug() << "file_delete" << file;
         m_currentJob = KIO::file_delete(file, KIO::HideProgressInfo);
+        m_currentJob->setParentJob(m_undoJob);
         m_undoJob->emitDeleting(file);
 
         QUrl url = file.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash);
@@ -594,15 +624,16 @@ void FileUndoManagerPrivate::stepRemovingDirectories()
         QUrl dir = m_dirCleanupStack.pop();
         //qDebug() << "rmdir" << dir;
         m_currentJob = KIO::rmdir(dir);
+        m_currentJob->setParentJob(m_undoJob);
         m_undoJob->emitDeleting(dir);
         addDirToUpdate(dir);
     } else {
         m_current.m_valid = false;
-        m_currentJob = 0;
+        m_currentJob = nullptr;
         if (m_undoJob) {
             //qDebug() << "deleting undojob";
             m_undoJob->emitResult();
-            m_undoJob = 0;
+            m_undoJob = nullptr;
         }
         QList<QUrl>::ConstIterator it = m_dirsToUpdate.constBegin();
         for (; it != m_dirsToUpdate.constEnd(); ++it) {
@@ -672,11 +703,11 @@ FileUndoManager::UiInterface *FileUndoManager::uiInterface() const
 
 ////
 
-class FileUndoManager::UiInterface::UiInterfacePrivate
+class Q_DECL_HIDDEN FileUndoManager::UiInterface::UiInterfacePrivate
 {
 public:
     UiInterfacePrivate()
-        : m_parentWidget(0), m_showProgressInfo(true)
+        : m_parentWidget(nullptr), m_showProgressInfo(true)
     {}
     QWidget *m_parentWidget;
     bool m_showProgressInfo;
@@ -694,7 +725,7 @@ FileUndoManager::UiInterface::~UiInterface()
 
 void FileUndoManager::UiInterface::jobError(KIO::Job *job)
 {
-    job->ui()->showErrorMessage();
+    job->uiDelegate()->showErrorMessage();
 }
 
 bool FileUndoManager::UiInterface::copiedFileWasModified(const QUrl &src, const QUrl &dest, const QDateTime &srcTime, const QDateTime &destTime)
@@ -748,3 +779,4 @@ void FileUndoManager::UiInterface::virtual_hook(int, void *)
 
 #include "moc_fileundomanager_p.cpp"
 #include "moc_fileundomanager.cpp"
+#include "fileundomanager.moc"
