@@ -22,6 +22,7 @@
 #define OPENWITHDIALOG_P_H
 
 #include <QtCore/QAbstractItemModel>
+#include <QSortFilterProxyModel>
 #include <QTreeView>
 
 class KApplicationModelPrivate;
@@ -34,7 +35,7 @@ class KApplicationModel : public QAbstractItemModel
     Q_OBJECT
 
 public:
-    KApplicationModel(QObject *parent = 0);
+    KApplicationModel(QObject *parent = nullptr);
     virtual ~KApplicationModel();
     bool canFetchMore(const QModelIndex &parent) const Q_DECL_OVERRIDE;
     int columnCount(const QModelIndex &parent = QModelIndex()) const Q_DECL_OVERRIDE;
@@ -50,6 +51,7 @@ public:
     QString entryPathFor(const QModelIndex &index) const;
     QString execFor(const QModelIndex &index) const;
     bool isDirectory(const QModelIndex &index) const;
+    void fetchAll(const QModelIndex &parent);
 
 private:
     friend class KApplicationModelPrivate;
@@ -58,6 +60,18 @@ private:
     Q_DISABLE_COPY(KApplicationModel)
 };
 
+/**
+ * @internal
+ */
+class QTreeViewProxyFilter : public QSortFilterProxyModel
+{
+     Q_OBJECT
+
+public:
+    QTreeViewProxyFilter(QObject *parent = nullptr);
+    bool filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const Q_DECL_OVERRIDE;
+};
+
 class KApplicationViewPrivate;
 
 /**
@@ -68,10 +82,11 @@ class KApplicationView : public QTreeView
     Q_OBJECT
 
 public:
-    KApplicationView(QWidget *parent = 0);
+    KApplicationView(QWidget *parent = nullptr);
     ~KApplicationView();
 
-    void setModel(QAbstractItemModel *model) Q_DECL_OVERRIDE;
+    void setModels(KApplicationModel *model, QSortFilterProxyModel *proxyModel);
+    QSortFilterProxyModel* proxyModel();
 
     bool isDirSel() const;
 
