@@ -139,7 +139,8 @@ bool KRun::isExecutableFile(const QUrl &url, const QString &mimetype)
 #ifdef Q_OS_WIN
                 mimeType.inherits(QLatin1String("application/x-ms-dos-executable")) ||
 #endif
-                mimeType.inherits(QStringLiteral("application/x-executable-script"))
+                mimeType.inherits(QStringLiteral("application/x-executable-script")) ||
+                mimeType.inherits(QStringLiteral("application/x-sharedlib"))
            ) {
             return true;
         }
@@ -350,8 +351,11 @@ static qint64 runCommandInternal(KProcess *proc, const KService *service, const
                 data.setSilent(KStartupInfoData::Yes);
             }
             data.setDesktop(KWindowSystem::currentDesktop());
-            if (window) {
-                data.setLaunchedBy(window->winId());
+            // QTBUG-59017 Calling winId() on an embedded widget will break interaction
+            // with it on high-dpi multi-screen setups (cf. also Bug 363548), hence using
+            // its parent window instead
+            if (window && window->window()) {
+                data.setLaunchedBy(window->window()->winId());
             }
             if (service && !service->entryPath().isEmpty()) {
                 data.setApplicationId(service->entryPath());
@@ -816,7 +820,12 @@ qint64 KRun::runService(const KService &_service, const QList<QUrl> &_urls, QWid
             KStartupInfoId id;
             id.initId(myasn);
             KStartupInfoData data;
-            data.setLaunchedBy(window->winId());
+            // QTBUG-59017 Calling winId() on an embedded widget will break interaction
+            // with it on high-dpi multi-screen setups (cf. also Bug 363548), hence using
+            // its parent window instead
+            if (window->window()) {
+                data.setLaunchedBy(window->window()->winId());
+            }
             KStartupInfo::sendChange(id, data);
         }
     }
@@ -1076,6 +1085,9 @@ void KRun::KRunPrivate::showPrompt()
 
 bool KRun::KRunPrivate::isPromptNeeded()
 {
+    if (m_strURL == QUrl("remote:/x-wizard_service.desktop")) {
+        return false;
+    }
     const QMimeDatabase db;
     const QMimeType mime = db.mimeTypeForUrl(m_strURL);
 
@@ -1438,6 +1450,8 @@ bool KRun::isExecutable(const QString &serviceType)
 {
     return (serviceType == QLatin1String("application/x-desktop") ||
             serviceType == QLatin1String("application/x-executable") ||
+            /* See https://bugs.freedesktop.org/show_bug.cgi?id=97226 */
+            serviceType == QLatin1String("application/x-sharedlib") ||
             serviceType == QLatin1String("application/x-ms-dos-executable") ||
             serviceType == QLatin1String("application/x-shellscript"));
 }
@@ -1591,9 +1605,10 @@ void
 KProcessRunner::slotProcessExited(int exitCode, QProcess::ExitStatus exitStatus)
 {
     //qDebug() << m_executable << "exitCode=" << exitCode << "exitStatus=" << exitStatus;
-    Q_UNUSED(exitStatus);
+    Q_UNUSED(exitStatus)
 
     terminateStartupNotification(); // do this before the messagebox
+
     if (exitCode != 0 && !m_executable.isEmpty()) {
         // Let's see if the error is because the exe doesn't exist.
         // When this happens, waitForStarted returns false, but not if kioexec
@@ -1602,12 +1617,21 @@ KProcessRunner::slotProcessExited(int exitCode, QProcess::ExitStatus exitStatus)
         // We'll try to find the executable relatively to current directory,
         // (or with a full path, if m_executable is absolute), and then in the PATH.
         if (!QFile(m_executable).exists() && QStandardPaths::findExecutable(m_executable).isEmpty()) {
-            QEventLoopLocker locker;
-            KMessageBox::sorry(nullptr, i18n("Could not find the program '%1'", m_executable));
+            const QString &error = i18n("Could not find the program '%1'", m_executable);
+
+            if (qApp) {
+                QTimer::singleShot(0, qApp, [=]() {
+                        QEventLoopLocker locker;
+                        KMessageBox::sorry(nullptr, error);
+                    });
+            } else {
+                qWarning() << error;
+            }
         } else {
             //qDebug() << process->readAllStandardError();
         }
     }
+
     deleteLater();
 }
 
