@@ -23,9 +23,11 @@
 #include "hostinfo.h"
 
 #include <kiconloader.h>
-#include <kservicetypetrader.h>
+#include <kservice.h>
 #include <kio/global.h>
 
+#include <KPluginMetaData>
+
 #include <QtCore/QHashIterator>
 #include <QtCore/QStringBuilder>
 #include <QtNetwork/QHostInfo>
@@ -80,7 +82,7 @@ static QString lookupIconNameFor(const QUrl &url, KUriFilterData::UriTypes type)
     return iconName;
 }
 
-class KUriFilterSearchProvider::KUriFilterSearchProviderPrivate
+class Q_DECL_HIDDEN KUriFilterSearchProvider::KUriFilterSearchProviderPrivate
 {
 public:
     KUriFilterSearchProviderPrivate() {}
@@ -184,7 +186,6 @@ public:
 
     ~KUriFilterDataPrivate()
     {
-        qDeleteAll(searchProviderMap.begin(), searchProviderMap.end());
     }
 
     void setData(const QUrl &u, const QString &typedUrl)
@@ -479,6 +480,11 @@ KUriFilterPlugin::KUriFilterPlugin(const QString &name, QObject *parent)
     setObjectName(name);
 }
 
+// KF6 TODO
+//KUriFilterPlugin::~KUriFilterPlugin()
+//{
+//}
+
 KCModule *KUriFilterPlugin::configModule(QWidget *, const char *) const
 {
     return nullptr;
@@ -549,13 +555,10 @@ public:
     KUriFilterPrivate() {}
     ~KUriFilterPrivate()
     {
-        qDeleteAll(plugins);
-        plugins.clear();
+        qDeleteAll(pluginList);
+        pluginList.clear();
     }
-    QHash<QString, KUriFilterPlugin *> plugins;
-    // NOTE: DO NOT REMOVE this variable! Read the
-    // comments in KUriFilter::loadPlugins to understand why...
-    QStringList pluginNames;
+    QVector<KUriFilterPlugin *> pluginList;
 };
 
 class KUriFilterSingleton
@@ -586,21 +589,11 @@ bool KUriFilter::filterUri(KUriFilterData &data, const QStringList &filters)
 {
     bool filtered = false;
 
-    // If no specific filters were requested, iterate through all the plugins.
-    // Otherwise, only use available filters.
-    if (filters.isEmpty()) {
-        QStringListIterator it(d->pluginNames);
-        while (it.hasNext()) {
-            KUriFilterPlugin *plugin = d->plugins.value(it.next());
-            if (plugin &&  plugin->filterUri(data)) {
-                filtered = true;
-            }
-        }
-    } else {
-        QStringListIterator it(filters);
-        while (it.hasNext()) {
-            KUriFilterPlugin *plugin = d->plugins.value(it.next());
-            if (plugin &&  plugin->filterUri(data)) {
+    for (KUriFilterPlugin *plugin : d->pluginList) {
+        // If no specific filters were requested, iterate through all the plugins.
+        // Otherwise, only use available filters.
+        if (filters.isEmpty() || filters.contains(plugin->objectName())) {
+            if (plugin->filterUri(data)) {
                 filtered = true;
             }
         }
@@ -667,26 +660,34 @@ bool KUriFilter::filterSearchUri(KUriFilterData &data, SearchFilterTypes types)
 
 QStringList KUriFilter::pluginNames() const
 {
-    return d->pluginNames;
+    QStringList res;
+    res.reserve(d->pluginList.size());
+    std::transform(d->pluginList.constBegin(), d->pluginList.constEnd(), std::back_inserter(res), [](const KUriFilterPlugin *plugin) { return plugin->objectName(); });
+    return res;
 }
 
 void KUriFilter::loadPlugins()
 {
-    const KService::List offers = KServiceTypeTrader::self()->query(QStringLiteral("KUriFilter/Plugin"));
+    QVector<KPluginMetaData> plugins = KPluginLoader::findPlugins("kf5/urifilters");
+    // Sort the plugins by order of priority
+    std::sort(plugins.begin(), plugins.end(), [](const KPluginMetaData &a, const KPluginMetaData &b) {
+            return a.rawData().value("X-KDE-InitialPreference").toInt() > b.rawData().value("X-KDE-InitialPreference").toInt();
+    });
 
-    // NOTE: Plugin priority is determined by the InitialPreference entry in
-    // the .desktop files, so the trader result is already sorted and should
-    // not be manually sorted.
-    Q_FOREACH (const KService::Ptr &ptr, offers) {
-        KUriFilterPlugin *plugin = ptr->createInstance<KUriFilterPlugin>();
-        if (plugin) {
-            const QString &pluginName = plugin->objectName();
-            Q_ASSERT(!pluginName.isEmpty());
-            d->plugins.insert(pluginName, plugin);
-            // Needed to ensure the order of filtering is honored since
-            // items are ordered arbitrarily in a QHash and QMap always
-            // sorts by keys. Both undesired behavior.
-            d->pluginNames << pluginName;
+    QStringList pluginNames;
+    pluginNames.reserve(plugins.count());
+
+    for (const KPluginMetaData &pluginMetaData : plugins) {
+        const QString fileName = pluginMetaData.fileName().section(QLatin1Char('/'), -1);
+        if (!pluginNames.contains(fileName)) {
+            pluginNames << fileName;
+            KPluginFactory *factory = qobject_cast<KPluginFactory *>(pluginMetaData.instantiate());
+            if (factory) {
+                KUriFilterPlugin *plugin = factory->create<KUriFilterPlugin>(nullptr);
+                if (plugin) {
+                    d->pluginList << plugin;
+                }
+            }
         }
     }
 }
