@@ -20,6 +20,8 @@
 */
 
 #include "kio/renamedialog.h"
+#include "kio_widgets_debug.h"
+#include "../pathhelpers_p.h"
 
 #include <QApplication>
 #include <QCheckBox>
@@ -91,22 +93,22 @@ static KSqueezedTextLabel *createSqueezedLabel(QWidget *parent, const QString &t
 }
 
 /** @internal */
-class RenameDialog::RenameDialogPrivate
+class Q_DECL_HIDDEN RenameDialog::RenameDialogPrivate
 {
 public:
     RenameDialogPrivate()
     {
-        bCancel = 0;
-        bRename = bSkip = bOverwrite = 0;
-        bResume = bSuggestNewName = 0;
-        bApplyAll = 0;
-        m_pLineEdit = 0;
+        bCancel = nullptr;
+        bRename = bSkip = bOverwrite = nullptr;
+        bResume = bSuggestNewName = nullptr;
+        bApplyAll = nullptr;
+        m_pLineEdit = nullptr;
         m_srcPendingPreview = false;
         m_destPendingPreview = false;
-        m_srcPreview = 0;
-        m_destPreview = 0;
-        m_srcArea = Q_NULLPTR;
-        m_destArea = Q_NULLPTR;
+        m_srcPreview = nullptr;
+        m_destPreview = nullptr;
+        m_srcArea = nullptr;
+        m_destArea = nullptr;
     }
 
     void setRenameBoxText(const QString &fileName)
@@ -208,6 +210,7 @@ RenameDialog::RenameDialog(QWidget *parent, const QString &_caption,
         QLabel *lb = new QLabel(i18n("This action would overwrite '%1' with itself.\n"
                                      "Please enter a new file name:",
                                      KStringHandler::csqueeze(d->src.toDisplayString(QUrl::PreferLocalFile), 100)), this);
+        lb->setTextFormat(Qt::PlainText);
 
         d->bRename->setText(i18n("C&ontinue"));
         pLayout->addWidget(lb);
@@ -280,8 +283,10 @@ RenameDialog::RenameDialog(QWidget *parent, const QString &_caption,
         gridLayout->addWidget(destTitle, gridRow, 1);
 
         QLabel *srcUrlLabel = createSqueezedLabel(parent, d->src.toDisplayString(QUrl::PreferLocalFile));
+        srcUrlLabel->setTextFormat(Qt::PlainText);
         gridLayout->addWidget(srcUrlLabel, ++gridRow, 0);
         QLabel *destUrlLabel = createSqueezedLabel(parent, d->dest.toDisplayString(QUrl::PreferLocalFile));
+        destUrlLabel->setTextFormat(Qt::PlainText);
         gridLayout->addWidget(destUrlLabel, gridRow, 1);
 
         // The labels containing previews or icons
@@ -312,6 +317,7 @@ RenameDialog::RenameDialog(QWidget *parent, const QString &_caption,
         }
 
         QLabel *lb = new KSqueezedTextLabel(sentence1, this);
+        lb->setTextFormat(Qt::PlainText);
         pLayout->addWidget(lb);
     }
 
@@ -429,7 +435,7 @@ QUrl RenameDialog::autoDestUrl() const
     const QUrl destDirectory = d->dest.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash);
     const QString newName = KIO::suggestName(destDirectory, d->dest.fileName());
     QUrl newDest(destDirectory);
-    newDest.setPath(newDest.path() + QLatin1Char('/') + newName);
+    newDest.setPath(concatPaths(newDest.path(), newName));
     return newDest;
 }
 
@@ -451,7 +457,7 @@ void RenameDialog::renamePressed()
         const QUrl u = newDestUrl();
         if (!u.isValid()) {
             KMessageBox::error(this, i18n("Malformed URL\n%1", u.errorString()));
-            qWarning() << u.errorString();
+            qCWarning(KIO_WIDGETS) << u.errorString();
             return;
         }
 
@@ -588,10 +594,10 @@ void RenameDialog::showDestPreview(const KFileItem &fileitem, const QPixmap &pix
 
 void RenameDialog::resizePanels()
 {
-    Q_ASSERT(d->m_srcArea != Q_NULLPTR);
-    Q_ASSERT(d->m_destArea != Q_NULLPTR);
-    Q_ASSERT(d->m_srcPreview != Q_NULLPTR);
-    Q_ASSERT(d->m_destPreview != Q_NULLPTR);
+    Q_ASSERT(d->m_srcArea != nullptr);
+    Q_ASSERT(d->m_destArea != nullptr);
+    Q_ASSERT(d->m_srcPreview != nullptr);
+    Q_ASSERT(d->m_destPreview != nullptr);
 
     // using QDesktopWidget geometry as Kephal isn't accessible here in kdelibs
     const QSize screenSize = QApplication::desktop()->availableGeometry(this).size();
