@@ -40,6 +40,8 @@ namespace KIO
 {
 
 /**
+ * @class KIO::RenameDialog renamedialog.h <KIO/RenameDialog>
+ *
  * The dialog shown when a CopyJob realizes that a destination file already exists,
  * and wants to offer the user with the choice to either Rename, Overwrite, Skip;
  * this dialog is also used when a .part file exists and the user can choose to
@@ -122,8 +124,6 @@ private Q_SLOTS:
 
 private:
     QScrollArea *createContainerLayout(QWidget *parent, const KFileItem &item, QLabel *preview);
-    QLabel *createLabel(QWidget *parent, const QString &text, const bool containerTitle);
-    KSqueezedTextLabel *createSqueezedLabel(QWidget *parent, const QString &text);
     class RenameDialogPrivate;
     RenameDialogPrivate *const d;
 };
